@@ -8,5 +8,7 @@ const char* RemoteControlExtension::kFilterString =
     "/iq/xrme[@xmlns='http://purplehatstands.com/xmlns/xrme/remotecontrol']";
 const char* MediaStorageExtension::kFilterString =
     "/iq/xrme[@xmlns='http://purplehatstands.com/xmlns/xrme/mediastorage']";
+const char* SIPNegotiation::kFilterString =
+    "/iq/xrme[@xmlns='http://purplehatstands.com/xmlns/xrme/sipnegotiation']";
 
 }
