@@ -38,7 +38,10 @@ CREATE TABLE jamendo.songs (
   score NOT NULL DEFAULT 0,
   beginning NOT NULL DEFAULT 0,
 
-  cue_path TEXT
+  cue_path TEXT,
+  unavailable INTEGER DEFAULT 0,
+
+  effective_albumartist TEXT
 );
 
 CREATE VIRTUAL TABLE jamendo.songs_fts USING fts3(
