@@ -19,8 +19,10 @@
 #include "baranalyzer.h"
 #include "blockanalyzer.h"
 #include "boomanalyzer.h"
+#include "nyancatanalyzer.h"
 #include "sonogram.h"
 #include "turbine.h"
+#include "core/logging.h"
 
 #include <QMouseEvent>
 #include <QHBoxLayout>
@@ -71,6 +73,8 @@ AnalyzerContainer::AnalyzerContainer(QWidget *parent)
   AddAnalyzerType<BoomAnalyzer>();
   AddAnalyzerType<Sonogram>();
   AddAnalyzerType<TurbineAnalyzer>();
+  AddAnalyzerType<NyanCatAnalyzer>();
+
   connect(mapper_, SIGNAL(mapped(int)), SLOT(ChangeAnalyzer(int)));
   disable_action_ =
       context_menu_->addAction(tr("No analyzer"), this, SLOT(DisableAnalyzer()));
@@ -136,7 +140,7 @@ void AnalyzerContainer::ChangeAnalyzer(int id) {
   QObject* instance = analyzer_types_[id]->newInstance(Q_ARG(QWidget*, this));
 
   if (!instance) {
-    qWarning() << "Couldn't intialise a new" << analyzer_types_[id]->className();
+    qLog(Warning) << "Couldn't intialise a new" << analyzer_types_[id]->className();
     return;
   }
 
