@@ -19,6 +19,7 @@
 #include "database.h"
 #include "scopedtransaction.h"
 #include "utilities.h"
+#include "core/logging.h"
 
 #include <QCoreApplication>
 #include <QDir>
@@ -28,10 +29,11 @@
 #include <QSqlQuery>
 #include <QtDebug>
 #include <QThread>
+#include <QUrl>
 #include <QVariant>
 
 const char* Database::kDatabaseFilename = "clementine.db";
-const int Database::kSchemaVersion = 28;
+const int Database::kSchemaVersion = 36;
 const char* Database::kMagicAllSongsTables = "%allsongstables";
 
 int Database::sNextConnectionId = 1;
@@ -237,7 +239,7 @@ void Database::StaticInit() {
 
   QLibrary library(plugin_path);
   if (!library.load()) {
-    qDebug() << "QLibrary::load() failed for " << plugin_path;
+    qLog(Error) << "QLibrary::load() failed for " << plugin_path;
     return;
   }
 
@@ -260,7 +262,7 @@ void Database::StaticInit() {
       !_sqlite3_value_text ||
       !_sqlite3_result_int64 ||
       !_sqlite3_user_data) {
-    qDebug() << "Couldn't resolve sqlite symbols";
+    qLog(Error) << "Couldn't resolve sqlite symbols";
     sLoadedSqliteSymbols = false;
   } else {
     sLoadedSqliteSymbols = true;
@@ -384,7 +386,7 @@ QSqlDatabase Database::Connect() {
   set_fts_tokenizer.bindValue(":pointer", QByteArray(
       reinterpret_cast<const char*>(&sFTSTokenizer), sizeof(&sFTSTokenizer)));
   if (!set_fts_tokenizer.exec()) {
-    qWarning() << "Couldn't register FTS3 tokenizer";
+    qLog(Warning) << "Couldn't register FTS3 tokenizer";
   }
 
   // We want Unicode aware LIKE clauses and FTS if possible.
@@ -438,7 +440,7 @@ QSqlDatabase Database::Connect() {
                         " WHERE type='table'").arg(key), db);
     if (!q.exec() || !q.next()) {
       ScopedTransaction t(&db);
-      ExecFromFile(attached_databases_[key].schema_, db);
+      ExecFromFile(attached_databases_[key].schema_, db, 0);
       t.Commit();
     }
   }
@@ -456,7 +458,7 @@ void Database::UpdateMainSchema(QSqlDatabase* db) {
   startup_schema_version_ = schema_version;
 
   if (schema_version > kSchemaVersion) {
-    qWarning() << "The database schema (version" << schema_version << ") is newer than I was expecting";
+    qLog(Warning) << "The database schema (version" << schema_version << ") is newer than I was expecting";
     return;
   }
   if (schema_version < kSchemaVersion) {
@@ -469,7 +471,7 @@ void Database::UpdateMainSchema(QSqlDatabase* db) {
 
 void Database::RecreateAttachedDb(const QString& database_name) {
   if (!attached_databases_.contains(database_name)) {
-    qWarning() << "Attached database does not exist:" << database_name;
+    qLog(Warning) << "Attached database does not exist:" << database_name;
     return;
   }
 
@@ -482,12 +484,12 @@ void Database::RecreateAttachedDb(const QString& database_name) {
     QSqlQuery q("DETACH DATABASE :alias", db);
     q.bindValue(":alias", database_name);
     if (!q.exec()) {
-      qWarning() << "Failed to detach database" << database_name;
+      qLog(Warning) << "Failed to detach database" << database_name;
       return;
     }
 
     if (!QFile::remove(filename)) {
-      qWarning() << "Failed to remove file" << filename;
+      qLog(Warning) << "Failed to remove file" << filename;
     }
   }
 
@@ -505,27 +507,67 @@ void Database::UpdateDatabaseSchema(int version, QSqlDatabase &db) {
     filename = ":/schema/schema.sql";
   else
     filename = QString(":/schema/schema-%1.sql").arg(version);
-
+  
   ScopedTransaction t(&db);
-  ExecFromFile(filename, db);
+  
+  if (version == 31) {
+    // This version used to do a bad job of converting filenames in the songs
+    // table to file:// URLs.  Now we do it properly here instead.
+    
+    UrlEncodeFilenameColumn("songs", db);
+    UrlEncodeFilenameColumn("playlist_items", db);
+
+    foreach (const QString& table, db.tables()) {
+      if (table.startsWith("device_") && table.endsWith("_songs")) {
+        UrlEncodeFilenameColumn(table, db);
+      }
+    }
+  }
+  
+  ExecFromFile(filename, db, version - 1);
   t.Commit();
 }
 
-void Database::ExecFromFile(const QString &filename, QSqlDatabase &db) {
+void Database::UrlEncodeFilenameColumn(const QString& table, QSqlDatabase& db) {
+  QSqlQuery select(QString("SELECT ROWID, filename FROM %1").arg(table), db);
+  QSqlQuery update(QString("UPDATE %1 SET filename=:filename WHERE ROWID=:id").arg(table), db);
+  
+  select.exec();
+  if (CheckErrors(select)) return;
+  while (select.next()) {
+    const int rowid = select.value(0).toInt();
+    const QString filename = select.value(1).toString();
+    
+    if (filename.isEmpty() || filename.contains("://")) {
+      continue;
+    }
+    
+    const QUrl url = QUrl::fromLocalFile(filename);
+    
+    update.bindValue(":filename", url.toEncoded());
+    update.bindValue(":id", rowid);
+    update.exec();
+    CheckErrors(update);
+  }
+}
+
+void Database::ExecFromFile(const QString &filename, QSqlDatabase &db,
+                            int schema_version) {
   // Open and read the database schema
   QFile schema_file(filename);
   if (!schema_file.open(QIODevice::ReadOnly))
     qFatal("Couldn't open schema file %s", filename.toUtf8().constData());
-  ExecCommands(QString::fromUtf8(schema_file.readAll()), db);
+  ExecCommands(QString::fromUtf8(schema_file.readAll()), db, schema_version);
 }
 
-void Database::ExecCommands(const QString &schema, QSqlDatabase &db) {
+void Database::ExecCommands(const QString& schema, QSqlDatabase& db,
+                            int schema_version) {
   // Run each command
   QStringList commands(schema.split(";\n\n"));
 
   // We don't want this list to reflect possible DB schema changes
   // so we initialize it before executing any statements.
-  QStringList tables = SongsTables(db);
+  QStringList tables = SongsTables(db, schema_version);
 
   foreach (const QString& command, commands) {
     // There are now lots of "songs" tables that need to have the same schema:
@@ -533,7 +575,7 @@ void Database::ExecCommands(const QString &schema, QSqlDatabase &db) {
     // in the schema files to update all songs tables at once.
     if (command.contains(kMagicAllSongsTables)) {
       foreach (const QString& table, tables) {
-        qDebug() << "Updating" << table << "for" << kMagicAllSongsTables;
+        qLog(Info) << "Updating" << table << "for" << kMagicAllSongsTables;
         QString new_command(command);
         new_command.replace(kMagicAllSongsTables, table);
         QSqlQuery query(db.exec(new_command));
@@ -548,7 +590,7 @@ void Database::ExecCommands(const QString &schema, QSqlDatabase &db) {
   }
 }
 
-QStringList Database::SongsTables(QSqlDatabase& db) const {
+QStringList Database::SongsTables(QSqlDatabase& db, int schema_version) const {
   QStringList ret;
 
   // look for the tables in the main db
@@ -569,15 +611,20 @@ QStringList Database::SongsTables(QSqlDatabase& db) const {
     }
   }
 
+  if (schema_version > 29) {
+    // The playlist_items table became a songs table in version 29.
+    ret << "playlist_items";
+  }
+
   return ret;
 }
 
 bool Database::CheckErrors(const QSqlQuery& query) {
   QSqlError last_error = query.lastError();
   if (last_error.isValid()) {
-    qDebug() << "db error: " << last_error;
-    qDebug() << "faulty query: " << query.lastQuery();
-    qDebug() << "bound values: " << query.boundValues();
+    qLog(Error) << "db error: " << last_error;
+    qLog(Error) << "faulty query: " << query.lastQuery();
+    qLog(Error) << "bound values: " << query.boundValues();
 
     emit Error("LibraryBackend: " + last_error.text());
     return true;
