@@ -19,11 +19,13 @@
 #include "mpris_common.h"
 #include "mpris1.h"
 #include "mpris2.h"
-#include "core/artloader.h"
+#include "core/logging.h"
 #include "core/mpris2_player.h"
 #include "core/mpris2_root.h"
 #include "core/mpris2_tracklist.h"
 #include "core/player.h"
+#include "core/timeconstants.h"
+#include "covers/artloader.h"
 #include "engines/enginebase.h"
 #include "playlist/playlist.h"
 #include "playlist/playlistmanager.h"
@@ -55,7 +57,7 @@ Mpris2::Mpris2(PlayerInterface* player, ArtLoader* art_loader,
   new Mpris2Player(this);
 
   if (!QDBusConnection::sessionBus().registerService(kServiceName)) {
-    qWarning() << "Failed to register" << QString(kServiceName) << "on the session bus";
+    qLog(Warning) << "Failed to register" << QString(kServiceName) << "on the session bus";
     return;
   }
 
@@ -150,9 +152,7 @@ bool Mpris2::HasTrackList() const {
 }
 
 QString Mpris2::Identity() const {
-  return QString("%1 %2").arg(
-      QCoreApplication::applicationName(),
-      QCoreApplication::applicationVersion());
+  return QCoreApplication::applicationName();
 }
 
 QString Mpris2::DesktopEntryAbsolutePath() const {
@@ -304,25 +304,11 @@ void Mpris2::CurrentSongChanged(const Song& song) {
 // ... and we add the cover information later, when it's available.
 void Mpris2::ArtLoaded(const Song& song, const QString& art_uri) {
   last_metadata_ = QVariantMap();
+  song.ToXesam(&last_metadata_);
 
   using mpris::AddMetadata;
   AddMetadata("mpris:trackid", current_track_id(), &last_metadata_);
-  AddMetadata("xesam:url", song.filename(), &last_metadata_);
-  AddMetadata("xesam:title", song.PrettyTitle(), &last_metadata_);
-  AddMetadataAsList("xesam:artist", song.artist(), &last_metadata_);
-  AddMetadata("xesam:album", song.album(), &last_metadata_);
-  AddMetadataAsList("xesam:albumArtist", song.albumartist(), &last_metadata_);
-  AddMetadata("mpris:length", song.length_nanosec() / kNsecPerUsec, &last_metadata_);
-  AddMetadata("xesam:trackNumber", song.track(), &last_metadata_);
-  AddMetadataAsList("xesam:genre", song.genre(), &last_metadata_);
-  AddMetadata("xesam:discNumber", song.disc(), &last_metadata_);
-  AddMetadataAsList("xesam:comment", song.comment(), &last_metadata_);
-  AddMetadata("xesam:contentCreated", AsMPRISDateTimeType(song.ctime()), &last_metadata_);
-  AddMetadata("xesam:lastUsed", AsMPRISDateTimeType(song.lastplayed()), &last_metadata_);
-  AddMetadata("xesam:audioBPM", song.bpm(), &last_metadata_);
-  AddMetadataAsList("xesam:composer", song.composer(), &last_metadata_);
-  AddMetadata("xesam:useCount", song.playcount(), &last_metadata_);
-  AddMetadata("xesam:autoRating", song.score(), &last_metadata_);
+
   if (song.rating() != -1.0) {
     AddMetadata("rating", song.rating() * 5, &last_metadata_);
   }
