@@ -16,9 +16,11 @@
 */
 
 #include "organiseformat.h"
+#include "core/timeconstants.h"
 
 #include <QApplication>
 #include <QPalette>
+#include <QUrl>
 
 const char* OrganiseFormat::kTagPattern = "\\%([a-zA-Z]*)";
 const char* OrganiseFormat::kBlockPattern = "\\{([^{}]+)\\}";
@@ -128,7 +130,6 @@ QString OrganiseFormat::TagValue(const QString &tag, const Song &song) const {
   if (tag == "title")            value = song.title();
   else if (tag == "album")       value = song.album();
   else if (tag == "artist")      value = song.artist();
-  else if (tag == "albumartist") value = song.albumartist();
   else if (tag == "composer")    value = song.composer();
   else if (tag == "genre")       value = song.genre();
   else if (tag == "comment")     value = song.comment();
@@ -139,12 +140,14 @@ QString OrganiseFormat::TagValue(const QString &tag, const Song &song) const {
   else if (tag == "length")      value = QString::number(song.length_nanosec() / kNsecPerSec);
   else if (tag == "bitrate")     value = QString::number(song.bitrate());
   else if (tag == "samplerate")  value = QString::number(song.samplerate());
-  else if (tag == "extension")   value = song.filename().section('.', -1, -1);
+  else if (tag == "extension")   value = song.url().toLocalFile().section('.', -1, -1);
   else if (tag == "artistinitial") {
-    value = song.albumartist().trimmed();
-    if (value.isEmpty())  value = song.artist().trimmed();
+    value = song.effective_albumartist().trimmed();
     if (!value.isEmpty()) value = value[0].toUpper();
   }
+  else if (tag == "albumartist") {
+    value = song.is_compilation() ? "Various Artists" : song.effective_albumartist();
+  }
 
   if (replace_the_ && (tag == "artist" || tag == "albumartist"))
     value.replace(QRegExp("^the\\s+", Qt::CaseInsensitive), "");
