@@ -16,7 +16,10 @@
 */
 
 #include "fmpsparser.h"
+#include "logging.h"
+#include "mpris_common.h"
 #include "song.h"
+#include "timeconstants.h"
 
 #include <algorithm>
 
@@ -31,6 +34,7 @@
 #include <id3v1genres.h>
 #include <id3v2tag.h>
 #include <mp4file.h>
+#include <mp4tag.h>
 #include <mpcfile.h>
 #include <mpegfile.h>
 #include <oggfile.h>
@@ -44,49 +48,74 @@
 #include <wavfile.h>
 
 #ifdef HAVE_LIBLASTFM
-  #include "radio/fixlastfm.h"
+  #include "internet/fixlastfm.h"
   #include <lastfm/Track>
 #endif
 
 #include <QFile>
 #include <QFileInfo>
+#include <QLatin1Literal>
+#include <QSharedData>
 #include <QSqlQuery>
-#include <QtConcurrentRun>
 #include <QTextCodec>
 #include <QTime>
 #include <QVariant>
+#include <QtConcurrentRun>
 
 #ifdef Q_OS_WIN32
 # include <mswmdm.h>
 # include <QUuid>
 #endif // Q_OS_WIN32
 
+#ifdef HAVE_LIBGPOD
+# include <gpod/itdb.h>
+#endif
+
+#ifdef HAVE_LIBMTP
+# include <libmtp.h>
+#endif
+
 #include <boost/scoped_array.hpp>
 #include <boost/scoped_ptr.hpp>
 using boost::scoped_ptr;
 
-#include "albumcoverloader.h"
 #include "encoding.h"
 #include "utilities.h"
-
+#include "covers/albumcoverloader.h"
 #include "engines/enginebase.h"
 #include "library/sqlrow.h"
 #include "widgets/trackslider.h"
 
-static QStringList Prepend(const QString& text, const QStringList& list) {
+
+// Taglib added support for FLAC pictures in 1.7.0
+#if (TAGLIB_MAJOR_VERSION > 1) || (TAGLIB_MAJOR_VERSION == 1 && TAGLIB_MINOR_VERSION >= 7)
+# define TAGLIB_HAS_FLAC_PICTURELIST
+#endif
+
+
+namespace {
+
+QStringList Prepend(const QString& text, const QStringList& list) {
   QStringList ret(list);
   for (int i=0 ; i<ret.count() ; ++i)
     ret[i].prepend(text);
   return ret;
 }
 
-static QStringList Updateify(const QStringList& list) {
+QStringList Updateify(const QStringList& list) {
   QStringList ret(list);
   for (int i=0 ; i<ret.count() ; ++i)
     ret[i].prepend(ret[i] + " = :");
   return ret;
 }
 
+TagLib::String QStringToTaglibString(const QString& s) {
+  return TagLib::String(s.toUtf8().constData(), TagLib::String::UTF8);
+}
+
+} // namespace
+
+
 const QStringList Song::kColumns = QStringList()
     << "title" << "album" << "artist" << "albumartist" << "composer" << "track"
     << "disc" << "bpm" << "year" << "genre" << "comment" << "compilation"
@@ -95,7 +124,7 @@ const QStringList Song::kColumns = QStringList()
     << "art_manual" << "filetype" << "playcount" << "lastplayed" << "rating"
     << "forced_compilation_on" << "forced_compilation_off"
     << "effective_compilation" << "skipcount" << "score" << "beginning" << "length"
-    << "cue_path";
+    << "cue_path" << "unavailable" << "effective_albumartist";
 
 const QString Song::kColumnSpec = Song::kColumns.join(", ");
 const QString Song::kBindSpec = Prepend(":", Song::kColumns).join(", ");
@@ -113,38 +142,85 @@ const QString Song::kFtsUpdateSpec = Updateify(Song::kFtsColumns).join(", ");
 const QString Song::kManuallyUnsetCover = "(unset)";
 const QString Song::kEmbeddedCover = "(embedded)";
 
-QString Song::JoinSpec(const QString& table) {
-  return Prepend(table + ".", kColumns).join(", ");
-}
-
-QString Song::TextForFiletype(FileType type) {
-  switch (type) {
-    case Song::Type_Asf:       return QObject::tr("Windows Media audio");
-    case Song::Type_Flac:      return QObject::tr("Flac");
-    case Song::Type_Mp4:       return QObject::tr("MP4 AAC");
-    case Song::Type_Mpc:       return QObject::tr("MPC");
-    case Song::Type_Mpeg:      return QObject::tr("MP3"); // Not technically correct
-    case Song::Type_OggFlac:   return QObject::tr("Ogg Flac");
-    case Song::Type_OggSpeex:  return QObject::tr("Ogg Speex");
-    case Song::Type_OggVorbis: return QObject::tr("Ogg Vorbis");
-    case Song::Type_Aiff:      return QObject::tr("AIFF");
-    case Song::Type_Wav:       return QObject::tr("Wav");
-    case Song::Type_TrueAudio: return QObject::tr("TrueAudio");
-
-    case Song::Type_Stream:    return QObject::tr("Stream");
-
-    case Song::Type_Unknown:
-    default:
-      return QObject::tr("Unknown");
-  }
-}
-
-
-static TagLib::String QStringToTaglibString(const QString& s);
-
 TagLibFileRefFactory Song::kDefaultFactory;
+QMutex Song::sTaglibMutex;
 
-QMutex Song::taglib_mutex_;
+
+struct Song::Private : public QSharedData {
+  Private();
+
+  // This is here and not in Song itself so we don't have to include
+  // <xiphcomment.h> in the main header.
+  void ParseOggTag(const TagLib::Ogg::FieldListMap& map, const QTextCodec* codec,
+                   QString* disc, QString* compilation);
+
+  bool valid_;
+  int id_;
+
+  QString title_;
+  QString album_;
+  QString artist_;
+  QString albumartist_;
+  QString composer_;
+  int track_;
+  int disc_;
+  float bpm_;
+  int year_;
+  QString genre_;
+  QString comment_;
+  bool compilation_;            // From the file tag
+  bool sampler_;                // From the library scanner
+  bool forced_compilation_on_;  // Set by the user
+  bool forced_compilation_off_; // Set by the user
+
+  float rating_;
+  int playcount_;
+  int skipcount_;
+  int lastplayed_;
+  int score_;
+
+  // The beginning of the song in seconds. In case of single-part media
+  // streams, this will equal to 0. In case of multi-part streams on the
+  // other hand, this will mark the beginning of a section represented by
+  // this Song object. This is always greater than 0.
+  qint64 beginning_;
+  // The end of the song in seconds. In case of single-part media
+  // streams, this will equal to the song's length. In case of multi-part
+  // streams on the other hand, this will mark the end of a section
+  // represented by this Song object.
+  // This may be negative indicating that the length of this song is
+  // unknown.
+  qint64 end_;
+
+  int bitrate_;
+  int samplerate_;
+
+  int directory_id_;
+  QUrl url_;
+  QString basefilename_;
+  int mtime_;
+  int ctime_;
+  int filesize_;
+  FileType filetype_;
+
+  // If the song has a CUE, this contains it's path.
+  QString cue_path_;
+
+  // Filenames to album art for this song.
+  QString art_automatic_; // Guessed by LibraryWatcher
+  QString art_manual_;    // Set by the user - should take priority
+
+  QImage image_;
+
+  // Whether this song was loaded from a file using taglib.
+  bool init_from_file_;
+  // Whether our encoding guesser thinks these tags might be incorrectly encoded.
+  bool suspicious_tags_;
+
+  // Whether the song does not exist on the file system anymore, but is still
+  // stored in the database so as to remember the user's metadata.
+  bool unavailable_;
+};
 
 
 Song::Private::Private()
@@ -173,7 +249,8 @@ Song::Private::Private()
     filesize_(-1),
     filetype_(Type_Unknown),
     init_from_file_(false),
-    suspicious_tags_(false)
+    suspicious_tags_(false),
+    unavailable_(false)
 {
 }
 
@@ -202,6 +279,130 @@ Song::Song(FileRefFactory* factory)
       factory_(factory) {
 }
 
+Song::~Song() {
+}
+
+Song& Song::operator =(const Song& other) {
+  d = other.d;
+  factory_ = other.factory_;
+  return *this;
+}
+
+bool Song::is_valid() const { return d->valid_; }
+bool Song::is_unavailable() const { return d->unavailable_; }
+int Song::id() const { return d->id_; }
+const QString& Song::title() const { return d->title_; }
+const QString& Song::album() const { return d->album_; }
+const QString& Song::artist() const { return d->artist_; }
+const QString& Song::albumartist() const { return d->albumartist_; }
+const QString& Song::effective_albumartist() const { return d->albumartist_.isEmpty() ? d->artist_ : d->albumartist_; }
+const QString& Song::playlist_albumartist() const { return is_compilation() ? d->albumartist_ : effective_albumartist(); }
+const QString& Song::composer() const { return d->composer_; }
+int Song::track() const { return d->track_; }
+int Song::disc() const { return d->disc_; }
+float Song::bpm() const { return d->bpm_; }
+int Song::year() const { return d->year_; }
+const QString& Song::genre() const { return d->genre_; }
+const QString& Song::comment() const { return d->comment_; }
+bool Song::is_compilation() const {
+  return (d->compilation_ || d->sampler_ || d->forced_compilation_on_)
+          && ! d->forced_compilation_off_;
+}
+float Song::rating() const { return d->rating_; }
+int Song::playcount() const { return d->playcount_; }
+int Song::skipcount() const { return d->skipcount_; }
+int Song::lastplayed() const { return d->lastplayed_; }
+int Song::score() const { return d->score_; }
+const QString& Song::cue_path() const { return d->cue_path_; }
+bool Song::has_cue() const { return !d->cue_path_.isEmpty(); }
+qint64 Song::beginning_nanosec() const { return d->beginning_; }
+qint64 Song::end_nanosec() const { return d->end_; }
+qint64 Song::length_nanosec() const { return d->end_ - d->beginning_; }
+int Song::bitrate() const { return d->bitrate_; }
+int Song::samplerate() const { return d->samplerate_; }
+int Song::directory_id() const { return d->directory_id_; }
+const QUrl& Song::url() const { return d->url_; }
+const QString& Song::basefilename() const { return d->basefilename_; }
+uint Song::mtime() const { return d->mtime_; }
+uint Song::ctime() const { return d->ctime_; }
+int Song::filesize() const { return d->filesize_; }
+Song::FileType Song::filetype() const { return d->filetype_; }
+bool Song::is_stream() const { return d->filetype_ == Type_Stream; }
+bool Song::is_cdda() const { return d->filetype_ == Type_Cdda; }
+const QString& Song::art_automatic() const { return d->art_automatic_; }
+const QString& Song::art_manual() const { return d->art_manual_; }
+bool Song::has_manually_unset_cover() const { return d->art_manual_ == kManuallyUnsetCover; }
+void Song::manually_unset_cover() { d->art_manual_ = kManuallyUnsetCover; }
+bool Song::has_embedded_cover() const { return d->art_automatic_ == kEmbeddedCover; }
+void Song::set_embedded_cover() { d->art_automatic_ = kEmbeddedCover; }
+const QImage& Song::image() const { return d->image_; }
+void Song::set_id(int id) { d->id_ = id; }
+void Song::set_valid(bool v) { d->valid_ = v; }
+void Song::set_title(const QString& v) { d->title_ = v; }
+void Song::set_album(const QString& v) { d->album_ = v; }
+void Song::set_artist(const QString& v) { d->artist_ = v; }
+void Song::set_albumartist(const QString& v) { d->albumartist_ = v; }
+void Song::set_composer(const QString& v) { d->composer_ = v; }
+void Song::set_track(int v) { d->track_ = v; }
+void Song::set_disc(int v) { d->disc_ = v; }
+void Song::set_bpm(float v) { d->bpm_ = v; }
+void Song::set_year(int v) { d->year_ = v; }
+void Song::set_genre(const QString& v) { d->genre_ = v; }
+void Song::set_comment(const QString& v) { d->comment_ = v; }
+void Song::set_compilation(bool v) { d->compilation_ = v; }
+void Song::set_sampler(bool v) { d->sampler_ = v; }
+void Song::set_beginning_nanosec(qint64 v) { d->beginning_ = qMax(0ll, v); }
+void Song::set_end_nanosec(qint64 v) { d->end_ = v; }
+void Song::set_length_nanosec(qint64 v) { d->end_ = d->beginning_ + v; }
+void Song::set_bitrate(int v) { d->bitrate_ = v; }
+void Song::set_samplerate(int v) { d->samplerate_ = v; }
+void Song::set_mtime(int v) { d->mtime_ = v; }
+void Song::set_ctime(int v) { d->ctime_ = v; }
+void Song::set_filesize(int v) { d->filesize_ = v; }
+void Song::set_filetype(FileType v) { d->filetype_ = v; }
+void Song::set_art_automatic(const QString& v) { d->art_automatic_ = v; }
+void Song::set_art_manual(const QString& v) { d->art_manual_ = v; }
+void Song::set_image(const QImage& i) { d->image_ = i; }
+void Song::set_forced_compilation_on(bool v) { d->forced_compilation_on_ = v; }
+void Song::set_forced_compilation_off(bool v) { d->forced_compilation_off_ = v; }
+void Song::set_rating(float v) { d->rating_ = v; }
+void Song::set_playcount(int v) { d->playcount_ = v; }
+void Song::set_skipcount(int v) { d->skipcount_ = v; }
+void Song::set_lastplayed(int v) { d->lastplayed_ = v; }
+void Song::set_score(int v) { d->score_ = qBound(0, v, 100); }
+void Song::set_cue_path(const QString& v) { d->cue_path_ = v; }
+void Song::set_unavailable(bool v) { d->unavailable_ = v; }
+void Song::set_url(const QUrl& v) { d->url_ = v; }
+void Song::set_basefilename(const QString& v) { d->basefilename_ = v; }
+void Song::set_directory_id(int v) { d->directory_id_ = v; }
+
+QString Song::JoinSpec(const QString& table) {
+  return Prepend(table + ".", kColumns).join(", ");
+}
+
+QString Song::TextForFiletype(FileType type) {
+  switch (type) {
+    case Song::Type_Asf:       return QObject::tr("Windows Media audio");
+    case Song::Type_Flac:      return QObject::tr("Flac");
+    case Song::Type_Mp4:       return QObject::tr("MP4 AAC");
+    case Song::Type_Mpc:       return QObject::tr("MPC");
+    case Song::Type_Mpeg:      return QObject::tr("MP3"); // Not technically correct
+    case Song::Type_OggFlac:   return QObject::tr("Ogg Flac");
+    case Song::Type_OggSpeex:  return QObject::tr("Ogg Speex");
+    case Song::Type_OggVorbis: return QObject::tr("Ogg Vorbis");
+    case Song::Type_Aiff:      return QObject::tr("AIFF");
+    case Song::Type_Wav:       return QObject::tr("Wav");
+    case Song::Type_TrueAudio: return QObject::tr("TrueAudio");
+    case Song::Type_Cdda:      return QObject::tr("CDDA");
+
+    case Song::Type_Stream:    return QObject::tr("Stream");
+
+    case Song::Type_Unknown:
+    default:
+      return QObject::tr("Unknown");
+  }
+}
+
 void Song::Init(const QString& title, const QString& artist,
                 const QString& album, qint64 length_nanosec) {
   d->valid_ = true;
@@ -249,11 +450,11 @@ QString Song::Decode(const QString& tag, const QTextCodec* codec) {
 bool Song::HasProperMediaFile() const {
 #ifndef QT_NO_DEBUG_OUTPUT
   if (qApp->thread() == QThread::currentThread())
-    qWarning() << Q_FUNC_INFO << "on GUI thread!";
+    qLog(Warning) << "HasProperMediaFile() on GUI thread!";
 #endif
 
-  QMutexLocker l(&taglib_mutex_);
-  scoped_ptr<TagLib::FileRef> fileref(factory_->GetFileRef(d->filename_));
+  QMutexLocker l(&sTaglibMutex);
+  scoped_ptr<TagLib::FileRef> fileref(factory_->GetFileRef(d->url_.toLocalFile()));
 
   return !fileref->isNull() && fileref->tag();
 }
@@ -261,18 +462,18 @@ bool Song::HasProperMediaFile() const {
 void Song::InitFromFile(const QString& filename, int directory_id) {
 #ifndef QT_NO_DEBUG_OUTPUT
   if (qApp->thread() == QThread::currentThread())
-    qWarning() << Q_FUNC_INFO << "on GUI thread!";
+    qLog(Warning) << "InitFromFile() on GUI thread!";
 #endif
 
   d->init_from_file_ = true;
 
-  d->filename_ = filename;
+  d->url_ = QUrl::fromLocalFile(filename);
   d->directory_id_ = directory_id;
 
   QFileInfo info(filename);
   d->basefilename_ = info.fileName();
 
-  QMutexLocker l(&taglib_mutex_);
+  QMutexLocker l(&sTaglibMutex);
   scoped_ptr<TagLib::FileRef> fileref(factory_->GetFileRef(filename));
 
   if(fileref->isNull()) {
@@ -359,14 +560,31 @@ void Song::InitFromFile(const QString& filename, int directory_id) {
     }
   } else if (TagLib::Ogg::Vorbis::File* file = dynamic_cast<TagLib::Ogg::Vorbis::File*>(fileref->file())) {
     if (file->tag()) {
-      ParseOggTag(file->tag()->fieldListMap(), NULL, &disc, &compilation);
+      d->ParseOggTag(file->tag()->fieldListMap(), NULL, &disc, &compilation);
     }
     d->comment_ = Decode(tag->comment());
   } else if (TagLib::FLAC::File* file = dynamic_cast<TagLib::FLAC::File*>(fileref->file())) {
     if ( file->xiphComment() ) {
-      ParseOggTag(file->xiphComment()->fieldListMap(), NULL, &disc, &compilation);
+      d->ParseOggTag(file->xiphComment()->fieldListMap(), NULL, &disc, &compilation);
+#ifdef TAGLIB_HAS_FLAC_PICTURELIST
+      if (!file->pictureList().isEmpty()) {
+        set_embedded_cover();
+      }
+#endif
     }
     d->comment_ = Decode(tag->comment());
+  } else if (TagLib::MP4::File* file = dynamic_cast<TagLib::MP4::File*>(fileref->file())) {
+    if (file->tag()) {
+      TagLib::MP4::Tag* mp4_tag = file->tag();
+      const TagLib::MP4::ItemListMap& items = mp4_tag->itemListMap();
+      TagLib::MP4::ItemListMap::ConstIterator it = items.find("aART");
+      if (it != items.end()) {
+        TagLib::StringList album_artists = it->second.toStringList();
+        if (!album_artists.isEmpty()) {
+          d->albumartist_ = Decode(album_artists.front());
+        }
+      }
+    }
   } else if (tag) {
     d->comment_ = Decode(tag->comment());
   }
@@ -446,16 +664,20 @@ void Song::ParseFMPSFrame(const QString& name, const QString& value) {
   }
 }
 
-void Song::ParseOggTag(const TagLib::Ogg::FieldListMap& map, const QTextCodec* codec,
-                       QString* disc, QString* compilation) {
+void Song::Private::ParseOggTag(const TagLib::Ogg::FieldListMap& map,
+                                const QTextCodec* codec,
+                                QString* disc, QString* compilation) {
   if (!map["COMPOSER"].isEmpty())
-    d->composer_ = Decode(map["COMPOSER"].front(), codec);
+    composer_ = Decode(map["COMPOSER"].front(), codec);
 
-  if (!map["ALBUMARTIST"].isEmpty())
-    d->albumartist_ = Decode(map["ALBUMARTIST"].front(), codec);
+  if (!map["ALBUMARTIST"].isEmpty()) {
+    albumartist_ = Decode(map["ALBUMARTIST"].front(), codec);
+  } else if (!map["ALBUM ARTIST"].isEmpty()) {
+    albumartist_ = Decode(map["ALBUM ARTIST"].front(), codec);
+  }
 
   if (!map["BPM"].isEmpty() )
-    d->bpm_ = TStringToQString( map["BPM"].front() ).trimmed().toFloat();
+    bpm_ = TStringToQString( map["BPM"].front() ).trimmed().toFloat();
 
   if (!map["DISCNUMBER"].isEmpty() )
     *disc = TStringToQString( map["DISCNUMBER"].front() ).trimmed();
@@ -464,7 +686,7 @@ void Song::ParseOggTag(const TagLib::Ogg::FieldListMap& map, const QTextCodec* c
     *compilation = TStringToQString( map["COMPILATION"].front() ).trimmed();
 
   if (!map["COVERART"].isEmpty())
-    set_embedded_cover();
+    art_automatic_ = kEmbeddedCover;
 }
 
 void Song::GuessFileType(TagLib::FileRef* fileref) {
@@ -496,12 +718,12 @@ void Song::GuessFileType(TagLib::FileRef* fileref) {
     d->filetype_ = Type_TrueAudio;
 }
 
-void Song::InitFromQuery(const SqlRow& q, int col) {
+void Song::InitFromQuery(const SqlRow& q, bool reliable_metadata, int col) {
   d->valid_ = true;
-
-  d->init_from_file_ = true;
+  d->init_from_file_ = reliable_metadata;
 
   #define tostr(n)      (q.value(n).isNull() ? QString::null : q.value(n).toString())
+  #define tobytearray(n)(q.value(n).isNull() ? QByteArray()  : q.value(n).toByteArray())
   #define toint(n)      (q.value(n).isNull() ? -1 : q.value(n).toInt())
   #define tolonglong(n) (q.value(n).isNull() ? -1 : q.value(n).toLongLong())
   #define tofloat(n)    (q.value(n).isNull() ? -1 : q.value(n).toDouble())
@@ -524,8 +746,8 @@ void Song::InitFromQuery(const SqlRow& q, int col) {
   d->samplerate_ = toint(col + 14);
 
   d->directory_id_ = toint(col + 15);
-  d->filename_ = tostr(col + 16);
-  d->basefilename_ = QFileInfo(d->filename_).fileName();
+  d->url_ = QUrl::fromEncoded(tobytearray(col + 16));
+  d->basefilename_ = QFileInfo(d->url_.toLocalFile()).fileName();
   d->mtime_ = toint(col + 17);
   d->ctime_ = toint(col + 18);
   d->filesize_ = toint(col + 19);
@@ -554,6 +776,9 @@ void Song::InitFromQuery(const SqlRow& q, int col) {
   set_length_nanosec(tolonglong(col + 33));
 
   d->cue_path_ = tostr(col + 34);
+  d->unavailable_ = q.value(col + 35).toBool();
+
+  // effective_albumartist = 36
 
   #undef tostr
   #undef toint
@@ -561,6 +786,24 @@ void Song::InitFromQuery(const SqlRow& q, int col) {
   #undef tofloat
 }
 
+void Song::InitFromFilePartial(const QString& filename) {
+  d->url_ = QUrl::fromLocalFile(filename);
+  // We currently rely on filename suffix to know if it's a music file or not.
+  // TODO: I know this is not satisfying, but currently, we rely on TagLib
+  // which seems to have the behavior (filename checks). Someday, it would be
+  // nice to perform some magic tests everywhere.
+  QFileInfo info(filename);
+  d->basefilename_ = info.fileName();
+  QString suffix = info.suffix().toLower();
+  if (suffix == "mp3" || suffix == "ogg" || suffix == "flac" || suffix == "mpc"
+      || suffix == "m4a" || suffix == "aac" || suffix == "wma" || suffix == "mp4"
+      || suffix == "spx" || suffix == "wav") {
+    d->valid_ = true;
+  } else {
+    d->valid_ = false;
+  }
+}
+
 #ifdef HAVE_LIBLASTFM
 void Song::InitFromLastFM(const lastfm::Track& track) {
   d->valid_ = true;
@@ -576,7 +819,7 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
 #endif // HAVE_LIBLASTFM
 
 #ifdef HAVE_LIBGPOD
-  void Song::InitFromItdb(const Itdb_Track* track) {
+  void Song::InitFromItdb(const Itdb_Track* track, const QString& prefix) {
     d->valid_ = true;
 
     d->title_ = QString::fromUtf8(track->title);
@@ -603,9 +846,16 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
     d->skipcount_ = track->skipcount;
     d->lastplayed_ = track->time_played;
 
-    d->filename_ = QString::fromLocal8Bit(track->ipod_path);
-    d->filename_.replace(':', '/');
-    d->basefilename_ = QFileInfo(d->filename_).fileName();
+    QString filename = QString::fromLocal8Bit(track->ipod_path);
+    filename.replace(':', '/');
+
+    if (prefix.contains("://")) {
+      d->url_ = prefix + filename;
+    } else {
+      d->url_ = QUrl::fromLocalFile(prefix + filename);
+    }
+
+    d->basefilename_ = QFileInfo(filename).fileName();
   }
 
   void Song::ToItdb(Itdb_Track *track) const {
@@ -638,7 +888,7 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
 #endif
 
 #ifdef HAVE_LIBMTP
-  void Song::InitFromMTP(const LIBMTP_track_t* track) {
+  void Song::InitFromMTP(const LIBMTP_track_t* track, const QString& host) {
     d->valid_ = true;
 
     d->title_ = QString::fromUtf8(track->title);
@@ -646,8 +896,8 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
     d->album_ = QString::fromUtf8(track->album);
     d->composer_ = QString::fromUtf8(track->composer);
     d->genre_ = QString::fromUtf8(track->genre);
-    d->filename_ = QString::number(track->item_id);
-    d->basefilename_ = d->filename_;
+    d->url_ = QUrl(QString("mtp://%1/%2").arg(host, track->item_id));
+    d->basefilename_ = QString::number(track->item_id);
 
     d->track_ = track->tracknumber;
     set_length_nanosec(track->duration * kNsecPerMsec);
@@ -747,7 +997,7 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
         break;
       }
       default:
-        qWarning() << "Type" << value.type() << "not handled";
+        qLog(Warning) << "Type" << value.type() << "not handled";
         Q_ASSERT(0);
         break;
     }
@@ -821,7 +1071,7 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
         d->bitrate_ = item_value.toInt();
 
       else if (wcscmp(name, g_wszWMDMFileName) == 0)
-        d->filename_ = item_value.toString();
+        d->url_ = QUrl::fromLocalFile(item_value.toString());
 
       else if (wcscmp(name, g_wszWMDMDuration) == 0)
         set_length_nanosec(item_value.toULongLong() * 1e2);
@@ -895,7 +1145,7 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
 
       // Make a final guess based on the file extension
       {
-        QString ext = d->filename_.section('.', -1, -1).toLower();
+        QString ext = d->url_.path().section('.', -1, -1).toLower();
         if (ext == "mp3" || ext == "wma" || ext == "flac" || ext == "ogg" ||
             ext == "spx" || ext == "mp4" || ext == "aac" || ext == "m4a")
           break;
@@ -922,7 +1172,7 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
     AddWmdmItem(metadata, g_wszWMDMComposer, d->composer_);
     AddWmdmItem(metadata, g_wszWMDMBitrate, d->bitrate_);
     AddWmdmItem(metadata, g_wszWMDMFileName, d->basefilename_);
-    AddWmdmItem(metadata, g_wszWMDMDuration, qint64(length_nanosec()) / 1e2);
+    AddWmdmItem(metadata, g_wszWMDMDuration, qint64(length_nanosec() / 1e2));
     AddWmdmItem(metadata, L"WMDM/FileSize", d->filesize_);
 
     WMDM_FORMATCODE format;
@@ -989,7 +1239,7 @@ void Song::BindToQuery(QSqlQuery *query) const {
   query->bindValue(":samplerate", intval(d->samplerate_));
 
   query->bindValue(":directory", notnullintval(d->directory_id_));
-  query->bindValue(":filename", d->filename_);
+  query->bindValue(":filename", d->url_.toEncoded());
   query->bindValue(":mtime", notnullintval(d->mtime_));
   query->bindValue(":ctime", notnullintval(d->ctime_));
   query->bindValue(":filesize", notnullintval(d->filesize_));
@@ -1015,6 +1265,8 @@ void Song::BindToQuery(QSqlQuery *query) const {
   query->bindValue(":length", intval(length_nanosec()));
 
   query->bindValue(":cue_path", d->cue_path_);
+  query->bindValue(":unavailable", d->unavailable_ ? 1 : 0);
+  query->bindValue(":effective_albumartist", this->effective_albumartist());
 
   #undef intval
   #undef notnullintval
@@ -1039,21 +1291,22 @@ void Song::ToLastFM(lastfm::Track* track) const {
   mtrack.setTitle(d->title_);
   mtrack.setDuration(length_nanosec() / kNsecPerSec);
   mtrack.setTrackNumber(d->track_);
-  mtrack.setSource(lastfm::Track::Player);
+
+  if (d->filetype_ == Type_Stream && d->end_ == -1) {
+    mtrack.setSource(lastfm::Track::NonPersonalisedBroadcast);
+  } else {
+    mtrack.setSource(lastfm::Track::Player);
+  }
 }
 #endif // HAVE_LIBLASTFM
 
-QUrl Song::url() const {
-  return QFile::exists(filename())
-             ? QUrl::fromLocalFile(filename())
-             : filename();
-}
-
 QString Song::PrettyTitle() const {
   QString title(d->title_);
 
   if (title.isEmpty())
     title = d->basefilename_;
+  if (title.isEmpty())
+    title = d->url_.toString();
 
   return title;
 }
@@ -1136,21 +1389,18 @@ void Song::SetTextFrame(const QString& id, const QString& value,
   tag->addFrame(frame);
 }
 
-TagLib::String QStringToTaglibString(const QString& s) {
-  return TagLib::String(s.toUtf8().constData(), TagLib::String::UTF8);
-}
-
 bool Song::IsEditable() const {
-  return d->valid_ && !d->filename_.isNull() && !is_stream() &&
+  return d->valid_ && !d->url_.isEmpty() && !is_stream() &&
          d->filetype_ != Type_Unknown && !has_cue();
 }
 
 bool Song::Save() const {
-  if (d->filename_.isNull())
+  const QString filename = d->url_.toLocalFile();
+  if (filename.isNull())
     return false;
 
-  QMutexLocker l(&taglib_mutex_);
-  scoped_ptr<TagLib::FileRef> fileref(factory_->GetFileRef(d->filename_));
+  QMutexLocker l(&sTaglibMutex);
+  scoped_ptr<TagLib::FileRef> fileref(factory_->GetFileRef(filename));
 
   if (!fileref || fileref->isNull()) // The file probably doesn't exist
     return false;
@@ -1191,7 +1441,7 @@ bool Song::Save() const {
   if (ret) {
     // Linux: inotify doesn't seem to notice the change to the file unless we
     // change the timestamps as well. (this is what touch does)
-    utimensat(0, QFile::encodeName(d->filename_).constData(), NULL, 0);
+    utimensat(0, QFile::encodeName(filename).constData(), NULL, 0);
   }
   #endif  // Q_OS_LINUX
 
@@ -1209,16 +1459,21 @@ QFuture<bool> Song::BackgroundSave() const {
 
 bool Song::operator==(const Song& other) const {
   // TODO: this isn't working for radios
-  return filename() == other.filename() &&
+  return url() == other.url() &&
          beginning_nanosec() == other.beginning_nanosec();
 }
 
+uint qHash(const Song& song) {
+  // Should compare the same fields as operator==
+  return qHash(song.url().toString()) ^ qHash(song.beginning_nanosec());
+}
+
 QImage Song::LoadEmbeddedArt(const QString& filename) {
   QImage ret;
   if (filename.isEmpty())
     return ret;
 
-  QMutexLocker l(&taglib_mutex_);
+  QMutexLocker l(&sTaglibMutex);
 
 #ifdef Q_OS_WIN32
   TagLib::FileRef ref(filename.toStdWString().c_str());
@@ -1243,7 +1498,7 @@ QImage Song::LoadEmbeddedArt(const QString& filename) {
     return ret;
   }
 
-  // Ogg vorbis/flac/speex
+  // Ogg vorbis/speex
   TagLib::Ogg::XiphComment* xiph_comment =
       dynamic_cast<TagLib::Ogg::XiphComment*>(ref.file()->tag());
 
@@ -1260,8 +1515,29 @@ QImage Song::LoadEmbeddedArt(const QString& filename) {
 
     if (!ret.loadFromData(image_data))
       ret.loadFromData(image_data_b64); //maybe it's not b64 after all
-  }
 
+    return ret;
+  }
+  
+#ifdef TAGLIB_HAS_FLAC_PICTURELIST
+  // Flac
+  TagLib::FLAC::File* flac_file = dynamic_cast<TagLib::FLAC::File*>(ref.file());
+  if (flac_file && flac_file->xiphComment()) {
+    TagLib::List<TagLib::FLAC::Picture*> pics = flac_file->pictureList();
+    if (!pics.isEmpty()) {
+      // Use the first picture in the file - this could be made cleverer and
+      // pick the front cover if it's present.
+
+      std::list<TagLib::FLAC::Picture*>::iterator it = pics.begin();
+      TagLib::FLAC::Picture* picture = *it;
+
+      QByteArray image_data(picture->data().data(), picture->data().size());
+      ret.loadFromData(image_data);
+      return ret;
+    }
+  }
+#endif
+    
   return ret;
 }
 
@@ -1277,3 +1553,36 @@ bool Song::IsOnSameAlbum(const Song& other) const {
 
   return album() == other.album() && artist() == other.artist();
 }
+
+QString Song::AlbumKey() const {
+  return QString("%1|%2|%3").arg(
+        is_compilation() ? "_compilation" : artist(),
+        has_cue() ? cue_path() : "",
+        album());
+}
+
+void Song::ToXesam(QVariantMap* map) const {
+  using mpris::AddMetadata;
+  using mpris::AddMetadataAsList;
+  using mpris::AsMPRISDateTimeType;
+
+  AddMetadata("xesam:url", url().toString(), map);
+  AddMetadata("xesam:title", PrettyTitle(), map);
+  AddMetadataAsList("xesam:artist", artist(), map);
+  AddMetadata("xesam:album", album(), map);
+  AddMetadataAsList("xesam:albumArtist", albumartist(), map);
+  AddMetadata("mpris:length", length_nanosec() / kNsecPerUsec, map);
+  AddMetadata("xesam:trackNumber", track(), map);
+  AddMetadataAsList("xesam:genre", genre(), map);
+  AddMetadata("xesam:discNumber", disc(), map);
+  AddMetadataAsList("xesam:comment", comment(), map);
+  AddMetadata("xesam:contentCreated", AsMPRISDateTimeType(ctime()), map);
+  AddMetadata("xesam:lastUsed", AsMPRISDateTimeType(lastplayed()), map);
+  AddMetadata("xesam:audioBPM", bpm(), map);
+  AddMetadataAsList("xesam:composer", composer(), map);
+  AddMetadata("xesam:useCount", playcount(), map);
+  AddMetadata("xesam:autoRating", score(), map);
+  if (rating() != -1.0) {
+    AddMetadata("xesam:userRating", rating(), map);
+  }
+}
