@@ -15,24 +15,33 @@
    along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "config.h"
 #include "songloader.h"
+#include "core/logging.h"
 #include "core/song.h"
+#include "core/timeconstants.h"
 #include "library/librarybackend.h"
 #include "library/sqlrow.h"
 #include "playlistparsers/parserbase.h"
 #include "playlistparsers/cueparser.h"
 #include "playlistparsers/playlistparser.h"
-#include "radio/fixlastfm.h"
+#include "internet/fixlastfm.h"
 
 #include <QBuffer>
 #include <QDirIterator>
 #include <QFileInfo>
 #include <QTimer>
+#include <QUrl>
 #include <QtConcurrentRun>
 #include <QtDebug>
 
 #include <boost/bind.hpp>
 
+#ifdef HAVE_AUDIOCD
+# include <gst/cdda/gstcddabasesrc.h>
+#endif
+
+
 QSet<QString> SongLoader::sRawUriSchemes;
 const int SongLoader::kDefaultTimeout = 5000;
 
@@ -82,9 +91,127 @@ SongLoader::Result SongLoader::Load(const QUrl& url) {
   return LoadRemote();
 }
 
+SongLoader::Result SongLoader::LoadLocalPartial(const QString& filename) {
+  qLog(Debug) << "Fast Loading local file" << filename;
+  // First check to see if it's a directory - if so we can load all the songs
+  // inside right away.
+  if (QFileInfo(filename).isDir()) {
+    LoadLocalDirectory(filename);
+    return Success;
+  }
+  Song song;
+  song.InitFromFilePartial(filename);
+  if (song.is_valid())
+    songs_ << song;
+  return Success;
+}
+
+SongLoader::Result SongLoader::LoadAudioCD() {
+#ifdef HAVE_AUDIOCD
+  // Create gstreamer cdda element
+  GstElement* cdda = gst_element_make_from_uri (GST_URI_SRC, "cdda://", NULL);
+  if (cdda == NULL) {
+    qLog(Error) << "Error while creating CDDA GstElement";
+    return Error;
+  }
+
+  // Change the element's state to ready and paused, to be able to query it
+  if (gst_element_set_state(cdda, GST_STATE_READY) == GST_STATE_CHANGE_FAILURE ||
+      gst_element_set_state(cdda, GST_STATE_PAUSED) == GST_STATE_CHANGE_FAILURE) {
+    qLog(Error) << "Error while changing CDDA GstElement's state";
+    gst_element_set_state(cdda, GST_STATE_NULL);
+    gst_object_unref(GST_OBJECT(cdda));
+    return Error;
+  }
+
+  // Get number of tracks
+  GstFormat fmt = gst_format_get_by_nick ("track");
+  GstFormat out_fmt = fmt;
+  gint64 num_tracks = 0;
+  if (!gst_element_query_duration (cdda, &out_fmt, &num_tracks) || out_fmt != fmt) {
+    qLog(Error) << "Error while querying cdda GstElement";
+    gst_object_unref(GST_OBJECT(cdda));
+    return Error;
+  }
+
+  for (int track_number = 1; track_number <= num_tracks; track_number++) {
+    // Init song
+    Song song;
+    guint64 duration = 0;
+    // quint64 == ulonglong and guint64 == ulong, therefore we must cast
+    if (gst_tag_list_get_uint64 (GST_CDDA_BASE_SRC(cdda)->tracks[track_number-1].tags,
+                                 GST_TAG_DURATION, &duration)) {
+      song.set_length_nanosec((quint64)duration);
+    }
+    song.set_valid(true);
+    song.set_filetype(Song::Type_Cdda);
+    song.set_url(QUrl(QString("cdda://%1").arg(track_number)));
+    song.set_title(QString("Track %1").arg(track_number));
+    song.set_track(track_number);
+    songs_ << song;
+  }
+
+  // Generate MusicBrainz DiscId
+  gst_tag_register_musicbrainz_tags();
+  GstElement *pipe = gst_pipeline_new ("pipeline");
+  gst_bin_add (GST_BIN (pipe), cdda);
+  gst_element_set_state (pipe, GST_STATE_READY);
+  gst_element_set_state (pipe, GST_STATE_PAUSED);
+  GstMessage *msg = gst_bus_timed_pop_filtered (GST_ELEMENT_BUS (pipe),
+                    GST_CLOCK_TIME_NONE,
+                    GST_MESSAGE_TAG);
+  GstTagList *tags = NULL;
+  gst_message_parse_tag (msg, &tags);
+  char *string_mb = NULL;
+  if (gst_tag_list_get_string (tags, GST_TAG_CDDA_MUSICBRAINZ_DISCID, &string_mb)) {
+    QString musicbrainz_discid(string_mb);
+    qLog(Info) << "MusicBrainz discid: " << musicbrainz_discid;
+
+    MusicBrainzClient *musicbrainz_client = new MusicBrainzClient(this);
+    connect(musicbrainz_client,
+            SIGNAL(Finished(const QString&, const QString&, MusicBrainzClient::ResultList)),
+            SLOT(AudioCDTagsLoaded(const QString&, const QString&, MusicBrainzClient::ResultList)));
+    musicbrainz_client->StartDiscIdRequest(musicbrainz_discid);
+    g_free(string_mb);
+  }
+  
+  // Clean all the Gstreamer objects we have used: we don't need them anymore
+  gst_object_unref(GST_OBJECT(cdda));
+  gst_element_set_state (pipe, GST_STATE_NULL);
+  gst_object_unref(GST_OBJECT(pipe));
+  gst_object_unref(GST_OBJECT(msg));
+  gst_object_unref(GST_OBJECT(tags));
+
+  return Success;
+#else // HAVE_AUDIOCD
+  return Error;
+#endif
+}
+
+void SongLoader::AudioCDTagsLoaded(const QString& artist, const QString& album,
+                                   const MusicBrainzClient::ResultList& results) {
+  // Remove previously added songs metadata, because there are not needed
+  // and that we are going to fill it with new (more complete) ones
+  songs_.clear();
+  int track_number = 1;
+  foreach (const MusicBrainzClient::Result& ret, results) {
+    Song song;
+    song.set_artist(artist);
+    song.set_album(album);
+    song.set_title(ret.title_);
+    song.set_length_nanosec(ret.duration_msec_ * kNsecPerMsec);
+    song.set_track(track_number);
+    song.set_year(ret.year_);
+    // We need to set url: that's how playlist will find the correct item to update
+    song.set_url(QUrl(QString("cdda://%1").arg(track_number++)));
+    songs_ << song;
+  }
+  emit LoadFinished(true);
+}
+
 SongLoader::Result SongLoader::LoadLocal(const QString& filename, bool block,
                                          bool ignore_playlists) {
-  qDebug() << "Loading local file" << filename;
+  qLog(Debug) << "Loading local file" << filename;
 
   // First check to see if it's a directory - if so we can load all the songs
   // inside right away.
@@ -114,11 +241,11 @@ SongLoader::Result SongLoader::LoadLocal(const QString& filename, bool block,
 
   if (parser) {
     if (ignore_playlists) {
-      qDebug() << "Skipping" << parser->name() << "playlist while loading directory";
+      qLog(Debug) << "Skipping" << parser->name() << "playlist while loading directory";
       return Success;
     }
 
-    qDebug() << "Parsing using" << parser->name();
+    qLog(Debug) << "Parsing using" << parser->name();
 
     // It's a playlist!
     if (!block) {
@@ -131,11 +258,11 @@ SongLoader::Result SongLoader::LoadLocal(const QString& filename, bool block,
   }
 
   // Not a playlist, so just assume it's a song
-  QFileInfo info(filename);
+  QUrl url = QUrl::fromLocalFile(filename);
 
   LibraryQuery query;
   query.SetColumnSpec("%songs_table.ROWID, " + Song::kColumnSpec);
-  query.AddWhere("filename", info.canonicalFilePath());
+  query.AddWhere("filename", url.toEncoded());
 
   SongList song_list;
 
@@ -143,7 +270,7 @@ SongLoader::Result SongLoader::LoadLocal(const QString& filename, bool block,
     // we may have many results when the file has many sections
     do {
       Song song;
-      song.InitFromQuery(query);
+      song.InitFromQuery(query, true);
 
       song_list << song;
     } while(query.Next());
@@ -166,8 +293,7 @@ SongLoader::Result SongLoader::LoadLocal(const QString& filename, bool block,
 
     }
   }
-
-  foreach(const Song& song, song_list) {
+  foreach (const Song& song, song_list) {
     if (song.is_valid())
       songs_ << song;
   }
@@ -175,6 +301,27 @@ SongLoader::Result SongLoader::LoadLocal(const QString& filename, bool block,
   return Success;
 }
 
+void SongLoader::EffectiveSongsLoad() {
+  for (int i = 0; i < songs_.size(); i++) {
+    Song& song = songs_[i];
+
+    if (song.filetype() != Song::Type_Unknown) {
+      // Maybe we loaded the metadata already, for example from a cuesheet.
+      continue;
+    }
+
+    // First, try to get the song from the library
+    Song library_song = library_->GetSongByUrl(song.url());
+    if (library_song.is_valid()) {
+      song = library_song;
+    } else {
+      // it's a normal media file
+      QString filename = song.url().toLocalFile();
+      song.InitFromFile(filename, -1);
+    }
+  }
+}
+
 void SongLoader::LoadPlaylistAndEmit(ParserBase* parser, const QString& filename) {
   LoadPlaylist(parser, filename);
   emit LoadFinished(true);
@@ -196,7 +343,7 @@ static bool CompareSongs(const Song& left, const Song& right) {
   if (left.disc() > right.disc()) return false;
   if (left.track() < right.track()) return true;
   if (left.track() > right.track()) return false;
-  return left.filename() < right.filename();
+  return left.url() < right.url();
 }
 
 void SongLoader::LoadLocalDirectoryAndEmit(const QString& filename) {
@@ -209,8 +356,7 @@ void SongLoader::LoadLocalDirectory(const QString& filename) {
                   QDirIterator::Subdirectories);
 
   while (it.hasNext()) {
-    // This is in another thread so we can do blocking calls.
-    LoadLocal(it.next(), true, true);
+    LoadLocalPartial(it.next());
   }
 
   qStableSort(songs_.begin(), songs_.end(), CompareSongs);
@@ -220,7 +366,7 @@ void SongLoader::AddAsRawStream() {
   Song song;
   song.set_valid(true);
   song.set_filetype(Song::Type_Stream);
-  song.set_filename(url_.toString());
+  song.set_url(url_);
   song.set_title(url_.toString());
   songs_ << song;
 }
@@ -240,14 +386,14 @@ void SongLoader::StopTypefind() {
   timeout_timer_->stop();
 
   if (success_ && parser_) {
-    qDebug() << "Parsing" << url_ << "with" << parser_->name();
+    qLog(Debug) << "Parsing" << url_ << "with" << parser_->name();
 
     // Parse the playlist
     QBuffer buf(&buffer_);
     buf.open(QIODevice::ReadOnly);
     songs_ = parser_->Load(&buf);
   } else if (success_) {
-    qDebug() << "Loading" << url_ << "as raw stream";
+    qLog(Debug) << "Loading" << url_ << "as raw stream";
 
     // It wasn't a playlist - just put the URL in as a stream
     AddAsRawStream();
@@ -257,7 +403,7 @@ void SongLoader::StopTypefind() {
 }
 
 SongLoader::Result SongLoader::LoadRemote() {
-  qDebug() << "Loading remote file" << url_;
+  qLog(Debug) << "Loading remote file" << url_;
 
   // It's not a local file so we have to fetch it to see what it is.  We use
   // gstreamer to do this since it handles funky URLs for us (http://, ssh://,
@@ -277,7 +423,7 @@ SongLoader::Result SongLoader::LoadRemote() {
   GstElement* source = gst_element_make_from_uri(
       GST_URI_SRC, url_.toEncoded().constData(), NULL);
   if (!source) {
-    qWarning() << "Couldn't create gstreamer source element for" << url_.toString();
+    qLog(Warning) << "Couldn't create gstreamer source element for" << url_.toString();
     return Error;
   }
 
@@ -313,7 +459,7 @@ void SongLoader::TypeFound(GstElement*, uint, GstCaps* caps, void* self) {
 
   // Check the mimetype
   instance->mime_type_ = gst_structure_get_name(gst_caps_get_structure(caps, 0));
-  qDebug() << "Mime type is" << instance->mime_type_;
+  qLog(Debug) << "Mime type is" << instance->mime_type_;
   if (instance->mime_type_ == "text/plain" ||
       instance->mime_type_ == "text/uri-list" ||
       instance->mime_type_ == "application/xml") {
@@ -326,7 +472,7 @@ void SongLoader::TypeFound(GstElement*, uint, GstCaps* caps, void* self) {
   instance->StopTypefindAsync(true);
 }
 
-void SongLoader::DataReady(GstPad *, GstBuffer *buf, void *self) {
+void SongLoader::DataReady(GstPad*, GstBuffer* buf, void* self) {
   SongLoader* instance = static_cast<SongLoader*>(self);
 
   if (instance->state_ == Finished)
@@ -335,10 +481,11 @@ void SongLoader::DataReady(GstPad *, GstBuffer *buf, void *self) {
   // Append the data to the buffer
   instance->buffer_.append(reinterpret_cast<const char*>(GST_BUFFER_DATA(buf)),
                            GST_BUFFER_SIZE(buf));
-  qDebug() << "Received total" << instance->buffer_.size() << "bytes";
+  qLog(Debug) << "Received total" << instance->buffer_.size() << "bytes";
 
   if (instance->state_ == WaitingForMagic &&
-      instance->buffer_.size() >= PlaylistParser::kMagicSize) {
+      (instance->buffer_.size() >= PlaylistParser::kMagicSize ||
+       !instance->IsPipelinePlaying())) {
     // Got enough that we can test the magic
     instance->MagicReady();
   }
@@ -384,8 +531,8 @@ void SongLoader::ErrorMessageReceived(GstMessage* msg) {
   gchar* debugs;
 
   gst_message_parse_error(msg, &error, &debugs);
-  qDebug() << error->message;
-  qDebug() << debugs;
+  qLog(Error) << error->message;
+  qLog(Error) << debugs;
 
   QString message_str = error->message;
 
@@ -404,6 +551,7 @@ void SongLoader::ErrorMessageReceived(GstMessage* msg) {
 }
 
 void SongLoader::EndOfStreamReached() {
+  qLog(Debug) << Q_FUNC_INFO << state_;
   switch (state_) {
   case Finished:
     break;
@@ -428,10 +576,11 @@ void SongLoader::EndOfStreamReached() {
 }
 
 void SongLoader::MagicReady() {
+  qLog(Debug) << Q_FUNC_INFO;
   parser_ = playlist_parser_->ParserForMagic(buffer_, mime_type_);
 
   if (!parser_) {
-    qWarning() << url_.toString() << "is text, but not a recognised playlist";
+    qLog(Warning) << url_.toString() << "is text, but not a recognised playlist";
     // It doesn't look like a playlist, so just finish
     StopTypefindAsync(false);
     return;
@@ -439,7 +588,7 @@ void SongLoader::MagicReady() {
 
   // It is a playlist - we'll get more data and parse the whole thing in
   // EndOfStreamReached
-  qDebug() << "Magic says" << parser_->name();
+  qLog(Debug) << "Magic says" << parser_->name();
   if (parser_->name() == "ASX/INI" && url_.scheme() == "http") {
     // This is actually a weird MS-WMSP stream. Changing the protocol to MMS from
     // HTTP makes it playable.
@@ -448,6 +597,16 @@ void SongLoader::MagicReady() {
     StopTypefindAsync(true);
   }
   state_ = WaitingForData;
+
+  if (!IsPipelinePlaying()) {
+    EndOfStreamReached();
+  }
+}
+
+bool SongLoader::IsPipelinePlaying() {
+  GstState pipeline_state;
+  gst_element_get_state(pipeline_.get(), &pipeline_state, NULL, GST_MSECOND);
+  return pipeline_state == GST_STATE_PLAYING;
 }
 
 void SongLoader::StopTypefindAsync(bool success) {
