@@ -22,6 +22,7 @@
 #include <QUrl>
 
 #include "song.h"
+#include "musicbrainz/musicbrainzclient.h"
 
 #include <boost/shared_ptr.hpp>
 
@@ -53,6 +54,16 @@ public:
   void set_timeout(int msec) { timeout_ = msec; }
 
   Result Load(const QUrl& url);
+  // To effectively load the songs:
+  // when we call Load() on a directory, it will return WillLoadAsync, load the
+  // files with only filenames and emit LoadFinished(). When LoadFinished() is
+  // received by songloaderinserter, it will insert songs (incompletely loaded)
+  // in playlist, and call EffectiveSongsLoad() in a background thread to
+  // perform the real load of the songs. Next, UpdateItems() will be called on
+  // playlist and replace the partially-loaded items by the new ones, fully
+  // loaded.
+  void EffectiveSongsLoad();
+  Result LoadAudioCD();
 
 signals:
   void LoadFinished(bool success);
@@ -60,6 +71,8 @@ signals:
 private slots:
   void Timeout();
   void StopTypefind();
+  void AudioCDTagsLoaded(const QString& artist, const QString& album,
+                         const MusicBrainzClient::ResultList& results);
 
 private:
   enum State {
@@ -70,6 +83,7 @@ private:
   };
 
   Result LoadLocal(const QString& filename, bool block = false, bool ignore_playlists = false);
+  Result LoadLocalPartial(const QString& filename);
   void LoadLocalDirectory(const QString& filename);
   void LoadPlaylist(ParserBase* parser, const QString& filename);
   void LoadLocalDirectoryAndEmit(const QString& filename);
@@ -89,6 +103,7 @@ private:
   void ErrorMessageReceived(GstMessage* msg);
   void EndOfStreamReached();
   void MagicReady();
+  bool IsPipelinePlaying();
 
 private:
   static QSet<QString> sRawUriSchemes;
