@@ -42,7 +42,9 @@ public:
   int StartTask(const QString& name);
   void SetTaskBlocksLibraryScans(int id);
   void SetTaskProgress(int id, int progress, int max = 0);
+  void IncreaseTaskProgress(int id, int progress, int max = 0);
   void SetTaskFinished(int id);
+  int GetTaskProgress(int id);
 
 signals:
   void TasksChanged();
