@@ -20,17 +20,23 @@
 
 #include <QColor>
 #include <QLocale>
+#include <QCryptographicHash>
+#include <QSize>
 #include <QString>
+#include <QUrl>
 
 #include <boost/scoped_array.hpp>
 
 class QIODevice;
+class QMouseEvent;
+class QXmlStreamReader;
 
 namespace Utilities {
   QString PrettyTime(int seconds);
   QString PrettyTimeDelta(int seconds);
   QString PrettyTimeNanosec(qint64 nanoseconds);
   QString PrettySize(quint64 bytes);
+  QString PrettySize(const QSize& size);
   QString WordyTime(quint64 seconds);
   QString WordyTimeNanosec(qint64 nanoseconds);
   QString Ago(int seconds_since_epoch, const QLocale& locale);
@@ -45,7 +51,35 @@ namespace Utilities {
   bool CopyRecursive(const QString& source, const QString& destination);
   bool Copy(QIODevice* source, QIODevice* destination);
 
-  void OpenInFileBrowser(const QStringList& filenames);
+  void OpenInFileBrowser(const QList<QUrl>& filenames);
+
+  enum HashFunction {
+    Md5_Algo,
+    Sha256_Algo,
+  };
+  QByteArray Hmac(const QByteArray& key, const QByteArray& data, HashFunction algo);
+  QByteArray HmacMd5(const QByteArray& key, const QByteArray& data);
+  QByteArray HmacSha256(const QByteArray& key, const QByteArray& data);
+  QByteArray Sha256(const QByteArray& data);
+
+
+  // Picks an unused ephemeral port number.  Doesn't hold the port open so
+  // there's the obvious race condition
+  quint16 PickUnusedPort();
+
+
+  // Forwards a mouse event to a different widget, remapping the event's widget
+  // coordinates relative to those of the target widget.
+  void ForwardMouseEvent(const QMouseEvent* e, QWidget* target);
+
+  // Checks if the mouse event was inside the widget's rectangle.
+  bool IsMouseEventInWidget(const QMouseEvent* e, const QWidget* widget);
+
+
+  // Reads all children of the current element, and returns with the stream
+  // reader either on the EndElement for the current element, or the end of the
+  // file - whichever came first.
+  void ConsumeCurrentElement(QXmlStreamReader* reader);
 
 
   enum ConfigPath {
@@ -54,9 +88,12 @@ namespace Utilities {
     Path_NetworkCache,
     Path_GstreamerRegistry,
     Path_DefaultMusicLibrary,
-    Path_Scripts,
+    Path_LocalSpotifyBlob,
   };
   QString GetConfigPath(ConfigPath config);
+
+  // Returns the minor version of OS X (ie. 6 for Snow Leopard, 7 for Lion).
+  qint32 GetMacVersion();
 }
 
 class ScopedWCharArray {
