@@ -44,6 +44,9 @@ public:
   ~ConnectedDevice();
 
   virtual void Init() = 0;
+  // For some devices (e.g. CD devices) we don't have callbacks to be notified
+  // when something change: we can call this method to refresh device's state
+  virtual void Refresh() { }
 
   virtual TranscodeMode GetTranscodeMode() const;
   virtual Song::FileType GetTranscodeFormat() const;
