@@ -21,11 +21,17 @@
 #include "devicemanager.h"
 #include "devicestatefiltermodel.h"
 #include "filesystemdevice.h"
+#include "core/logging.h"
 #include "core/musicstorage.h"
 #include "core/taskmanager.h"
 #include "core/utilities.h"
 #include "ui/iconloader.h"
 
+#ifdef HAVE_AUDIOCD
+#  include "cddalister.h"
+#  include "cddadevice.h"
+#endif
+
 #ifdef Q_OS_DARWIN
 #  include "macdevicelister.h"
 #endif
@@ -182,6 +188,10 @@ DeviceManager::DeviceManager(BackgroundThread<Database>* database,
   connected_devices_model_ = new DeviceStateFilterModel(this);
   connected_devices_model_->setSourceModel(this);
 
+  // CD devices are detected via the DiskArbitration framework instead on Darwin.
+#if defined(HAVE_AUDIOCD) && !defined(Q_OS_DARWIN)
+  AddLister(new CddaLister);
+#endif
 #ifdef HAVE_DEVICEKIT
   AddLister(new DeviceKitLister);
 #endif
@@ -202,6 +212,10 @@ DeviceManager::DeviceManager(BackgroundThread<Database>* database,
 
   AddDeviceClass<FilesystemDevice>();
 
+#ifdef HAVE_AUDIOCD
+  AddDeviceClass<CddaDevice>();
+#endif
+
 #ifdef HAVE_LIBGPOD
   AddDeviceClass<GPodDevice>();
 #endif
@@ -240,13 +254,15 @@ QVariant DeviceManager::data(const QModelIndex& index, int role) const {
 
       if (info.size_)
         text = text + QString(" (%1)").arg(Utilities::PrettySize(info.size_));
+      if (info.device_.get())
+        info.device_->Refresh();
       return text;
     }
 
     case Qt::DecorationRole: {
       QPixmap pixmap = info.icon_.pixmap(kDeviceIconSize);
 
-      if (!info.BestBackend()->lister_) {
+      if (info.backends_.isEmpty() || !info.BestBackend()->lister_) {
         // Disconnected but remembered
         QPainter p(&pixmap);
         p.drawPixmap(kDeviceIconSize - kDeviceIconOverlaySize,
@@ -302,16 +318,19 @@ QVariant DeviceManager::data(const QModelIndex& index, int role) const {
       if (!info.device_) {
         if (info.database_id_ == -1 &&
             !info.BestBackend()->lister_->DeviceNeedsMount(info.BestBackend()->unique_id_)) {
-          boost::scoped_ptr<QMessageBox> dialog(new QMessageBox(
-              QMessageBox::Information, tr("Connect device"),
-              tr("This is the first time you have connected this device.  Clementine will now scan the device to find music files - this may take some time."),
-              QMessageBox::Cancel));
-          QPushButton* connect =
-              dialog->addButton(tr("Connect device"), QMessageBox::AcceptRole);
-          dialog->exec();
 
-          if (dialog->clickedButton() != connect)
-            return QVariant();
+          if (info.BestBackend()->lister_->AskForScan(info.BestBackend()->unique_id_)) {
+            boost::scoped_ptr<QMessageBox> dialog(new QMessageBox(
+                QMessageBox::Information, tr("Connect device"),
+                tr("This is the first time you have connected this device.  Clementine will now scan the device to find music files - this may take some time."),
+                QMessageBox::Cancel));
+            QPushButton* connect =
+                dialog->addButton(tr("Connect device"), QMessageBox::AcceptRole);
+            dialog->exec();
+
+            if (dialog->clickedButton() != connect)
+              return QVariant();
+          }
         }
 
         const_cast<DeviceManager*>(this)->Connect(index.row());
@@ -389,7 +408,7 @@ int DeviceManager::FindDeviceByUrl(const QList<QUrl>& urls) const {
 void DeviceManager::PhysicalDeviceAdded(const QString &id) {
   DeviceLister* lister = qobject_cast<DeviceLister*>(sender());
 
-  qDebug() << "Device added:" << id;
+  qLog(Info) << "Device added:" << id;
 
   // Do we have this device already?
   int i = FindDeviceById(id);
@@ -438,7 +457,7 @@ void DeviceManager::PhysicalDeviceAdded(const QString &id) {
 void DeviceManager::PhysicalDeviceRemoved(const QString &id) {
   DeviceLister* lister = qobject_cast<DeviceLister*>(sender());
 
-  qDebug() << "Device removed:" << id;
+  qLog(Info) << "Device removed:" << id;
 
   int i = FindDeviceById(id);
   if (i == -1) {
@@ -533,7 +552,7 @@ boost::shared_ptr<ConnectedDevice> DeviceManager::Connect(int row) {
   // Take the first URL that we have a handler for
   QUrl device_url;
   foreach (const QUrl& url, urls) {
-    qDebug() << "Connecting" << url;
+    qLog(Info) << "Connecting" << url;
 
     // Find a device class for this URL's scheme
     if (device_classes_.contains(url.scheme())) {
@@ -578,7 +597,7 @@ boost::shared_ptr<ConnectedDevice> DeviceManager::Connect(int row) {
   ret.reset(static_cast<ConnectedDevice*>(instance));
 
   if (!ret) {
-    qWarning() << "Could not create device for" << device_url.toString();
+    qLog(Warning) << "Could not create device for" << device_url.toString();
   } else {
     ret->Init();
 
