@@ -18,6 +18,7 @@
 #include "devicemanager.h"
 #include "gpoddevice.h"
 #include "gpodloader.h"
+#include "core/logging.h"
 #include "library/librarybackend.h"
 #include "library/librarymodel.h"
 
@@ -98,9 +99,8 @@ Itdb_Track* GPodDevice::AddTrackToITunesDb(const Song& metadata) {
 void GPodDevice::AddTrackToModel(Itdb_Track* track, const QString& prefix) {
   // Add it to our LibraryModel
   Song metadata_on_device;
-  metadata_on_device.InitFromItdb(track);
+  metadata_on_device.InitFromItdb(track, prefix);
   metadata_on_device.set_directory_id(1);
-  metadata_on_device.set_filename(prefix + metadata_on_device.filename());
   songs_to_add_ << metadata_on_device;
 }
 
@@ -114,7 +114,7 @@ bool GPodDevice::CopyToStorage(const CopyJob& job) {
   itdb_cp_track_to_ipod(track, QDir::toNativeSeparators(job.source_)
                         .toLocal8Bit().constData(), &error);
   if (error) {
-    qDebug() << "GPodDevice error:" << error->message;
+    qLog(Error) << "copying failed:" << error->message;
     emit Error(QString::fromUtf8(error->message));
     g_error_free(error);
 
@@ -139,7 +139,7 @@ void GPodDevice::WriteDatabase(bool success) {
     GError* error = NULL;
     itdb_write(db_, &error);
     if (error) {
-      qDebug() << "GPodDevice error:" << error->message;
+      qLog(Error) << "writing database failed:" << error->message;
       emit Error(QString::fromUtf8(error->message));
       g_error_free(error);
     } else {
@@ -186,7 +186,7 @@ bool GPodDevice::RemoveTrackFromITunesDb(const QString& path, const QString& rel
   }
 
   if (track == NULL) {
-    qWarning() << "Couldn't find song" << path << "in iTunesDB";
+    qLog(Warning) << "Couldn't find song" << path << "in iTunesDB";
     return false;
   }
 
@@ -208,11 +208,11 @@ bool GPodDevice::RemoveTrackFromITunesDb(const QString& path, const QString& rel
 bool GPodDevice::DeleteFromStorage(const DeleteJob& job) {
   Q_ASSERT(db_);
 
-  if (!RemoveTrackFromITunesDb(job.metadata_.filename(), url_.path()))
+  if (!RemoveTrackFromITunesDb(job.metadata_.url().toLocalFile(), url_.path()))
     return false;
 
   // Remove the file
-  if (!QFile::remove(job.metadata_.filename()))
+  if (!QFile::remove(job.metadata_.url().toLocalFile()))
     return false;
 
   // Remove it from our library model
