@@ -15,9 +15,10 @@
    along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "config.h"
 #include "macdevicelister.h"
-
 #include "mtpconnection.h"
+#include "core/logging.h"
 
 #include <CoreFoundation/CFRunLoop.h>
 #include <DiskArbitration/DiskArbitration.h>
@@ -25,6 +26,7 @@
 #include <IOKit/IOCFPlugin.h>
 #include <IOKit/usb/IOUSBLib.h>
 #include <IOKit/storage/IOMedia.h>
+#include <IOKit/storage/IOCDMedia.h>
 
 #import <AppKit/NSWorkspace.h>
 #import <Foundation/NSAutoreleasePool.h>
@@ -93,7 +95,7 @@ void MacDeviceLister::Init() {
     LIBMTP_device_entry_t* devices = NULL;
     int num = 0;
     if (LIBMTP_Get_Supported_Devices_List(&devices, &num) != 0) {
-      qWarning() << "Failed to get MTP device list";
+      qLog(Warning) << "Failed to get MTP device list";
     } else {
       for (int i = 0; i < num; ++i) {
         LIBMTP_device_entry_t device = devices[i];
@@ -143,7 +145,7 @@ void MacDeviceLister::Init() {
   if (err == KERN_SUCCESS) {
     USBDeviceAddedCallback(this, it);
   } else {
-    qWarning() << "Could not add notification on USB device connection";
+    qLog(Warning) << "Could not add notification on USB device connection";
   }
 
   err = IOServiceAddMatchingNotification(
@@ -156,7 +158,7 @@ void MacDeviceLister::Init() {
   if (err == KERN_SUCCESS) {
     USBDeviceRemovedCallback(this, it);
   } else {
-    qWarning() << "Could not add notification USB device removal";
+    qLog(Warning) << "Could not add notification USB device removal";
   }
 
   CFRunLoopSourceRef io_source = IONotificationPortGetRunLoopSource(notification_port);
@@ -251,6 +253,22 @@ NSObject* GetPropertyForDevice(io_object_t device, CFStringRef key) {
   return nil;
 }
 
+int GetUSBDeviceClass(io_object_t device) {
+  CFTypeRef interface_class = IORegistryEntrySearchCFProperty(
+      device,
+      kIOServicePlane,
+      CFSTR(kUSBInterfaceClass),
+      kCFAllocatorDefault,
+      kIORegistryIterateRecursively);
+  NSNumber* number = (NSNumber*)interface_class;
+  if (number) {
+    int ret = [number unsignedShortValue];
+    CFRelease(interface_class);
+    return ret;
+  }
+  return 0;
+}
+
 QString GetIconForDevice(io_object_t device) {
   NSDictionary* media_icon = (NSDictionary*)GetPropertyForDevice(device, CFSTR("IOMediaIcon"));
   if (media_icon) {
@@ -299,30 +317,54 @@ QString FindDeviceProperty(const QString& bsd_name, CFStringRef property) {
 
 }
 
-int MacDeviceLister::GetFreeSpace(const QUrl& url) {
+quint64 MacDeviceLister::GetFreeSpace(const QUrl& url) {
   QMutexLocker l(&libmtp_mutex_);
   MtpConnection connection(url);
   if (!connection.is_valid()) {
-    qWarning() << "Error connecting to MTP device, couldn't get device free space";
+    qLog(Warning) << "Error connecting to MTP device, couldn't get device free space";
     return -1;
   }
-  return connection.device()->storage->FreeSpaceInBytes;
+  LIBMTP_devicestorage_t* storage = connection.device()->storage;
+  quint64 free_bytes = 0;
+  while (storage) {
+    free_bytes += storage->FreeSpaceInBytes;
+    storage = storage->next;
+  }
+  return free_bytes;
 }
 
-int MacDeviceLister::GetCapacity(const QUrl& url) {
+quint64 MacDeviceLister::GetCapacity(const QUrl& url) {
   QMutexLocker l(&libmtp_mutex_);
   MtpConnection connection(url);
   if (!connection.is_valid()) {
-    qWarning() << "Error connecting to MTP device, couldn't get device capacity";
+    qLog(Warning) << "Error connecting to MTP device, couldn't get device capacity";
     return -1;
   }
-  return connection.device()->storage->MaxCapacity;
+  LIBMTP_devicestorage_t* storage = connection.device()->storage;
+  quint64 capacity_bytes = 0;
+  while (storage) {
+    capacity_bytes += storage->MaxCapacity;
+    storage = storage->next;
+  }
+  return capacity_bytes;
 }
 
 void MacDeviceLister::DiskAddedCallback(DADiskRef disk, void* context) {
   MacDeviceLister* me = reinterpret_cast<MacDeviceLister*>(context);
 
   NSDictionary* properties = (NSDictionary*)DADiskCopyDescription(disk);
+
+  NSString* kind = [properties objectForKey:(NSString*)kDADiskDescriptionMediaKindKey];
+#ifdef HAVE_AUDIOCD
+  if (kind && strcmp([kind UTF8String], kIOCDMediaClass) == 0) {
+    // CD inserted.
+    QString bsd_name = QString::fromAscii(DADiskGetBSDName(disk));
+    me->cd_devices_ << bsd_name;
+    emit me->DeviceAdded(bsd_name);
+    return;
+  }
+#endif
+
   NSURL* volume_path = 
       [[properties objectForKey:(NSString*)kDADiskDescriptionVolumePathKey] copy];
 
@@ -359,9 +401,16 @@ void MacDeviceLister::DiskRemovedCallback(DADiskRef disk, void* context) {
   MacDeviceLister* me = reinterpret_cast<MacDeviceLister*>(context);
   // We cannot access the USB tree when the disk is removed but we still get
   // the BSD disk name.
+
+  QString bsd_name = QString::fromAscii(DADiskGetBSDName(disk));
+  if (me->cd_devices_.remove(bsd_name)) {
+    emit me->DeviceRemoved(bsd_name);
+    return;
+  }
+
   for (QMap<QString, QString>::iterator it = me->current_devices_.begin();
        it != me->current_devices_.end(); ++it) {
-    if (it.value() == QString::fromLocal8Bit(DADiskGetBSDName(disk))) {
+    if (it.value() == bsd_name) {
       emit me->DeviceRemoved(it.key());
       me->current_devices_.erase(it);
       break;
@@ -427,6 +476,9 @@ void MacDeviceLister::USBDeviceAddedCallback(void* refcon, io_iterator_t it) {
       NSString* product = (NSString*)GetPropertyForDevice(object, CFSTR(kUSBProductString));
       NSNumber* vendor_id = (NSNumber*)GetPropertyForDevice(object, CFSTR(kUSBVendorID));
       NSNumber* product_id = (NSNumber*)GetPropertyForDevice(object, CFSTR(kUSBProductID));
+      int interface_class = GetUSBDeviceClass(object);
+      qLog(Debug) << "Interface class:" << interface_class;
+
       QString serial = GetSerialForMTPDevice(object);
 
       MTPDevice device;
@@ -439,8 +491,14 @@ void MacDeviceLister::USBDeviceAddedCallback(void* refcon, io_iterator_t it) {
       device.bus = -1;
       device.address = -1;
 
-      if (device.vendor_id == 0x5ac) {
-        // I think we can safely skip Apple products.
+      if (device.vendor_id == kAppleVendorID || // I think we can safely skip Apple products.
+          // Blacklist ilok2 as this probe may be breaking it.
+          (device.vendor_id == 0x088e && device.product_id == 0x5036) ||
+          // Blacklist eLicenser
+          (device.vendor_id == 0x0819 && device.product_id == 0x0101) ||
+          // Skip HID devices and hubs.
+          interface_class == kUSBHIDInterfaceClass ||
+          interface_class == kUSBHubClass) {
         continue;
       }
 
@@ -571,7 +629,7 @@ void MacDeviceLister::USBDeviceRemovedCallback(void* refcon, io_iterator_t it) {
       device.product = QString::fromUtf8([product UTF8String]);
       device.vendor_id = [vendor_id unsignedShortValue];
       device.product_id = [product_id unsignedShortValue];
-
+  
       me->RemovedMTPDevice(serial);
     }
   }
@@ -599,6 +657,10 @@ bool IsMTPSerial(const QString& serial) {
   return serial.startsWith("MTP");
 }
 
+bool MacDeviceLister::IsCDDevice(const QString& serial) const {
+  return cd_devices_.contains(serial);
+}
+
 QString MacDeviceLister::MakeFriendlyName(const QString& serial) {
   if (IsMTPSerial(serial)) {
     const MTPDevice& device = mtp_devices_[serial];
@@ -608,12 +670,24 @@ QString MacDeviceLister::MakeFriendlyName(const QString& serial) {
       return device.vendor + " " + device.product;
     }
   }
-  QString bsd_name = current_devices_[serial];
+
+  QString bsd_name = IsCDDevice(serial) ? *cd_devices_.find(serial) : current_devices_[serial];
   DASessionRef session = DASessionCreate(kCFAllocatorDefault);
   DADiskRef disk = DADiskCreateFromBSDName(
       kCFAllocatorDefault, session, bsd_name.toAscii().constData());
 
+  if (IsCDDevice(serial)) {
+    NSDictionary* properties = (NSDictionary*)DADiskCopyDescription(disk);
+    NSString* device_name = (NSString*)[properties objectForKey:
+        (NSString*)kDADiskDescriptionMediaNameKey];
+
+    CFRelease(disk);
+    CFRelease(session);
+    return QString::fromUtf8([device_name UTF8String]);
+  }
+
   io_object_t device = DADiskCopyIOMedia(disk);
+
   QString vendor = GetUSBRegistryEntryString(device, CFSTR(kUSBVendorString));
   QString product = GetUSBRegistryEntryString(device, CFSTR(kUSBProductString));
   IOObjectRelease(device);
@@ -641,6 +715,10 @@ QList<QUrl> MacDeviceLister::MakeDeviceUrls(const QString& serial) {
     return QList<QUrl>() << url;
   }
 
+  if (IsCDDevice(serial)) {
+    return QList<QUrl>() << QString("cdda:///dev/r" + serial);
+  }
+
   QString bsd_name = current_devices_[serial];
   DASessionRef session = DASessionCreate(kCFAllocatorDefault);
   DADiskRef disk = DADiskCreateFromBSDName(
@@ -667,6 +745,11 @@ QVariantList MacDeviceLister::DeviceIcons(const QString& serial) {
   if (IsMTPSerial(serial)) {
     return QVariantList();
   }
+
+  if (IsCDDevice(serial)) {
+    return QVariantList() << "media-optical";
+  }
+
   QString bsd_name = current_devices_[serial];
   DASessionRef session = DASessionCreate(kCFAllocatorDefault);
   DADiskRef disk = DADiskCreateFromBSDName(
@@ -767,6 +850,10 @@ quint64 MacDeviceLister::DeviceFreeSpace(const QString& serial){
 
 QVariantMap MacDeviceLister::DeviceHardwareInfo(const QString& serial){return QVariantMap();}
 
+bool MacDeviceLister::AskForScan(const QString& serial) const {
+  return !IsCDDevice(serial);
+}
+
 void MacDeviceLister::UnmountDevice(const QString& serial) {
   if (IsMTPSerial(serial)) return;
 
@@ -782,7 +869,7 @@ void MacDeviceLister::UnmountDevice(const QString& serial) {
 void MacDeviceLister::DiskUnmountCallback(
     DADiskRef disk, DADissenterRef dissenter, void* context) {
   if (dissenter) {
-    qWarning() << "Another app blocked the unmount";
+    qLog(Warning) << "Another app blocked the unmount";
   } else {
     DiskRemovedCallback(disk, context);
   }
