@@ -20,6 +20,7 @@
 //License:   See COPYING
 
 #include "enginebase.h"
+#include "core/timeconstants.h"
 
 #include <cmath>
 
@@ -29,10 +30,14 @@ const char* Engine::Base::kSettingsGroup = "Player";
 
 Engine::Base::Base()
   : volume_(50),
+    beginning_nanosec_(0),
+    end_nanosec_(0),
     scope_(kScopeSize),
     fadeout_enabled_(true),
-    fadeout_duration_(2000),
+    fadeout_duration_nanosec_(2 * kNsecPerSec), // 2s
     crossfade_enabled_(true),
+    autocrossfade_enabled_(false),
+    crossfade_same_album_(false),
     next_background_stream_id_(0),
     about_to_end_emitted_(false)
 {
@@ -41,8 +46,15 @@ Engine::Base::Base()
 Engine::Base::~Base() {
 }
 
-bool Engine::Base::Load(const QUrl &url, TrackChangeType) {
+bool Engine::Base::Load(const QUrl& url, TrackChangeFlags,
+                        bool force_stop_at_end,
+                        quint64 beginning_nanosec, qint64 end_nanosec) {
+  Q_UNUSED(force_stop_at_end);
+
   url_ = url;
+  beginning_nanosec_ = beginning_nanosec;
+  end_nanosec_ = end_nanosec;
+
   about_to_end_emitted_ = false;
   return true;
 }
@@ -63,9 +75,10 @@ void Engine::Base::ReloadSettings() {
   s.beginGroup(kSettingsGroup);
 
   fadeout_enabled_ = s.value("FadeoutEnabled", true).toBool();
-  fadeout_duration_ = s.value("FadeoutDuration", 2000).toInt();
+  fadeout_duration_nanosec_ = s.value("FadeoutDuration", 2000).toLongLong() * kNsecPerMsec;
   crossfade_enabled_ = s.value("CrossfadeEnabled", true).toBool();
   autocrossfade_enabled_ = s.value("AutoCrossfadeEnabled", false).toBool();
+  crossfade_same_album_ = !s.value("NoCrossfadeSameAlbum", true).toBool();
 }
 
 void Engine::Base::EmitAboutToEnd() {
@@ -80,8 +93,11 @@ int Engine::Base::AddBackgroundStream(const QUrl& url) {
   return -1;
 }
 
-bool Engine::Base::Play(const QUrl& u, TrackChangeType c) {
-  if (!Load(u, c))
+bool Engine::Base::Play(const QUrl& u, TrackChangeFlags c,
+                        bool force_stop_at_end,
+                        quint64 beginning_nanosec, qint64 end_nanosec) {
+  if (!Load(u, c, force_stop_at_end, beginning_nanosec, end_nanosec))
     return false;
-  return Play();
+
+  return Play(0);
 }
