@@ -43,28 +43,44 @@ class Base : public QObject, boost::noncopyable {
   virtual ~Base();
 
   virtual bool Init() = 0;
-  virtual bool CanDecode(const QUrl &url) = 0;
 
-  virtual void StartPreloading(const QUrl&) {}
-  virtual bool Play(uint offset = 0) = 0;
+  virtual void StartPreloading(const QUrl&, bool, qint64, qint64) {}
+  virtual bool Play(quint64 offset_nanosec) = 0;
   virtual void Stop() = 0;
   virtual void Pause() = 0;
   virtual void Unpause() = 0;
-  virtual void Seek( uint ms ) = 0;
+  virtual void Seek(quint64 offset_nanosec) = 0;
 
   virtual int AddBackgroundStream(const QUrl& url);
   virtual void StopBackgroundStream(int id) {}
   virtual void SetBackgroundStreamVolume(int id, int volume) {}
 
   virtual State state() const = 0;
-  virtual uint position() const = 0;
-  virtual uint length() const { return 0; }
+  virtual qint64 position_nanosec() const = 0;
+  virtual qint64 length_nanosec() const = 0;
 
-  // Helpers
-  virtual bool Load(const QUrl &url, TrackChangeType change);
-  bool Play(const QUrl &u, TrackChangeType c);
-  void SetVolume( uint value );
+  // Subclasses should respect given markers (beginning and end) which are
+  // in miliseconds.
+  virtual bool Load(const QUrl& url, TrackChangeFlags change,
+                    bool force_stop_at_end,
+                    quint64 beginning_nanosec, qint64 end_nanosec);
+  // Sets new values for the beginning and end markers of the currently playing
+  // song.
+  // This doesn't change the state of engine or the stream's current position.
+  virtual void RefreshMarkers(quint64 beginning_nanosec, qint64 end_nanosec) {
+    beginning_nanosec_ = beginning_nanosec;
+    end_nanosec_ = end_nanosec;
+  }
 
+  // Plays a media stream represented with the URL 'u' from the given 'beginning'
+  // to the given 'end' (usually from 0 to a song's length). Both markers
+  // should be passed in nanoseconds. 'end' can be negative, indicating that the
+  // real length of 'u' stream is unknown.
+  bool Play(const QUrl& u, TrackChangeFlags c,
+            bool force_stop_at_end,
+            quint64 beginning_nanosec, qint64 end_nanosec);
+
+  void SetVolume(uint value);
 
   // Simple accessors
   inline uint volume() const { return volume_; }
@@ -72,6 +88,7 @@ class Base : public QObject, boost::noncopyable {
   bool is_fadeout_enabled() const { return fadeout_enabled_; }
   bool is_crossfade_enabled() const { return crossfade_enabled_; }
   bool is_autocrossfade_enabled() const { return autocrossfade_enabled_; }
+  bool crossfade_same_album() const { return crossfade_same_album_; }
 
   static const char* kSettingsGroup;
   static const int kScopeSize = 1024;
@@ -89,11 +106,22 @@ class Base : public QObject, boost::noncopyable {
 
   void TrackEnded();
 
+  void FadeoutFinishedSignal();
+
   void StatusText(const QString&);
   void Error(const QString&);
 
+  // Emitted when Engine was unable to play a song with the given QUrl.
+  void InvalidSongRequested(const QUrl&);
+  // Emitted when Engine successfully started playing a song with the 
+  // given QUrl.
+  void ValidSongRequested(const QUrl&);
+
   void MetaData(const Engine::SimpleMetaBundle&);
 
+  // Signals that the engine's state has changed (a stream was stopped for example).
+  // Always use the state from event, because it's not guaranteed that immediate
+  // subsequent call to state() won't return a stale value.
   void StateChanged(Engine::State);
 
  protected:
@@ -104,14 +132,17 @@ class Base : public QObject, boost::noncopyable {
   void EmitAboutToEnd();
 
  protected:
-  uint  volume_;
-  QUrl  url_;
+  uint volume_;
+  quint64 beginning_nanosec_;
+  qint64 end_nanosec_;
+  QUrl url_;
   Scope scope_;
 
   bool fadeout_enabled_;
-  int fadeout_duration_;
+  qint64 fadeout_duration_nanosec_;
   bool crossfade_enabled_;
   bool autocrossfade_enabled_;
+  bool crossfade_same_album_;
   int next_background_stream_id_;
 
  private:
