@@ -17,12 +17,20 @@
 
 #include <limits>
 
-#include "gstelementdeleter.h"
-#include "gstenginepipeline.h"
-#include "gstengine.h"
 #include "bufferconsumer.h"
+#include "config.h"
+#include "gstelementdeleter.h"
+#include "gstengine.h"
+#include "gstenginepipeline.h"
+#include "core/logging.h"
+#include "core/utilities.h"
+#include "internet/internetmodel.h"
+
+#ifdef HAVE_SPOTIFY
+#  include "internet/spotifyserver.h"
+#  include "internet/spotifyservice.h"
+#endif
 
-#include <QDebug>
 #include <QtConcurrentRun>
 
 const int GstEnginePipeline::kGstStateTimeoutNanosecs = 10000000;
@@ -44,6 +52,7 @@ GstEnginePipeline::GstEnginePipeline(GstEngine* engine)
     sink_(GstEngine::kAutoSink),
     segment_start_(0),
     segment_start_received_(false),
+    emit_track_ended_on_segment_start_(false),
     eq_enabled_(false),
     eq_preamp_(0),
     rg_enabled_(false),
@@ -65,6 +74,7 @@ GstEnginePipeline::GstEnginePipeline(GstEngine* engine)
     pipeline_(NULL),
     uridecodebin_(NULL),
     audiobin_(NULL),
+    queue_(NULL),
     audioconvert_(NULL),
     rgvolume_(NULL),
     rglimiter_(NULL),
@@ -102,12 +112,11 @@ void GstEnginePipeline::set_buffer_duration_nanosec(qint64 buffer_duration_nanos
 bool GstEnginePipeline::ReplaceDecodeBin(GstElement* new_bin) {
   if (!new_bin) return false;
 
-  // Destroy the old one, if any
+  // Destroy the old elements if they are set
+  // Note that the caller to this function MUST schedule the old uridecodebin_
+  // for deletion in the main thread.
   if (uridecodebin_) {
     gst_bin_remove(GST_BIN(pipeline_), uridecodebin_);
-
-    // Note that the caller to this function MUST schedule the old bin for
-    // deletion in the main thread
   }
 
   uridecodebin_ = new_bin;
@@ -120,13 +129,48 @@ bool GstEnginePipeline::ReplaceDecodeBin(GstElement* new_bin) {
 }
 
 bool GstEnginePipeline::ReplaceDecodeBin(const QUrl& url) {
-  GstElement* new_bin = engine_->CreateElement("uridecodebin");
-  g_object_set(G_OBJECT(new_bin), "uri", url.toEncoded().constData(), NULL);
-  g_object_set(G_OBJECT(new_bin), "buffer-duration", buffer_duration_nanosec_, NULL);
-  g_object_set(G_OBJECT(new_bin), "download", true, NULL);
-  g_object_set(G_OBJECT(new_bin), "use-buffering", true, NULL);
-  g_signal_connect(G_OBJECT(new_bin), "drained", G_CALLBACK(SourceDrainedCallback), this);
-  g_signal_connect(G_OBJECT(new_bin), "pad-added", G_CALLBACK(NewPadCallback), this);
+  GstElement* new_bin = NULL;
+
+  if (url.scheme() == "spotify") {
+    #ifdef HAVE_SPOTIFY
+      new_bin = gst_bin_new("spotify_bin");
+
+      // Create elements
+      GstElement* src = engine_->CreateElement("tcpserversrc", new_bin);
+      GstElement* gdp = engine_->CreateElement("gdpdepay", new_bin);
+      if (!src || !gdp)
+        return false;
+
+      // Pick a port number
+      const int port = Utilities::PickUnusedPort();
+      g_object_set(G_OBJECT(src), "host", "127.0.0.1", NULL);
+      g_object_set(G_OBJECT(src), "port", port, NULL);
+
+      // Link the elements
+      gst_element_link(src, gdp);
+
+      // Add a ghost pad
+      GstPad* pad = gst_element_get_static_pad(gdp, "src");
+      gst_element_add_pad(GST_ELEMENT(new_bin), gst_ghost_pad_new("src", pad));
+      gst_object_unref(GST_OBJECT(pad));
+
+      // Tell spotify to start sending data to us.
+      InternetModel::Service<SpotifyService>()->server()->StartPlaybackLater(url.toString(), port);
+    #else // HAVE_SPOTIFY
+      qLog(Error) << "Tried to play a spotify:// url, but spotify support is not compiled in";
+      return false;
+    #endif
+  } else {
+    new_bin = engine_->CreateElement("uridecodebin");
+    g_object_set(G_OBJECT(new_bin), "uri", url.toEncoded().constData(), NULL);
+    g_object_set(G_OBJECT(new_bin), "buffer-duration", buffer_duration_nanosec_, NULL);
+    g_object_set(G_OBJECT(new_bin), "download", true, NULL);
+    g_object_set(G_OBJECT(new_bin), "use-buffering", true, NULL);
+    g_signal_connect(G_OBJECT(new_bin), "drained", G_CALLBACK(SourceDrainedCallback), this);
+    g_signal_connect(G_OBJECT(new_bin), "pad-added", G_CALLBACK(NewPadCallback), this);
+    g_signal_connect(G_OBJECT(new_bin), "notify::source", G_CALLBACK(SourceSetupCallback), this);
+  }
+
   return ReplaceDecodeBin(new_bin);
 }
 
@@ -140,7 +184,7 @@ GstElement* GstEnginePipeline::CreateDecodeBinFromString(const char* pipeline) {
     int code = error->code;
     g_error_free(error);
 
-    qWarning() << message;
+    qLog(Warning) << message;
     emit Error(id(), message, domain, code);
 
     return NULL;
@@ -155,32 +199,72 @@ bool GstEnginePipeline::Init() {
   //   uri decode bin -> audio bin
   // The uri decode bin is a gstreamer builtin that automatically picks the
   // right type of source and decoder for the URI.
+
   // The audio bin gets created here and contains:
-  //   audioconvert -> rgvolume -> rglimiter -> equalizer_preamp -> equalizer ->
-  //   volume -> audioscale -> audioconvert -> audiosink
+  //   queue ! audioconvert ! <caps32>
+  //         ! ( rgvolume ! rglimiter ! audioconvert2 ) ! tee
+  // rgvolume and rglimiter are only created when replaygain is enabled.
+
+  // After the tee the pipeline splits.  One split is converted to 16-bit int
+  // samples for the scope, the other is kept as float32 and sent to the
+  // speaker.
+  //   tee1 ! probe_queue ! probe_converter ! <caps16> ! probe_sink
+  //   tee2 ! audio_queue ! equalizer_preamp ! equalizer ! volume ! audioscale
+  //        ! convert ! audiosink
 
   // Audio bin
   audiobin_ = gst_bin_new("audiobin");
   gst_bin_add(GST_BIN(pipeline_), audiobin_);
 
+  // Create the sink
   if (!(audiosink_ = engine_->CreateElement(sink_, audiobin_)))
     return false;
 
   if (GstEngine::DoesThisSinkSupportChangingTheOutputDeviceToAUserEditableString(sink_) && !device_.isEmpty())
     g_object_set(G_OBJECT(audiosink_), "device", device_.toUtf8().constData(), NULL);
 
-  if (!(equalizer_preamp_ = engine_->CreateElement("volume", audiobin_))) { return false; }
-  if (!(equalizer_ = engine_->CreateElement("equalizer-nbands", audiobin_))) { return false; }
-  if (!(audioconvert_ = engine_->CreateElement("audioconvert", audiobin_))) { return false; }
-  if (!(volume_ = engine_->CreateElement("volume", audiobin_))) { return false; }
-  if (!(audioscale_ = engine_->CreateElement("audioresample", audiobin_))) { return false; }
-  GstElement* scope_element = audioconvert_;
+  // Create all the other elements
+  GstElement *tee, *probe_queue, *probe_converter, *probe_sink, *audio_queue,
+             *convert;
+
+  queue_            = engine_->CreateElement("queue",            audiobin_);
+  audioconvert_     = engine_->CreateElement("audioconvert",     audiobin_);
+  tee               = engine_->CreateElement("tee",              audiobin_);
+
+  probe_queue       = engine_->CreateElement("queue",            audiobin_);
+  probe_converter   = engine_->CreateElement("audioconvert",     audiobin_);
+  probe_sink        = engine_->CreateElement("fakesink",         audiobin_);
+
+  audio_queue       = engine_->CreateElement("queue",            audiobin_);
+  equalizer_preamp_ = engine_->CreateElement("volume",           audiobin_);
+  equalizer_        = engine_->CreateElement("equalizer-nbands", audiobin_);
+  volume_           = engine_->CreateElement("volume",           audiobin_);
+  audioscale_       = engine_->CreateElement("audioresample",    audiobin_);
+  convert           = engine_->CreateElement("audioconvert",     audiobin_);
+
+  if (!queue_ || !audioconvert_ || !tee || !probe_queue || !probe_converter ||
+      !probe_sink || !audio_queue || !equalizer_preamp_ || !equalizer_ ||
+      !volume_ || !audioscale_ || !convert) {
+    return false;
+  }
+
+  // Create the replaygain elements if it's enabled.  event_probe is the
+  // audioconvert element we attach the probe to, which will change depending
+  // on whether replaygain is enabled.  convert_sink is the element after the
+  // first audioconvert, which again will change.
+  GstElement* event_probe = audioconvert_;
+  GstElement* convert_sink = tee;
 
   if (rg_enabled_) {
-    if (!(rgvolume_ = engine_->CreateElement("rgvolume", audiobin_))) { return false; }
-    if (!(rglimiter_ = engine_->CreateElement("rglimiter", audiobin_))) { return false; }
-    if (!(audioconvert2_ = engine_->CreateElement("audioconvert", audiobin_))) { return false; }
-    scope_element = audioconvert2_;
+    rgvolume_      = engine_->CreateElement("rgvolume",     audiobin_);
+    rglimiter_     = engine_->CreateElement("rglimiter",    audiobin_);
+    audioconvert2_ = engine_->CreateElement("audioconvert", audiobin_);
+    event_probe = audioconvert2_;
+    convert_sink = rgvolume_;
+
+    if (!rgvolume_ || !rglimiter_ || !audioconvert2_) {
+      return false;
+    }
 
     // Set replaygain settings
     g_object_set(G_OBJECT(rgvolume_), "album-mode", rg_mode_, NULL);
@@ -188,18 +272,22 @@ bool GstEnginePipeline::Init() {
     g_object_set(G_OBJECT(rglimiter_), "enabled", int(rg_compression_), NULL);
   }
 
-  GstPad* pad = gst_element_get_pad(audioconvert_, "sink");
+  // Create a pad on the outside of the audiobin and connect it to the pad of
+  // the first element.
+  GstPad* pad = gst_element_get_pad(queue_, "sink");
   gst_element_add_pad(audiobin_, gst_ghost_pad_new("sink", pad));
   gst_object_unref(pad);
 
   // Add a data probe on the src pad of the audioconvert element for our scope.
   // We do it here because we want pre-equalized and pre-volume samples
-  // so that our visualization are not affected by them
-  pad = gst_element_get_pad(scope_element, "src");
-  gst_pad_add_buffer_probe(pad, G_CALLBACK(HandoffCallback), this);
+  // so that our visualization are not be affected by them.
+  pad = gst_element_get_pad(event_probe, "src");
   gst_pad_add_event_probe(pad, G_CALLBACK(EventHandoffCallback), this);
-  gst_object_unref (pad);
+  gst_object_unref(pad);
 
+  // Configure the fakesink properly
+  g_object_set(G_OBJECT(probe_sink), "sync", TRUE, NULL);
+  
   // Set the equalizer bands
   g_object_set(G_OBJECT(equalizer_), "num-bands", 10, NULL);
 
@@ -217,26 +305,65 @@ bool GstEnginePipeline::Init() {
     g_object_unref(G_OBJECT(band));
   }
 
-  // Ensure we get the right type out of audioconvert for our scope
-  GstCaps* caps = gst_caps_new_simple ("audio/x-raw-int",
+  // Set the buffer duration.  We set this on the queue as well as on the
+  // decode bin (in ReplaceDecodeBin()) because setting it on the decode bin
+  // only affects network sources.
+  g_object_set(G_OBJECT(queue_), "max-size-time", buffer_duration_nanosec_, NULL);
+
+  gst_element_link(queue_, audioconvert_);
+  
+  // Create the caps to put in each path in the tee.  The scope path gets 16-bit
+  // ints and the audiosink path gets float32.
+  GstCaps* caps16 = gst_caps_new_simple ("audio/x-raw-int",
       "width", G_TYPE_INT, 16,
       "signed", G_TYPE_BOOLEAN, true,
       NULL);
-  gst_element_link_filtered(scope_element, equalizer_preamp_, caps);
-  gst_caps_unref(caps);
+  GstCaps* caps32 = gst_caps_new_simple ("audio/x-raw-float",
+      "width", G_TYPE_INT, 32,
+      NULL);
 
-  // Add an extra audioconvert at the end as osxaudiosink supports only one format.
-  GstElement* convert = engine_->CreateElement("audioconvert", audiobin_);
-  if (!convert) { return false; }
-  if (rg_enabled_)
-    gst_element_link_many(audioconvert_, rgvolume_, rglimiter_, audioconvert2_, NULL);
-  gst_element_link_many(equalizer_preamp_, equalizer_, volume_, audioscale_, convert, audiosink_, NULL);
+  // Link the elements with special caps
+  gst_element_link_filtered(probe_converter, probe_sink, caps16);
+  gst_element_link_filtered(audioconvert_, convert_sink, caps32);
+  gst_caps_unref(caps16);
+  gst_caps_unref(caps32);
 
+  // Link the outputs of tee to the queues on each path.
+  gst_pad_link(gst_element_get_request_pad(tee, "src%d"), gst_element_get_pad(probe_queue, "sink"));
+  gst_pad_link(gst_element_get_request_pad(tee, "src%d"), gst_element_get_pad(audio_queue, "sink"));
+
+  // Link replaygain elements if enabled.
+  if (rg_enabled_) {
+    gst_element_link_many(rgvolume_, rglimiter_, audioconvert2_, tee, NULL);
+  }
+
+  // Link everything else.
+  gst_element_link(probe_queue, probe_converter);
+  gst_element_link_many(audio_queue, equalizer_preamp_, equalizer_, volume_,
+                        audioscale_, convert, audiosink_, NULL);
+
+  // Add probes and handlers.
+  gst_pad_add_buffer_probe(gst_element_get_pad(probe_converter, "src"), G_CALLBACK(HandoffCallback), this);
   gst_bus_set_sync_handler(gst_pipeline_get_bus(GST_PIPELINE(pipeline_)), BusCallbackSync, this);
   bus_cb_id_ = gst_bus_add_watch(gst_pipeline_get_bus(GST_PIPELINE(pipeline_)), BusCallback, this);
+
+  MaybeLinkDecodeToAudio();
+
   return true;
 }
 
+void GstEnginePipeline::MaybeLinkDecodeToAudio() {
+  if (!uridecodebin_ || !audiobin_)
+    return;
+
+  GstPad* pad = gst_element_get_static_pad(uridecodebin_, "src");
+  if (!pad)
+    return;
+
+  gst_object_unref(pad);
+  gst_element_link(uridecodebin_, audiobin_);
+}
+
 bool GstEnginePipeline::InitFromString(const QString& pipeline) {
   pipeline_ = gst_pipeline_new("pipeline");
 
@@ -253,12 +380,23 @@ bool GstEnginePipeline::InitFromString(const QString& pipeline) {
 
 bool GstEnginePipeline::InitFromUrl(const QUrl &url, qint64 end_nanosec) {
   pipeline_ = gst_pipeline_new("pipeline");
-
-  url_ = url;
+  
+  if (url.scheme() == "cdda" && !url.path().isEmpty()) {
+    // Currently, Gstreamer can't handle input CD devices inside cdda URL. So
+    // we handle them ourselve: we extract the track number and re-create an
+    // URL with only cdda:// + the track number (which can be handled by
+    // Gstreamer). We keep the device in mind, and we will set it later using
+    // SourceSetupCallback
+    QStringList path = url.path().split('/');
+    url_ = QUrl(QString("cdda://%1").arg(path.takeLast()));
+    source_device_ = path.join("/");
+  } else {
+    url_ = url;
+  }
   end_offset_nanosec_ = end_nanosec;
 
   // Decode bin
-  if (!ReplaceDecodeBin(url)) return false;
+  if (!ReplaceDecodeBin(url_)) return false;
 
   return Init();
 }
@@ -277,7 +415,9 @@ GstEnginePipeline::~GstEnginePipeline() {
 gboolean GstEnginePipeline::BusCallback(GstBus*, GstMessage* msg, gpointer self) {
   GstEnginePipeline* instance = reinterpret_cast<GstEnginePipeline*>(self);
 
-  switch ( GST_MESSAGE_TYPE(msg)) {
+  qLog(Debug) << instance->id() << "bus message" << GST_MESSAGE_TYPE_NAME(msg);
+
+  switch (GST_MESSAGE_TYPE(msg)) {
     case GST_MESSAGE_ERROR:
       instance->ErrorMessageReceived(msg);
       break;
@@ -300,6 +440,8 @@ gboolean GstEnginePipeline::BusCallback(GstBus*, GstMessage* msg, gpointer self)
 GstBusSyncReply GstEnginePipeline::BusCallbackSync(GstBus*, GstMessage* msg, gpointer self) {
   GstEnginePipeline* instance = reinterpret_cast<GstEnginePipeline*>(self);
 
+  qLog(Debug) << instance->id() << "sync bus message" << GST_MESSAGE_TYPE_NAME(msg);
+
   switch (GST_MESSAGE_TYPE(msg)) {
     case GST_MESSAGE_EOS:
       emit instance->EndOfStreamReached(instance->id(), false);
@@ -361,7 +503,8 @@ void GstEnginePipeline::ErrorMessageReceived(GstMessage* msg) {
     return;
   }
 
-  qDebug() << debugstr;
+  qLog(Error) << id() << debugstr;
+
   emit Error(id(), message, domain, code);
 }
 
@@ -424,7 +567,7 @@ void GstEnginePipeline::NewPadCallback(GstElement*, GstPad* pad, gpointer self)
   GstPad* const audiopad = gst_element_get_pad(instance->audiobin_, "sink");
 
   if (GST_PAD_IS_LINKED(audiopad)) {
-    qDebug() << "audiopad is already linked. Unlinking old pad.";
+    qLog(Warning) << instance->id() << "audiopad is already linked, unlinking old pad";
     gst_pad_unlink(audiopad, GST_PAD_PEER(audiopad));
   }
 
@@ -493,6 +636,8 @@ bool GstEnginePipeline::HandoffCallback(GstPad*, GstBuffer* buf, gpointer self)
 bool GstEnginePipeline::EventHandoffCallback(GstPad*, GstEvent* e, gpointer self) {
   GstEnginePipeline* instance = reinterpret_cast<GstEnginePipeline*>(self);
 
+  qLog(Debug) << instance->id() << "event" << GST_EVENT_TYPE_NAME(e);
+
   if (GST_EVENT_TYPE(e) == GST_EVENT_NEWSEGMENT && !instance->segment_start_received_) {
     // The segment start time is used to calculate the proper offset of data
     // buffers from the start of the stream
@@ -500,6 +645,11 @@ bool GstEnginePipeline::EventHandoffCallback(GstPad*, GstEvent* e, gpointer self
     gst_event_parse_new_segment(e, NULL, NULL, NULL, &start, NULL, NULL);
     instance->segment_start_ = start;
     instance->segment_start_received_ = true;
+
+    if (instance->emit_track_ended_on_segment_start_) {
+      instance->emit_track_ended_on_segment_start_ = false;
+      emit instance->EndOfStreamReached(instance->id(), true);
+    }
   }
 
   return true;
@@ -513,6 +663,33 @@ void GstEnginePipeline::SourceDrainedCallback(GstURIDecodeBin* bin, gpointer sel
   }
 }
 
+void GstEnginePipeline::SourceSetupCallback(GstURIDecodeBin* bin, GParamSpec *pspec, gpointer self) {
+  GstEnginePipeline* instance = reinterpret_cast<GstEnginePipeline*>(self);
+  GstElement* element;
+  g_object_get(bin, "source", &element, NULL);
+  if (element &&
+      g_object_class_find_property(G_OBJECT_GET_CLASS(element), "device") &&
+      !instance->source_device().isEmpty()) {
+    // Gstreamer is not able to handle device in URL (refering to Gstreamer
+    // documentation, this might be added in the future). Despite that, for now
+    // we include device inside URL: we decompose it during Init and set device
+    // here, when this callback is called.
+    g_object_set(element, "device", instance->source_device().toLocal8Bit().constData(), NULL);
+  }
+  if (element &&
+      g_object_class_find_property(G_OBJECT_GET_CLASS(element), "extra-headers") &&
+      instance->url().host().contains("grooveshark")) {
+    // Grooveshark streaming servers will answer with a 400 error 'Bad request'
+    // if we don't specify 'Range' field in HTTP header.
+    // Maybe it could be usefull in some other cases, but for now, I prefer to
+    // keep this grooveshark specific.
+    GstStructure* headers;
+    headers = gst_structure_new("extra-headers", "Range", G_TYPE_STRING, "bytes=0-", NULL);
+    g_object_set(element, "extra-headers", headers, NULL);
+    gst_structure_free(headers);
+  }
+}
+
 void GstEnginePipeline::TransitionToNext() {
   GstElement* old_decode_bin = uridecodebin_;
 
@@ -520,6 +697,7 @@ void GstEnginePipeline::TransitionToNext() {
 
   ReplaceDecodeBin(next_url_);
   gst_element_set_state(uridecodebin_, GST_STATE_PLAYING);
+  MaybeLinkDecodeToAudio();
 
   url_ = next_url_;
   end_offset_nanosec_ = next_end_offset_nanosec_;
@@ -527,8 +705,10 @@ void GstEnginePipeline::TransitionToNext() {
   next_beginning_offset_nanosec_ = 0;
   next_end_offset_nanosec_ = 0;
 
-  // This just tells the UI that we've moved on to the next song
-  emit EndOfStreamReached(id(), true);
+  // This function gets called when the source has been drained, even if the
+  // song hasn't finished playing yet.  We'll get a new segment when it really
+  // does finish, so emit TrackEnded then.
+  emit_track_ended_on_segment_start_ = true;
 
   // This has to happen *after* the gst_element_set_state on the new bin to
   // fix an occasional race condition deadlock.
