@@ -89,7 +89,8 @@ void LibraryBackend::LoadDirectories() {
   }
 }
 
-void LibraryBackend::ChangeDirPath(int id, const QString &new_path) {
+void LibraryBackend::ChangeDirPath(int id, const QString& old_path,
+                                   const QString& new_path) {
   QMutexLocker l(db_->Mutex());
   QSqlDatabase db(db_->Connect());
   ScopedTransaction t(&db);
@@ -101,12 +102,15 @@ void LibraryBackend::ChangeDirPath(int id, const QString &new_path) {
   q.exec();
   if (db_->CheckErrors(q)) return;
 
-  const int path_len = new_path.length();
+  const QByteArray old_url = QUrl::fromLocalFile(old_path).toEncoded();
+  const QByteArray new_url = QUrl::fromLocalFile(new_path).toEncoded();
+
+  const int path_len = old_url.length();
 
   // Do the subdirs table
   q = QSqlQuery(QString("UPDATE %1 SET path=:path || substr(path, %2)"
                         " WHERE directory=:id").arg(subdirs_table_).arg(path_len), db);
-  q.bindValue(":path", new_path);
+  q.bindValue(":path", new_url);
   q.bindValue(":id", id);
   q.exec();
   if (db_->CheckErrors(q)) return;
@@ -114,7 +118,7 @@ void LibraryBackend::ChangeDirPath(int id, const QString &new_path) {
   // Do the songs table
   q = QSqlQuery(QString("UPDATE %1 SET filename=:path || substr(filename, %2)"
                         " WHERE directory=:id").arg(songs_table_).arg(path_len), db);
-  q.bindValue(":path", new_path);
+  q.bindValue(":path", new_url);
   q.bindValue(":id", id);
   q.exec();
   if (db_->CheckErrors(q)) return;
@@ -171,7 +175,7 @@ void LibraryBackend::UpdateTotalSongCount() {
   QMutexLocker l(db_->Mutex());
   QSqlDatabase db(db_->Connect());
 
-  QSqlQuery q(QString("SELECT COUNT(*) FROM %1").arg(songs_table_), db);
+  QSqlQuery q(QString("SELECT COUNT(*) FROM %1 WHERE unavailable = 0").arg(songs_table_), db);
   q.exec();
   if (db_->CheckErrors(q)) return;
   if (!q.next()) return;
@@ -240,7 +244,7 @@ SongList LibraryBackend::FindSongsInDirectory(int id) {
   SongList ret;
   while (q.next()) {
     Song song;
-    song.InitFromQuery(q);
+    song.InitFromQuery(q, true);
     ret << song;
   }
   return ret;
@@ -426,6 +430,25 @@ void LibraryBackend::DeleteSongs(const SongList &songs) {
   UpdateTotalSongCountAsync();
 }
 
+void LibraryBackend::MarkSongsUnavailable(const SongList &songs) {
+  QMutexLocker l(db_->Mutex());
+  QSqlDatabase db(db_->Connect());
+
+  QSqlQuery remove(QString("UPDATE %1 SET unavailable = 1 WHERE ROWID = :id")
+                   .arg(songs_table_), db);
+
+  ScopedTransaction transaction(&db);
+  foreach (const Song& song, songs) {
+    remove.bindValue(":id", song.id());
+    remove.exec();
+    db_->CheckErrors(remove);
+  }
+  transaction.Commit();
+
+  emit SongsDeleted(songs);
+  UpdateTotalSongCountAsync();
+}
+
 QStringList LibraryBackend::GetAll(const QString& column, const QueryOptions& opt) {
   LibraryQuery query(opt);
   query.SetColumnSpec("DISTINCT " + column);
@@ -483,7 +506,7 @@ SongList LibraryBackend::GetSongs(const QString& artist, const QString& album, c
   SongList ret;
   while (query.Next()) {
     Song song;
-    song.InitFromQuery(query);
+    song.InitFromQuery(query, true);
     ret << song;
   }
   return ret;
@@ -523,7 +546,7 @@ SongList LibraryBackend::GetSongsByForeignId(
 
   QSqlQuery q(QString("SELECT %2.ROWID, " + Song::kColumnSpec + ", %2.%3"
                       " FROM %2, %1"
-                      " WHERE %2.%3 IN (%4) AND %1.ROWID = %2.ROWID")
+                      " WHERE %2.%3 IN (%4) AND %1.ROWID = %2.ROWID AND unavailable = 0")
               .arg(songs_table_, table, column, in), db);
   q.exec();
   if (db_->CheckErrors(q)) return SongList();
@@ -535,7 +558,7 @@ SongList LibraryBackend::GetSongsByForeignId(
     if (index == -1)
       continue;
 
-    ret[index].InitFromQuery(q);
+    ret[index].InitFromQuery(q, true);
   }
   return ret.toList();
 }
@@ -558,35 +581,35 @@ SongList LibraryBackend::GetSongsById(const QStringList& ids, QSqlDatabase& db)
   SongList ret;
   while (q.next()) {
     Song song;
-    song.InitFromQuery(q);
+    song.InitFromQuery(q, true);
     ret << song;
   }
   return ret;
 }
 
-Song LibraryBackend::GetSongByFilename(const QString& filename, qint64 beginning) {
+Song LibraryBackend::GetSongByUrl(const QUrl& url, qint64 beginning) {
   LibraryQuery query;
   query.SetColumnSpec("%songs_table.ROWID, " + Song::kColumnSpec);
-  query.AddWhere("filename", filename);
+  query.AddWhere("filename", url.toEncoded());
   query.AddWhere("beginning", beginning);
 
   Song song;
   if (ExecQuery(&query) && query.Next()) {
-    song.InitFromQuery(query);
+    song.InitFromQuery(query, true);
   }
   return song;
 }
 
-SongList LibraryBackend::GetSongsByFilename(const QString& filename) {
+SongList LibraryBackend::GetSongsByUrl(const QUrl& url) {
   LibraryQuery query;
   query.SetColumnSpec("%songs_table.ROWID, " + Song::kColumnSpec);
-  query.AddWhere("filename", filename);
+  query.AddWhere("filename", url.toEncoded());
 
   SongList songlist;
   if (ExecQuery(&query)) {
     while(query.Next()) {
       Song song;
-      song.InitFromQuery(query);
+      song.InitFromQuery(query, true);
 
       songlist << song;
     }
@@ -622,7 +645,7 @@ SongList LibraryBackend::GetCompilationSongs(const QString& album, const QueryOp
   SongList ret;
   while (query.Next()) {
     Song song;
-    song.InitFromQuery(query);
+    song.InitFromQuery(query, true);
     ret << song;
   }
   return ret;
@@ -632,10 +655,11 @@ void LibraryBackend::UpdateCompilations() {
   QMutexLocker l(db_->Mutex());
   QSqlDatabase db(db_->Connect());
 
-  // Look for albums that have songs by more than one artist in the same
+  // Look for albums that have songs by more than one 'effective album artist' in the same
   // directory
 
-  QSqlQuery q("SELECT artist, album, filename, sampler FROM songs ORDER BY album", db);
+  QSqlQuery q(QString("SELECT effective_albumartist, album, filename, sampler "
+    "FROM %1 WHERE unavailable = 0 ORDER BY album").arg(songs_table_), db);
   q.exec();
   if (db_->CheckErrors(q)) return;
 
@@ -666,9 +690,9 @@ void LibraryBackend::UpdateCompilations() {
   QSqlQuery update(QString("UPDATE %1"
                            " SET sampler = :sampler,"
                            "     effective_compilation = ((compilation OR :sampler OR forced_compilation_on) AND NOT forced_compilation_off) + 0"
-                           " WHERE album = :album").arg(songs_table_), db);
+                           " WHERE album = :album AND unavailable = 0").arg(songs_table_), db);
   QSqlQuery find_songs(QString("SELECT ROWID, " + Song::kColumnSpec + " FROM %1"
-                               " WHERE album = :album AND sampler = :sampler")
+                               " WHERE album = :album AND sampler = :sampler AND unavailable = 0")
                        .arg(songs_table_), db);
 
   SongList deleted_songs;
@@ -681,7 +705,7 @@ void LibraryBackend::UpdateCompilations() {
     const CompilationInfo& info = it.value();
     QString album(it.key());
 
-    // If there were more artists than there were directories for this album,
+    // If there were more 'effective album artists' than there were directories for this album,
     // then it's a compilation
 
     if (info.artists.count() > info.directories.count()) {
@@ -711,7 +735,7 @@ void LibraryBackend::UpdateCompilations(QSqlQuery& find_songs, QSqlQuery& update
   find_songs.exec();
   while (find_songs.next()) {
     Song song;
-    song.InitFromQuery(find_songs);
+    song.InitFromQuery(find_songs, true);
     deleted_songs << song;
     song.set_sampler(true);
     added_songs << song;
@@ -754,7 +778,7 @@ LibraryBackend::AlbumList LibraryBackend::GetAlbums(const QString& artist,
     info.album_name = query.Value(0).toString();
     info.art_automatic = query.Value(4).toString();
     info.art_manual = query.Value(5).toString();
-    info.first_filename = query.Value(6).toString();
+    info.first_url = QUrl::fromEncoded(query.Value(6).toByteArray());
 
     if (info.artist == last_artist && info.album_name == last_album)
       continue;
@@ -784,7 +808,7 @@ LibraryBackend::Album LibraryBackend::GetAlbumArt(const QString& artist, const Q
   if (query.Next()) {
     ret.art_automatic = query.Value(0).toString();
     ret.art_manual = query.Value(1).toString();
-    ret.first_filename = query.Value(2).toString();
+    ret.first_url = QUrl::fromEncoded(query.Value(2).toByteArray());
   }
 
   return ret;
@@ -817,14 +841,14 @@ void LibraryBackend::UpdateManualAlbumArt(const QString &artist,
   SongList deleted_songs;
   while (query.Next()) {
     Song song;
-    song.InitFromQuery(query);
+    song.InitFromQuery(query, true);
     deleted_songs << song;
   }
 
 
   // Update the songs
   QString sql(QString("UPDATE %1 SET art_manual = :art"
-                      " WHERE album = :album").arg(songs_table_));
+                      " WHERE album = :album AND unavailable = 0").arg(songs_table_));
   if (!artist.isNull())
     sql += " AND artist = :artist";
 
@@ -844,7 +868,7 @@ void LibraryBackend::UpdateManualAlbumArt(const QString &artist,
   SongList added_songs;
   while (query.Next()) {
     Song song;
-    song.InitFromQuery(query);
+    song.InitFromQuery(query, true);
     added_songs << song;
   }
 
@@ -854,52 +878,53 @@ void LibraryBackend::UpdateManualAlbumArt(const QString &artist,
   }
 }
 
-void LibraryBackend::ForceCompilation(const QString& artist, const QString& album, bool on) {
+void LibraryBackend::ForceCompilation(const QString& album, const QList<QString>& artists, bool on) {
   QMutexLocker l(db_->Mutex());
   QSqlDatabase db(db_->Connect());
+  SongList deleted_songs, added_songs;
 
-  // Get the songs before they're updated
-  LibraryQuery query;
-  query.SetColumnSpec("ROWID, " + Song::kColumnSpec);
-  query.AddWhere("album", album);
-  if (!artist.isNull())
-    query.AddWhere("artist", artist);
+  foreach(const QString &artist, artists) {
+    // Get the songs before they're updated
+    LibraryQuery query;
+    query.SetColumnSpec("ROWID, " + Song::kColumnSpec);
+    query.AddWhere("album", album);
+    if (!artist.isNull())
+      query.AddWhere("artist", artist);
 
-  if (!ExecQuery(&query)) return;
+    if (!ExecQuery(&query)) return;
 
-  SongList deleted_songs;
-  while (query.Next()) {
-    Song song;
-    song.InitFromQuery(query);
-    deleted_songs << song;
-  }
+    while (query.Next()) {
+      Song song;
+      song.InitFromQuery(query, true);
+      deleted_songs << song;
+    }
 
-  // Update the songs
-  QString sql(QString("UPDATE %1 SET forced_compilation_on = :forced_compilation_on,"
-                      "              forced_compilation_off = :forced_compilation_off,"
-                      "              effective_compilation = ((compilation OR sampler OR :forced_compilation_on) AND NOT :forced_compilation_off) + 0"
-                      " WHERE album = :album").arg(songs_table_));
-  if (!artist.isEmpty())
-    sql += " AND artist = :artist";
+    // Update the songs
+    QString sql(QString("UPDATE %1 SET forced_compilation_on = :forced_compilation_on,"
+                        "              forced_compilation_off = :forced_compilation_off,"
+                        "              effective_compilation = ((compilation OR sampler OR :forced_compilation_on) AND NOT :forced_compilation_off) + 0"
+                        " WHERE album = :album AND unavailable = 0").arg(songs_table_));
+    if (!artist.isEmpty())
+      sql += " AND artist = :artist";
 
-  QSqlQuery q(sql, db);
-  q.bindValue(":forced_compilation_on", on ? 1 : 0);
-  q.bindValue(":forced_compilation_off", on ? 0 : 1);
-  q.bindValue(":album", album);
-  if (!artist.isEmpty())
-    q.bindValue(":artist", artist);
+    QSqlQuery q(sql, db);
+    q.bindValue(":forced_compilation_on", on ? 1 : 0);
+    q.bindValue(":forced_compilation_off", on ? 0 : 1);
+    q.bindValue(":album", album);
+    if (!artist.isEmpty())
+      q.bindValue(":artist", artist);
 
-  q.exec();
-  db_->CheckErrors(q);
+    q.exec();
+    db_->CheckErrors(q);
 
-  // Now get the updated songs
-  if (!ExecQuery(&query)) return;
+    // Now get the updated songs
+    if (!ExecQuery(&query)) return;
 
-  SongList added_songs;
-  while (query.Next()) {
-    Song song;
-    song.InitFromQuery(query);
-    added_songs << song;
+    while (query.Next()) {
+      Song song;
+      song.InitFromQuery(query, true);
+      added_songs << song;
+    }
   }
 
   if (!added_songs.isEmpty() || !deleted_songs.isEmpty()) {
@@ -929,7 +954,7 @@ SongList LibraryBackend::FindSongs(const smart_playlists::Search& search) {
   // Read the results
   while (query.next()) {
     Song song;
-    song.InitFromQuery(query);
+    song.InitFromQuery(query, true);
     ret << song;
   }
   return ret;
