@@ -21,9 +21,10 @@
 #include "librarydirectorymodel.h"
 #include "libraryview.h"
 #include "sqlrow.h"
-#include "core/albumcoverloader.h"
 #include "core/database.h"
+#include "core/logging.h"
 #include "core/taskmanager.h"
+#include "covers/albumcoverloader.h"
 #include "playlist/songmimedata.h"
 #include "smartplaylists/generator.h"
 #include "smartplaylists/generatormimedata.h"
@@ -47,11 +48,16 @@ using smart_playlists::QueryGenerator;
 
 const char* LibraryModel::kSmartPlaylistsMimeType = "application/x-clementine-smart-playlist-generator";
 const char* LibraryModel::kSmartPlaylistsSettingsGroup = "SerialisedSmartPlaylists";
-const int LibraryModel::kSmartPlaylistsVersion = 3;
+const int LibraryModel::kSmartPlaylistsVersion = 4;
+const int LibraryModel::kPrettyCoverSize = 32;
 
 typedef QFuture<SqlRowList> RootQueryFuture;
 typedef QFutureWatcher<SqlRowList> RootQueryWatcher;
 
+static bool IsArtistGroupBy(const LibraryModel::GroupBy by) {
+  return by == LibraryModel::GroupBy_Artist || by == LibraryModel::GroupBy_AlbumArtist;
+}
+
 LibraryModel::LibraryModel(LibraryBackend* backend, TaskManager* task_manager,
                            QObject* parent)
   : SimpleTreeModel<LibraryItem>(new LibraryItem(this), parent),
@@ -67,20 +73,28 @@ LibraryModel::LibraryModel(LibraryBackend* backend, TaskManager* task_manager,
     playlists_dir_icon_(IconLoader::Load("folder-sound")),
     playlist_icon_(":/icons/22x22/x-clementine-albums.png"),
     init_task_id_(-1),
-    pretty_cover_size_(32, 32),
     use_pretty_covers_(false),
-    show_dividers_(true)
+    show_dividers_(true),
+    cover_loader_(new BackgroundThreadImplementation<AlbumCoverLoader, AlbumCoverLoader>(this))
 {
   root_->lazy_loaded = true;
 
   group_by_[0] = GroupBy_Artist;
   group_by_[1] = GroupBy_Album;
   group_by_[2] = GroupBy_None;
-  
-  no_cover_icon_pretty_ = QImage(":nocover.png").scaled(pretty_cover_size_,
-                                                    Qt::KeepAspectRatio,
-                                                    Qt::SmoothTransformation);
 
+  cover_loader_->Start(true);
+  cover_loader_->Worker()->SetDesiredHeight(kPrettyCoverSize);
+  cover_loader_->Worker()->SetPadOutputImage(true);
+  cover_loader_->Worker()->SetScaleOutputImage(true);
+
+  connect(cover_loader_->Worker().get(),
+          SIGNAL(ImageLoaded(quint64,QImage)),
+          SLOT(AlbumArtLoaded(quint64,QImage)));
+
+  no_cover_icon_pretty_ = QImage(":nocover.png").scaled(
+        kPrettyCoverSize, kPrettyCoverSize,
+        Qt::KeepAspectRatio, Qt::SmoothTransformation);
 }
 
 LibraryModel::~LibraryModel() {
@@ -151,7 +165,7 @@ void LibraryModel::SongsDiscovered(const SongList& songs) {
 
       // Special case: if we're at the top level and the song is a compilation
       // and the top level is Artists, then we want the Various Artists node :(
-      if (i == 0 && type == GroupBy_Artist && song.is_compilation()) {
+      if (i == 0 && IsArtistGroupBy(type) && song.is_compilation()) {
         if (compilation_artist_node_ == NULL)
           CreateCompilationArtistNode(true, root_);
         container = compilation_artist_node_;
@@ -164,13 +178,15 @@ void LibraryModel::SongsDiscovered(const SongList& songs) {
           case GroupBy_Artist:      key = song.artist(); break;
           case GroupBy_Composer:    key = song.composer(); break;
           case GroupBy_Genre:       key = song.genre(); break;
-          case GroupBy_AlbumArtist: key = song.albumartist(); break;
+          case GroupBy_AlbumArtist: key = song.effective_albumartist(); break;
           case GroupBy_Year:
             key = QString::number(qMax(0, song.year())); break;
           case GroupBy_YearAlbum:
             key = PrettyYearAlbum(qMax(0, song.year()), song.album()); break;
           case GroupBy_FileType:    key = song.filetype(); break;
-          case GroupBy_None: Q_ASSERT(0); break;
+          case GroupBy_None:
+            qLog(Error) << "GroupBy_None";
+            break;
         }
 
         // Does it exist already?
@@ -215,7 +231,7 @@ LibraryItem* LibraryModel::CreateCompilationArtistNode(bool signal, LibraryItem*
 
   compilation_artist_node_ =
       new LibraryItem(LibraryItem::Type_Container, parent);
-  compilation_artist_node_->key = tr("Various Artists");
+  compilation_artist_node_->key = tr("Various artists");
   compilation_artist_node_->sort_text = " various";
   compilation_artist_node_->container_level = parent->container_level + 1;
 
@@ -258,7 +274,7 @@ QString LibraryModel::DividerKey(GroupBy type, LibraryItem* item) const {
   case GroupBy_None:
     return QString();
   }
-  Q_ASSERT(0);
+  qLog(Error) << "Unknown GroupBy type" << type << "for item" << item->display_text;
   return QString();
 }
 
@@ -290,7 +306,7 @@ QString LibraryModel::DividerDisplayText(GroupBy type, const QString& key) const
     // fallthrough
     ;
   }
-  Q_ASSERT(0);
+  qLog(Error) << "Unknown GroupBy type" << type << "for divider key" << key;
   return QString();
 }
 
@@ -375,35 +391,40 @@ void LibraryModel::SongsDeleted(const SongList& songs) {
   }
 }
 
-QVariant LibraryModel::AlbumIcon(const QModelIndex& index, int role) const {  
-  
-  // the easiest way to get from *here* to working out what album art an index 
-  // represents seems to be to get the node's child songs and look at their metadata.
-  // if none is found, return the generic CD icon
-
+QVariant LibraryModel::AlbumIcon(const QModelIndex& index) {
   // Cache the art in the item's metadata field
   LibraryItem* item = IndexToItem(index);
   if (!item)
     return no_cover_icon_pretty_;
   if (!item->metadata.image().isNull())
     return item->metadata.image();
-  
+
+  // No art is cached - load art for the first Song in the album.
   SongList songs = GetChildSongs(index);
   if (!songs.isEmpty()) {
-    const Song& s = songs.first();
-    QPixmap pixmap = AlbumCoverLoader::TryLoadPixmap(
-      s.art_automatic(), s.art_manual(), s.filename());
-
-    if (!pixmap.isNull()) {
-      QImage image = pixmap.toImage().scaled(
-            pretty_cover_size_, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-      item->metadata.set_image(image);
-      return image;
-    }
+    const quint64 id = cover_loader_->Worker()->LoadImageAsync(songs.first());
+    pending_art_[id] = item;
   }
+
   return no_cover_icon_pretty_;
 }
 
+void LibraryModel::AlbumArtLoaded(quint64 id, const QImage& image) {
+  LibraryItem* item = pending_art_.take(id);
+  if (!item)
+    return;
+
+  if (image.isNull()) {
+    // Set the no_cover image so we don't continually try to load art.
+    item->metadata.set_image(no_cover_icon_pretty_);
+  } else {
+    item->metadata.set_image(image);
+  }
+
+  const QModelIndex index = ItemToIndex(item);
+  emit dataChanged(index, index);
+}
+
 QVariant LibraryModel::data(const QModelIndex& index, int role) const {
   const LibraryItem* item = IndexToItem(index);
   
@@ -412,15 +433,17 @@ QVariant LibraryModel::data(const QModelIndex& index, int role) const {
   // QModelIndex& version of GetChildSongs, which satisfies const-ness, instead
   // of the LibraryItem* version, which doesn't.
   if (use_pretty_covers_) {    
-    bool album_node = false;
+    bool is_album_node = false;
     if (role == Qt::DecorationRole && item->type == LibraryItem::Type_Container) {
       GroupBy container_type = group_by_[item->container_level];
-      album_node = container_type == GroupBy_Album 
-                    || container_type == GroupBy_YearAlbum
-                    || container_type == GroupBy_AlbumArtist;
+      is_album_node = container_type == GroupBy_Album
+                   || container_type == GroupBy_YearAlbum
+                   || container_type == GroupBy_AlbumArtist;
+    }
+    if (is_album_node) {
+      // It has const behaviour some of the time - that's ok right?
+      return const_cast<LibraryModel*>(this)->AlbumIcon(index);
     }
-    if (album_node)
-      return AlbumIcon(index, role);
   }
 
   return data(item, role);
@@ -517,7 +540,7 @@ SqlRowList LibraryModel::RunRootQuery(const QueryOptions& query_options,
   InitQuery(child_type, &q);
 
   // Top-level artists is special - we don't want compilation albums appearing
-  if (child_type == GroupBy_Artist) {
+  if (IsArtistGroupBy(child_type)) {
     q.AddCompilationRequirement(false);
   }
 
@@ -550,13 +573,13 @@ void LibraryModel::LazyPopulate(LibraryItem* parent, bool signal) {
   InitQuery(child_type, &q);
 
   // Top-level artists is special - we don't want compilation albums appearing
-  if (child_level == 0 && child_type == GroupBy_Artist) {
+  if (child_level == 0 && IsArtistGroupBy(child_type)) {
     q.AddCompilationRequirement(false);
   }
 
   // Walk up through the item's parents adding filters as necessary
   LibraryItem* p = parent;
-  while (p != root_) {
+  while (p && p->type == LibraryItem::Type_Container) {
     FilterQuery(group_by_[p->container_level], p, &q);
     p = p->parent;
   }
@@ -631,6 +654,7 @@ void LibraryModel::BeginReset() {
   container_nodes_[1].clear();
   container_nodes_[2].clear();
   divider_nodes_.clear();
+  pending_art_.clear();
   compilation_artist_node_ = NULL;
   smart_playlist_node_ = NULL;
 
@@ -639,7 +663,7 @@ void LibraryModel::BeginReset() {
 
   if (show_various_artists_) {
     // Various artists?
-    if (group_by_[0] == GroupBy_Artist &&
+    if (IsArtistGroupBy(group_by_[0]) &&
         backend_->HasCompilations(query_options_))
       CreateCompilationArtistNode(false, root_);
   }
@@ -680,7 +704,7 @@ void LibraryModel::InitQuery(GroupBy type, LibraryQuery* q) {
     q->SetColumnSpec("DISTINCT genre");
     break;
   case GroupBy_AlbumArtist:
-    q->SetColumnSpec("DISTINCT albumartist");
+    q->SetColumnSpec("DISTINCT effective_albumartist");
     break;
   case GroupBy_None:
     q->SetColumnSpec("%songs_table.ROWID, " + Song::kColumnSpec);
@@ -722,13 +746,19 @@ void LibraryModel::FilterQuery(GroupBy type, LibraryItem* item, LibraryQuery* q)
     q->AddWhere("genre", item->key);
     break;
   case GroupBy_AlbumArtist:
-    q->AddWhere("albumartist", item->key);
+    if (item == compilation_artist_node_)
+      q->AddCompilationRequirement(true);
+    else {
+      if (item->container_level == 0) // Same stupid hack as above
+        q->AddCompilationRequirement(false);
+      q->AddWhere("effective_albumartist", item->key);
+    }
     break;
   case GroupBy_FileType:
     q->AddWhere("filetype", item->metadata.filetype());
     break;
   case GroupBy_None:
-    Q_ASSERT(0);
+    qLog(Error) << "Unknown GroupBy type" << type << "used in filter";
     break;
   }
 }
@@ -792,7 +822,7 @@ LibraryItem* LibraryModel::ItemFromQuery(GroupBy type,
     break;
 
   case GroupBy_None:
-    item->metadata.InitFromQuery(row);
+    item->metadata.InitFromQuery(row, true);
     item->key = item->metadata.title();
     item->display_text = item->metadata.TitleWithCompilationArtist();
     item->sort_text = SortTextForSong(item->metadata);
@@ -834,7 +864,7 @@ LibraryItem* LibraryModel::ItemFromSong(GroupBy type,
   case GroupBy_Composer:                      item->key = s.composer();
   case GroupBy_Genre: if (item->key.isNull()) item->key = s.genre();
   case GroupBy_Album: if (item->key.isNull()) item->key = s.album();
-  case GroupBy_AlbumArtist: if (item->key.isNull()) item->key = s.albumartist();
+  case GroupBy_AlbumArtist: if (item->key.isNull()) item->key = s.effective_albumartist();
     item->display_text = TextOrUnknown(item->key);
     item->sort_text = SortTextForArtist(item->key);
     break;
@@ -853,6 +883,8 @@ LibraryItem* LibraryModel::ItemFromSong(GroupBy type,
   }
 
   FinishItem(type, signal, create_divider, parent, item);
+  if (s.url().scheme() == "cdda")
+    item->lazy_loaded = true;
   return item;
 }
 
@@ -931,7 +963,7 @@ QString LibraryModel::SortTextForYear(int year) const {
 QString LibraryModel::SortTextForSong(const Song& song) const {
   QString ret = QString::number(qMax(0, song.disc()) * 1000 + qMax(0, song.track()));
   ret.prepend(QString("0").repeated(6 - ret.length()));
-  ret.append(song.filename());
+  ret.append(song.url().toString());
   return ret;
 }
 
@@ -982,7 +1014,7 @@ QMimeData* LibraryModel::mimeData(const QModelIndexList& indexes) const {
   }
 
   data->setUrls(urls);
-  data->name_for_new_playlist_ = LibraryView::GetNameForNewPlaylist(data->songs);
+  data->name_for_new_playlist_ = PlaylistManager::GetNameForNewPlaylist(data->songs);
 
   return data;
 }
@@ -1012,7 +1044,7 @@ void LibraryModel::GetChildSongs(LibraryItem* item, QList<QUrl>* urls,
     }
 
     case LibraryItem::Type_Song:
-      urls->append(QUrl::fromLocalFile(item->metadata.filename()));
+      urls->append(item->metadata.url());
       if (!song_ids->contains(item->metadata.id())) {
         songs->append(item->metadata);
         song_ids->insert(item->metadata.id());
@@ -1075,7 +1107,7 @@ const LibraryModel::GroupBy& LibraryModel::Grouping::operator [](int i) const {
     case 1: return second;
     case 2: return third;
   }
-  Q_ASSERT(0);
+  qLog(Error) << "LibraryModel::Grouping[] index out of range" << i;
   return first;
 }
 
@@ -1085,7 +1117,7 @@ LibraryModel::GroupBy& LibraryModel::Grouping::operator [](int i) {
     case 1: return second;
     case 2: return third;
   }
-  Q_ASSERT(0);
+  qLog(Error) << "LibraryModel::Grouping[] index out of range" << i;
   return first;
 }
 
@@ -1094,6 +1126,7 @@ void LibraryModel::CreateSmartPlaylists() {
   smart_playlist_node_->container_level = 0;
   smart_playlist_node_->sort_text = "\0";
   smart_playlist_node_->key = tr("Smart playlists");
+  smart_playlist_node_->lazy_loaded = true;
 
   QSettings s;
   s.beginGroup(kSmartPlaylistsSettingsGroup);
@@ -1133,7 +1166,7 @@ void LibraryModel::CreateSmartPlaylists() {
 void LibraryModel::ItemFromSmartPlaylist(const QSettings& s, bool notify) const {
   LibraryItem* item = new LibraryItem(LibraryItem::Type_SmartPlaylist,
                                       notify ? NULL : smart_playlist_node_);
-  item->display_text = s.value("name").toString();
+  item->display_text = tr(qPrintable(s.value("name").toString()));
   item->sort_text = item->display_text;
   item->key = s.value("type").toString();
   item->smart_playlist_data = s.value("data").toByteArray();
@@ -1237,3 +1270,5 @@ void LibraryModel::TotalSongCountUpdatedSlot(int count) {
   total_song_count_ = count;
   emit TotalSongCountUpdated(count);
 }
+
+
