@@ -16,6 +16,7 @@
 */
 
 #include "librarydirectorymodel.h"
+#include "libraryfilterwidget.h"
 #include "librarymodel.h"
 #include "libraryview.h"
 #include "libraryitem.h"
@@ -26,8 +27,6 @@
 #include "core/utilities.h"
 #include "devices/devicemanager.h"
 #include "devices/devicestatefiltermodel.h"
-#include "scripting/scriptmanager.h"
-#include "scripting/uiinterface.h"
 #include "smartplaylists/wizard.h"
 #include "ui/iconloader.h"
 #include "ui/organisedialog.h"
@@ -131,8 +130,11 @@ bool LibraryItemDelegate::helpEvent(QHelpEvent *event, QAbstractItemView *view,
 
 LibraryView::LibraryView(QWidget* parent)
   : AutoExpandingTreeView(parent),
-    scripts_(NULL),
+    cover_providers_(NULL),
     library_(NULL),
+    devices_(NULL),
+    task_manager_(NULL),
+    filter_(NULL),
     total_song_count_(-1),
     nomusic_(":nomusic.png"),
     context_menu_(NULL),
@@ -146,7 +148,6 @@ LibraryView::LibraryView(QWidget* parent)
   setDragDropMode(QAbstractItemView::DragOnly);
   setSelectionMode(QAbstractItemView::ExtendedSelection);
 
-  ReloadSettings();
   setStyleSheet("QTreeView::item{padding-top:1px;}");
 }
 
@@ -164,6 +165,10 @@ void LibraryView::ReloadSettings() {
   }
 }
 
+void LibraryView::SetCoverProviders(CoverProviders* cover_providers) {
+  cover_providers_ = cover_providers;
+}
+
 void LibraryView::SetTaskManager(TaskManager *task_manager) {
   task_manager_ = task_manager;
 }
@@ -177,8 +182,8 @@ void LibraryView::SetDeviceManager(DeviceManager* device_manager) {
   devices_ = device_manager;
 }
 
-void LibraryView::SetScriptManager(ScriptManager* scripts) {
-  scripts_ = scripts;
+void LibraryView::SetFilter(LibraryFilterWidget* filter) {
+  filter_ = filter;
 }
 
 void LibraryView::TotalSongCountUpdated(int count) {
@@ -277,13 +282,11 @@ void LibraryView::contextMenuEvent(QContextMenuEvent *e) {
 
     context_menu_->addSeparator();
 
+    context_menu_->addMenu(filter_->menu());
+
     copy_to_device_->setDisabled(devices_->connected_devices_model()->rowCount() == 0);
     connect(devices_->connected_devices_model(), SIGNAL(IsEmptyChanged(bool)),
             copy_to_device_, SLOT(setDisabled(bool)));
-
-    if(scripts_) {
-      scripts_->ui()->RegisterActionLocation("library_context_menu", context_menu_, NULL);
-    }
   }
 
   context_menu_index_ = indexAt(e->pos());
@@ -377,16 +380,17 @@ void LibraryView::ShowInVarious(bool on) {
   if (!context_menu_index_.isValid())
     return;
 
-  // Build a list of the selected unique album/artist combos
-  typedef QPair<QString, QString> AlbumArtist;
-  QSet<AlbumArtist> selected;
+  // Map is from album name -> all artists sharing that album name, built from each selected
+  // song. We put through "Various Artists" changes one album at a time, to make sure the old album
+  // node gets removed (due to all children removed), before the new one gets added
+  QMultiMap<QString, QString> albums;
   foreach (const Song& song, GetSelectedSongs()) {
-    selected << AlbumArtist(song.album(), song.artist());
+    if (albums.find(song.album(), song.artist()) == albums.end())
+      albums.insert( song.album(), song.artist() );
   }
 
-  foreach (const AlbumArtist& albumartist, selected.values()) {
-    library_->backend()->ForceCompilation(
-          albumartist.second, albumartist.first, on);
+  foreach (const QString& album, QSet<QString>::fromList(albums.keys())) {
+    library_->backend()->ForceCompilation(album, albums.values(album), on);
   }
 }
 
@@ -472,7 +476,7 @@ void LibraryView::Delete() {
 
 void LibraryView::EditTracks() {
   if(!edit_tag_dialog_) {
-    edit_tag_dialog_.reset(new EditTagDialog(this));
+    edit_tag_dialog_.reset(new EditTagDialog(cover_providers_, this));
   }
   edit_tag_dialog_->SetSongs(GetSelectedSongs());
   edit_tag_dialog_->SetTagCompleter(library_->backend());
@@ -547,48 +551,11 @@ void LibraryView::EditSmartPlaylistFinished() {
   library_->UpdateGenerator(context_menu_index_, wizard->CreateGenerator());
 }
 
-QString LibraryView::GetNameForNewPlaylist(const SongList& songs) {
-  if (songs.isEmpty()) {
-    return tr("Playlist");
-  }
-
-  QSet<QString> artists;
-  QSet<QString> albums;
-
-  foreach(const Song& song, songs) {
-    artists << (song.artist().isEmpty()
-                    ? tr("Unknown")
-                    : song.artist());
-    albums << (song.album().isEmpty()
-                    ? tr("Unknown")
-                    : song.album());
-
-    if(artists.size() > 1) {
-      break;
-    }
-  }
-
-  bool various_artists = artists.size() > 1;
-
-  QString result;
-  if(various_artists) {
-    result = tr("Various artists");
-  } else {
-    result = artists.values().first();
-  }
-
-  if(!various_artists && albums.size() == 1) {
-    result += " - " + albums.toList().first();
-  }
-
-  return result;
-}
-
 void LibraryView::ShowInBrowser() {
-  QStringList filenames;
+  QList<QUrl> urls;
   foreach (const Song& song, GetSelectedSongs()) {
-    filenames << song.filename();
+    urls << song.url();
   }
 
-  Utilities::OpenInFileBrowser(filenames);
+  Utilities::OpenInFileBrowser(urls);
 }
