@@ -16,11 +16,13 @@
 */
 
 #include "librarywatcher.h"
+
 #include "librarybackend.h"
+#include "core/filesystemwatcherinterface.h"
+#include "core/logging.h"
 #include "core/taskmanager.h"
 #include "playlistparsers/cueparser.h"
 
-#include <QFileSystemWatcher>
 #include <QDateTime>
 #include <QDirIterator>
 #include <QtDebug>
@@ -34,6 +36,11 @@
 #include <fileref.h>
 #include <tag.h>
 
+// This is defined by one of the windows headers that is included by taglib.
+#ifdef RemoveDirectory
+#undef RemoveDirectory
+#endif
+
 QStringList LibraryWatcher::sValidImages;
 
 const char* LibraryWatcher::kSettingsGroup = "LibraryWatcher";
@@ -42,6 +49,7 @@ LibraryWatcher::LibraryWatcher(QObject* parent)
   : QObject(parent),
     backend_(NULL),
     task_manager_(NULL),
+    fs_watcher_(FileSystemWatcherInterface::Create(this)),
     stop_requested_(false),
     scan_on_startup_(true),
     monitor_(true),
@@ -107,7 +115,7 @@ LibraryWatcher::ScanTransaction::~ScanTransaction() {
   if (watcher_->monitor_) {
     // Watch the new subdirectories
     foreach (const Subdirectory& subdir, new_subdirs) {
-      watcher_->AddWatch(watcher_->watched_dirs_[dir_].watcher, subdir.path);
+      watcher_->AddWatch(watcher_->watched_dirs_[dir_], subdir.path);
     }
   }
 }
@@ -131,7 +139,7 @@ SongList LibraryWatcher::ScanTransaction::FindSongsInSubdirectory(const QString
   // TODO: Make this faster
   SongList ret;
   foreach (const Song& song, cached_songs_) {
-    if (song.filename().section('/', 0, -2) == path)
+    if (song.url().toLocalFile().section('/', 0, -2) == path)
       ret << song;
   }
   return ret;
@@ -177,8 +185,6 @@ SubdirectoryList LibraryWatcher::ScanTransaction::GetAllSubdirs() {
 void LibraryWatcher::AddDirectory(const Directory& dir, const SubdirectoryList& subdirs) {
   DirData data;
   data.dir = dir;
-  data.watcher = new QFileSystemWatcher(this);
-  connect(data.watcher, SIGNAL(directoryChanged(QString)), SLOT(DirectoryChanged(QString)));
   watched_dirs_[dir.id] = data;
 
   if (subdirs.isEmpty()) {
@@ -201,7 +207,7 @@ void LibraryWatcher::AddDirectory(const Directory& dir, const SubdirectoryList&
         ScanSubdirectory(subdir.path, subdir, &transaction);
 
       if (monitor_)
-        AddWatch(data.watcher, subdir.path);
+        AddWatch(data, subdir.path);
     }
   }
 
@@ -315,7 +321,8 @@ void LibraryWatcher::ScanSubdirectory(
 
       // watch out for cue songs which have their mtime equal to qMax(media_file_mtime, cue_sheet_mtime)
       bool changed = (matching_song.mtime() != qMax(file_info.lastModified().toTime_t(), song_cue_mtime))
-                     || cue_deleted || cue_added;
+                     || cue_deleted || cue_added
+                     || matching_song.is_unavailable();
 
       // Also want to look to see whether the album art has changed
       QString image = ImageForSong(file, album_art);
@@ -328,7 +335,7 @@ void LibraryWatcher::ScanSubdirectory(
 
       // the song's changed - reread the metadata from file
       if (t->ignores_mtime() || changed) {
-        qDebug() << file << "changed";
+        qLog(Debug) << file << "changed";
 
         // if cue associated...
         if(!cue_deleted && (matching_song.has_cue() || cue_added)) {
@@ -346,7 +353,7 @@ void LibraryWatcher::ScanSubdirectory(
         continue;
       }
 
-      qDebug() << file << "created";
+      qLog(Debug) << file << "created";
       // choose an image for the song(s)
       QString image = ImageForSong(file, album_art);
 
@@ -362,8 +369,8 @@ void LibraryWatcher::ScanSubdirectory(
 
   // Look for deleted songs
   foreach (const Song& song, songs_in_db) {
-    if (!files_on_disk.contains(song.filename())) {
-      qDebug() << "Song deleted from disk:" << song.filename();
+    if (!song.is_unavailable() && !files_on_disk.contains(song.url().toLocalFile())) {
+      qLog(Debug) << "Song deleted from disk:" << song.url().toLocalFile();
       t->deleted_songs << song;
     }
   }
@@ -396,7 +403,7 @@ void LibraryWatcher::UpdateCueAssociatedSongs(const QString& file, const QString
   QFile cue(matching_cue);
   cue.open(QIODevice::ReadOnly);
 
-  SongList old_sections = backend_->GetSongsByFilename(file);
+  SongList old_sections = backend_->GetSongsByUrl(QUrl::fromLocalFile(file));
 
   QHash<quint64, Song> sections_map;
   foreach(const Song& song, old_sections) {
@@ -436,7 +443,7 @@ void LibraryWatcher::UpdateNonCueAssociatedSong(const QString& file, const Song&
   // 'raw' (cueless) song and we just remove the rest of the sections
   // from the library
   if(cue_deleted) {
-    foreach(const Song& song, backend_->GetSongsByFilename(file)) {
+    foreach(const Song& song, backend_->GetSongsByUrl(QUrl::fromLocalFile(file))) {
       if(!song.IsMetadataEqual(matching_song)) {
         t->deleted_songs << song;
       }
@@ -469,7 +476,7 @@ SongList LibraryWatcher::ScanNewFile(const QString& file, const QString& path,
     // media files. Playlist parser for CUEs considers every entry in sheet
     // valid and we don't want invalid media getting into library!
     foreach(const Song& cue_song, cue_parser_->Load(&cue, matching_cue, path)) {
-      if(cue_song.filename() == file && cue_song.HasProperMediaFile()) {
+      if(cue_song.url().toLocalFile() == file && cue_song.HasProperMediaFile()) {
         song_list << cue_song;
       }
     }
@@ -509,8 +516,14 @@ void LibraryWatcher::PreserveUserSetData(const QString& file, const QString& ima
   out->set_score(matching_song.score());
   out->set_art_manual(matching_song.art_manual());
 
-  if (!matching_song.IsMetadataEqual(*out)) {
-    qDebug() << file << "metadata changed";
+  // The song was deleted from the database (e.g. due to an unmounted 
+  // filesystem), but has been restored.
+  if (matching_song.is_unavailable()) {
+    qLog(Debug) << file << " unavailable song restored";
+    
+    t->new_songs << *out;
+  } else if (!matching_song.IsMetadataEqual(*out)) {
+    qLog(Debug) << file << "metadata changed";
 
     // Update the song in the DB
     t->new_songs << *out;
@@ -533,18 +546,17 @@ uint LibraryWatcher::GetMtimeForCue(const QString& cue_path) {
              : 0;
 }
 
-void LibraryWatcher::AddWatch(QFileSystemWatcher* w, const QString& path) {
+void LibraryWatcher::AddWatch(const DirData& dir, const QString& path) {
   if (!QFile::exists(path))
     return;
 
-  w->addPath(path);
+  connect(fs_watcher_, SIGNAL(PathChanged(const QString&)), this,
+      SLOT(DirectoryChanged(const QString&)), Qt::UniqueConnection);
+  fs_watcher_->AddPath(path);
+  subdir_mapping_[path] = dir;
 }
 
 void LibraryWatcher::RemoveDirectory(const Directory& dir) {
-  if (watched_dirs_.contains(dir.id)) {
-    delete watched_dirs_[dir.id].watcher;
-  }
-
   rescan_queue_.remove(dir.id);
   watched_dirs_.remove(dir.id);
 }
@@ -552,7 +564,7 @@ void LibraryWatcher::RemoveDirectory(const Directory& dir) {
 bool LibraryWatcher::FindSongByPath(const SongList& list, const QString& path, Song* out) {
   // TODO: Make this faster
   foreach (const Song& song, list) {
-    if (song.filename() == path) {
+    if (song.url().toLocalFile() == path) {
       *out = song;
       return true;
     }
@@ -562,17 +574,13 @@ bool LibraryWatcher::FindSongByPath(const SongList& list, const QString& path, S
 
 void LibraryWatcher::DirectoryChanged(const QString &subdir) {
   // Find what dir it was in
-  QFileSystemWatcher* watcher = qobject_cast<QFileSystemWatcher*>(sender());
-  if (!watcher)
+  QHash<QString, DirData>::const_iterator it = subdir_mapping_.constFind(subdir);
+  if (it == subdir_mapping_.constEnd()) {
     return;
-
-  Directory dir;
-  foreach (const DirData& info, watched_dirs_) {
-    if (info.watcher == watcher)
-      dir = info.dir;
   }
+  Directory dir = it->dir;
 
-  qDebug() << "Subdir" << subdir << "changed under directory" << dir.path << "id" << dir.id;
+  qLog(Debug) << "Subdir" << subdir << "changed under directory" << dir.path << "id" << dir.id;
 
   // Queue the subdir for rescanning
   if (!rescan_queue_[dir.id].contains(subdir))
@@ -676,7 +684,7 @@ void LibraryWatcher::ReloadSettings() {
   s.beginGroup(kSettingsGroup);
   scan_on_startup_ = s.value("startup_scan", true).toBool();
   monitor_ = s.value("monitor", true).toBool();
-  
+
   best_image_filters_.clear();
   QStringList filters = s.value("cover_art_patterns",
       QStringList() << "front" << "cover").toStringList();
@@ -685,18 +693,15 @@ void LibraryWatcher::ReloadSettings() {
     if (!s.isEmpty())
       best_image_filters_ << s;
   }
-  
+
   if (!monitor_ && was_monitoring_before) {
-    // Remove all directories from all QFileSystemWatchers
-    foreach (const DirData& data, watched_dirs_.values()) {
-      data.watcher->removePaths(data.watcher->directories());
-    }
+    fs_watcher_->Clear();
   } else if (monitor_ && !was_monitoring_before) {
     // Add all directories to all QFileSystemWatchers again
     foreach (const DirData& data, watched_dirs_.values()) {
       SubdirectoryList subdirs = backend_->SubdirsInDirectory(data.dir.id);
       foreach (const Subdirectory& subdir, subdirs) {
-        AddWatch(data.watcher, subdir.path);
+        AddWatch(data, subdir.path);
       }
     }
   }
