@@ -24,10 +24,11 @@
 #endif // Q_OS_WIN32
 
 #include "config.h"
-#include "core/artloader.h"
 #include "core/commandlineoptions.h"
+#include "core/crashreporting.h"
 #include "core/database.h"
 #include "core/encoding.h"
+#include "core/logging.h"
 #include "core/mac_startup.h"
 #include "core/network.h"
 #include "core/networkproxyfactory.h"
@@ -35,12 +36,20 @@
 #include "core/potranslator.h"
 #include "core/song.h"
 #include "core/taskmanager.h"
+#include "core/ubuntuunityhack.h"
 #include "core/utilities.h"
+#include "covers/albumcoverfetcher.h"
+#include "covers/amazoncoverprovider.h"
+#include "covers/artloader.h"
+#include "covers/coverproviders.h"
 #include "engines/enginebase.h"
+#include "globalsearch/globalsearch.h"
+#include "internet/digitallyimportedclient.h"
+#include "internet/internetmodel.h"
+#include "internet/somafmservice.h"
 #include "library/directory.h"
 #include "playlist/playlist.h"
 #include "playlist/playlistmanager.h"
-#include "radio/radiomodel.h"
 #include "smartplaylists/generator.h"
 #include "ui/equalizer.h"
 #include "ui/iconloader.h"
@@ -54,14 +63,17 @@
 
 #include <QDir>
 #include <QLibraryInfo>
+#include <QNetworkCookie>
 #include <QNetworkProxyFactory>
+#include <QSslSocket>
 #include <QSqlDatabase>
 #include <QSqlQuery>
+#include <QTextCodec>
 #include <QTranslator>
 #include <QtDebug>
 
 #include <glib-object.h>
-#include <glib/gutils.h>
+#include <glib.h>
 #include <gst/gst.h>
 
 #include <boost/scoped_ptr.hpp>
@@ -69,13 +81,17 @@ using boost::scoped_ptr;
 
 #include <echonest/Config.h>
 
+#ifdef HAVE_QCA
+  #include <QtCrypto>
+#endif
+
 #ifdef Q_OS_DARWIN
   #include <sys/resource.h>
   #include <sys/sysctl.h>
 #endif
 
 #ifdef HAVE_LIBLASTFM
-  #include "radio/lastfmservice.h"
+  #include "internet/lastfmservice.h"
 #else
   class LastFMService;
 #endif
@@ -84,6 +100,7 @@ using boost::scoped_ptr;
   #include "core/mpris.h"
   #include "core/mpris2.h"
   #include "dbus/metatypes.h"
+  #include "globalsearch/globalsearchservice.h"
   #include <QDBusArgument>
   #include <QDBusConnection>
   #include <QImage>
@@ -119,39 +136,45 @@ void LoadTranslation(const QString& prefix, const QString& path,
     QCoreApplication::installTranslator(t);
   else
     delete t;
+  QTextCodec::setCodecForTr(QTextCodec::codecForLocale());
 }
 
-void GLog(const gchar* domain,
-          GLogLevelFlags level,
-          const gchar* message,
-          gpointer user_data) {
-  qDebug() << "GLOG" << message;
+#ifdef HAVE_REMOTE
+#include <xrme/connection.h>
+#endif
+
+void IncreaseFDLimit() {
+#ifdef Q_OS_DARWIN
+  // Bump the soft limit for the number of file descriptors from the default of 256 to
+  // the maximum (usually 10240).
+  struct rlimit limit;
+  getrlimit(RLIMIT_NOFILE, &limit);
+
+  // getrlimit() lies about the hard limit so we have to check sysctl.
+  int max_fd = 0;
+  size_t len = sizeof(max_fd);
+  sysctlbyname("kern.maxfilesperproc", &max_fd, &len, NULL, 0);
+
+  limit.rlim_cur = max_fd;
+  int ret = setrlimit(RLIMIT_NOFILE, &limit);
+
+  if (ret == 0) {
+    qLog(Debug) << "Max fd:" << max_fd;
+  }
+#endif
 }
 
 int main(int argc, char *argv[]) {
+  if (CrashReporting::SendCrashReport(argc, argv)) {
+    return 0;
+  }
+
+  CrashReporting crash_reporting;
+
 #ifdef Q_OS_DARWIN
   // Do Mac specific startup to get media keys working.
   // This must go before QApplication initialisation.
   mac::MacMain();
-
-  {
-    // Bump the soft limit for the number of file descriptors from the default of 256 to
-    // the maximum (usually 10240).
-    struct rlimit limit;
-    getrlimit(RLIMIT_NOFILE, &limit);
-
-    // getrlimit() lies about the hard limit so we have to check sysctl.
-    int max_fd = 0;
-    size_t len = sizeof(max_fd);
-    sysctlbyname("kern.maxfilesperproc", &max_fd, &len, NULL, 0);
-
-    limit.rlim_cur = max_fd;
-    int ret = setrlimit(RLIMIT_NOFILE, &limit);
-
-    if (ret == 0) {
-      qDebug() << "Max fd:" << max_fd;
-    }
-  }
 #endif
 
   QCoreApplication::setApplicationName("Clementine");
@@ -181,14 +204,18 @@ int main(int argc, char *argv[]) {
   g_type_init();
   g_set_application_name(QCoreApplication::applicationName().toLocal8Bit());
 
-  g_log_set_default_handler(&GLog, NULL);
-
+  qRegisterMetaType<CoverSearchResult>("CoverSearchResult");
+  qRegisterMetaType<QList<CoverSearchResult> >("QList<CoverSearchResult>");
+  qRegisterMetaType<CoverSearchResults>("CoverSearchResults");
   qRegisterMetaType<Directory>("Directory");
   qRegisterMetaType<DirectoryList>("DirectoryList");
   qRegisterMetaType<Subdirectory>("Subdirectory");
   qRegisterMetaType<SubdirectoryList>("SubdirectoryList");
   qRegisterMetaType<Song>("Song");
+  qRegisterMetaType<QList<Song> >("QList<Song>");
   qRegisterMetaType<SongList>("SongList");
+  qRegisterMetaType<PlaylistItemPtr>("PlaylistItemPtr");
+  qRegisterMetaType<QList<PlaylistItemPtr> >("QList<PlaylistItemPtr>");
   qRegisterMetaType<PlaylistItemList>("PlaylistItemList");
   qRegisterMetaType<Engine::State>("Engine::State");
   qRegisterMetaType<Engine::SimpleMetaBundle>("Engine::SimpleMetaBundle");
@@ -200,14 +227,27 @@ int main(int argc, char *argv[]) {
   qRegisterMetaType<smart_playlists::GeneratorPtr>("smart_playlists::GeneratorPtr");
   qRegisterMetaType<ColumnAlignmentMap>("ColumnAlignmentMap");
   qRegisterMetaTypeStreamOperators<QMap<int, int> >("ColumnAlignmentMap");
+  qRegisterMetaType<QNetworkCookie>("QNetworkCookie");
+  qRegisterMetaType<QList<QNetworkCookie> >("QList<QNetworkCookie>");
+  qRegisterMetaType<SearchProvider::Result>("SearchProvider::Result");
+  qRegisterMetaType<SearchProvider::ResultList>("SearchProvider::ResultList");
+  qRegisterMetaType<DigitallyImportedClient::Channel>("DigitallyImportedClient::Channel");
+  qRegisterMetaType<SomaFMService::Stream>("SomaFMService::Stream");
+  qRegisterMetaTypeStreamOperators<DigitallyImportedClient::Channel>("DigitallyImportedClient::Channel");
+  qRegisterMetaTypeStreamOperators<SomaFMService::Stream>("SomaFMService::Stream");
 
   qRegisterMetaType<GstBuffer*>("GstBuffer*");
   qRegisterMetaType<GstElement*>("GstElement*");
   qRegisterMetaType<GstEnginePipeline*>("GstEnginePipeline*");
 
+#ifdef HAVE_REMOTE
+  qRegisterMetaType<xrme::SIPInfo>("xrme::SIPInfo");
+#endif
+
 #ifdef HAVE_LIBLASTFM
   lastfm::ws::ApiKey = LastFMService::kApiKey;
   lastfm::ws::SharedSecret = LastFMService::kSecret;
+  lastfm::setNetworkAccessManager(new NetworkAccessManager);
 #endif
 
   CommandlineOptions options(argc, argv);
@@ -218,6 +258,7 @@ int main(int argc, char *argv[]) {
     // This MUST be done before parsing the commandline options so QTextCodec
     // gets the right system locale for filenames.
     QtSingleCoreApplication a(argc, argv);
+    crash_reporting.SetApplicationPath(a.applicationFilePath());
 
     // Parse commandline options - need to do this before starting the
     // full QApplication so it works without an X server
@@ -226,7 +267,7 @@ int main(int argc, char *argv[]) {
 
     if (a.isRunning()) {
       if (options.is_empty()) {
-        qDebug() << "Clementine is already running - activating existing window";
+        qLog(Info) << "Clementine is already running - activating existing window";
       }
       if (a.sendMessage(options.Serialize(), 5000)) {
         return 0;
@@ -239,6 +280,20 @@ int main(int argc, char *argv[]) {
   UniversalEncodingHandler handler;
   TagLib::ID3v1::Tag::setStringHandler(&handler);
 
+#ifdef Q_OS_LINUX
+  // Force Clementine's menu to be shown in the Clementine window and not in
+  // the Unity global menubar thing.  See:
+  // https://bugs.launchpad.net/unity/+bug/775278
+  setenv("QT_X11_NO_NATIVE_MENUBAR", "1", true);
+#endif
+
+  // Initialise logging
+  logging::Init();
+  logging::SetLevels(options.log_levels());
+  g_log_set_default_handler(reinterpret_cast<GLogFunc>(&logging::GLog), NULL);
+
+  IncreaseFDLimit();
+
   QtSingleApplication a(argc, argv);
 #ifdef Q_OS_DARWIN
   QCoreApplication::setLibraryPaths(
@@ -260,10 +315,41 @@ int main(int argc, char *argv[]) {
   QCoreApplication::setAttribute(Qt::AA_DontShowIconsInMenus, true);
 #endif
 
+  // Set the permissions on the config file on Unix - it can contain passwords
+  // for internet services so it's important that other users can't read it.
+  // On Windows these are stored in the registry instead.
+#ifdef Q_OS_UNIX
+  {
+    QSettings s;
+
+    // Create the file if it doesn't exist already
+    if (!QFile::exists(s.fileName())) {
+      QFile file(s.fileName());
+      file.open(QIODevice::WriteOnly);
+    }
+
+    // Set -rw-------
+    QFile::setPermissions(s.fileName(), QFile::ReadOwner | QFile::WriteOwner);
+  }
+#endif
+
+#ifdef HAVE_QCA
+  QCA::Initializer qca_initializer;
+#endif
+
   // Resources
   Q_INIT_RESOURCE(data);
   Q_INIT_RESOURCE(translations);
 
+  // Grooveshark uses GoDaddy to sign its SSL certificates, which are in turn
+  // signed by a ValiCert CA.  This CA certificate isn't installed by default
+  // in Windows, it's only added by windows update, or manually browsing to a
+  // website with a certificate signed by ValiCert.  Here we explicitly add
+  // that CA to the default list used by QSslSocket, so it always works in
+  // Clementine.
+  QSslSocket::addDefaultCaCertificates(
+        QSslCertificate::fromPath(":/grooveshark-valicert-ca.pem", QSsl::Pem));
+
   // Has the user forced a different language?
   QString language = options.language();
   if (language.isEmpty()) {
@@ -291,22 +377,28 @@ int main(int argc, char *argv[]) {
   // Seed the random number generator
   srand(time(NULL));
 
+  // Initialize the repository of cover providers.  Last.fm registers itself
+  // when its service is created.
+  CoverProviders cover_providers;
+  cover_providers.AddProvider(new AmazonCoverProvider);
+
   // Create some key objects
   scoped_ptr<BackgroundThread<Database> > database(
       new BackgroundThreadImplementation<Database, Database>(NULL));
   database->Start(true);
   TaskManager task_manager;
   PlaylistManager playlists(&task_manager, NULL);
-  RadioModel radio_model(database.get(), &task_manager, NULL);
+  Player player(&playlists);
+  GlobalSearch global_search;
+  InternetModel internet_model(database.get(), &task_manager, &player,
+                               &cover_providers, &global_search, NULL);
 
-  // Get the last.fm service if it's available
-  LastFMService* lastfm_service = NULL;
-#ifdef HAVE_LIBLASTFM
-  lastfm_service = RadioModel::Service<LastFMService>();
-#endif
-
-  // Create the player
-  Player player(&playlists, lastfm_service);
+#ifdef Q_OS_LINUX
+  // In 11.04 Ubuntu decided that the system tray should be reserved for certain
+  // whitelisted applications.  Clementine will override this setting and insert
+  // itself into the list of whitelisted apps.
+  UbuntuUnityHack hack;
+#endif // Q_OS_LINUX
 
   // Create the tray icon and OSD
   scoped_ptr<SystemTrayIcon> tray_icon(SystemTrayIcon::CreateSystemTrayIcon());
@@ -325,6 +417,8 @@ int main(int argc, char *argv[]) {
   QObject::connect(&playlists, SIGNAL(CurrentSongChanged(Song)), &art_loader, SLOT(LoadArt(Song)));
   QObject::connect(&art_loader, SIGNAL(ThumbnailLoaded(Song, QString, QImage)),
                    &osd, SLOT(CoverArtPathReady(Song, QString)));
+
+  GlobalSearchService global_search_service(&global_search);
 #endif
 
   // Window
@@ -332,16 +426,43 @@ int main(int argc, char *argv[]) {
       database.get(),
       &task_manager,
       &playlists,
-      &radio_model,
+      &internet_model,
       &player,
       tray_icon.get(),
       &osd,
-      &art_loader);
+      &art_loader,
+      &cover_providers,
+      &global_search);
 #ifdef HAVE_DBUS
   QObject::connect(&mpris, SIGNAL(RaiseMainWindow()), &w, SLOT(Raise()));
 #endif
   QObject::connect(&a, SIGNAL(messageReceived(QByteArray)), &w, SLOT(CommandlineOptionsReceived(QByteArray)));
   w.CommandlineOptionsReceived(options);
 
-  return a.exec();
+  int ret = a.exec();
+
+#ifdef Q_OS_LINUX
+  // The nvidia driver would cause Clementine (or any application that used
+  // opengl) to use 100% cpu on shutdown.  See:
+  //   http://code.google.com/p/clementine-player/issues/detail?id=2088
+  //   https://bugs.gentoo.org/show_bug.cgi?id=375615
+  // Work around this problem by exiting immediately (and not running the buggy
+  // nvidia atexit shutdown handler) if we're using one of the affected versions
+  // of the nvidia driver.
+
+  QFile self_maps("/proc/self/maps");
+  if (self_maps.open(QIODevice::ReadOnly)) {
+    QByteArray data = self_maps.readAll();
+    if (data.contains("libnvidia-tls.so.285.03") ||
+        data.contains("libnvidia-tls.so.280.13") ||
+        data.contains("libnvidia-tls.so.275.28") ||
+        data.contains("libnvidia-tls.so.275.19")) {
+      qLog(Warning) << "Exiting immediately to work around NVIDIA driver bug";
+      _exit(ret);
+    }
+    self_maps.close();
+  }
+#endif
+
+  return ret;
 }
