@@ -20,23 +20,27 @@
 #include "playlistfilter.h"
 #include "playlistitemmimedata.h"
 #include "playlistundocommands.h"
+#include "playlistview.h"
 #include "queue.h"
 #include "songloaderinserter.h"
 #include "songmimedata.h"
 #include "songplaylistitem.h"
+#include "core/logging.h"
 #include "core/modelfuturewatcher.h"
+#include "core/timeconstants.h"
+#include "internet/jamendoplaylistitem.h"
+#include "internet/jamendoservice.h"
+#include "internet/magnatuneplaylistitem.h"
+#include "internet/magnatuneservice.h"
+#include "internet/internetmimedata.h"
+#include "internet/internetmodel.h"
+#include "internet/internetplaylistitem.h"
+#include "internet/internetsongmimedata.h"
+#include "internet/savedradio.h"
 #include "library/library.h"
 #include "library/librarybackend.h"
 #include "library/librarymodel.h"
 #include "library/libraryplaylistitem.h"
-#include "radio/jamendoplaylistitem.h"
-#include "radio/jamendoservice.h"
-#include "radio/magnatuneplaylistitem.h"
-#include "radio/magnatuneservice.h"
-#include "radio/radiomimedata.h"
-#include "radio/radiomodel.h"
-#include "radio/radioplaylistitem.h"
-#include "radio/savedradio.h"
 #include "smartplaylists/generator.h"
 #include "smartplaylists/generatorinserter.h"
 #include "smartplaylists/generatormimedata.h"
@@ -45,6 +49,7 @@
 #include <QBuffer>
 #include <QDirIterator>
 #include <QFileInfo>
+#include <QLinkedList>
 #include <QMimeData>
 #include <QMutableListIterator>
 #include <QSortFilterProxyModel>
@@ -52,8 +57,8 @@
 #include <QtConcurrentRun>
 #include <QtDebug>
 
-#include <boost/bind.hpp>
 #include <algorithm>
+#include <boost/bind.hpp>
 
 using smart_playlists::Generator;
 using smart_playlists::GeneratorInserter;
@@ -61,6 +66,7 @@ using smart_playlists::GeneratorPtr;
 
 using boost::shared_ptr;
 
+const char* Playlist::kCddaMimeType = "x-content/audio-cdda";
 const char* Playlist::kRowsMimetype = "application/x-clementine-playlist-rows";
 const char* Playlist::kPlayNowMimetype = "application/x-clementine-play-now";
 
@@ -72,10 +78,14 @@ const QRgb Playlist::kDynamicHistoryColor = qRgb(0x80, 0x80, 0x80);
 
 const char* Playlist::kSettingsGroup = "Playlist";
 
+const int Playlist::kUndoStackSize = 20;
+const int Playlist::kUndoItemLimit = 500;
+
 Playlist::Playlist(PlaylistBackend* backend,
                    TaskManager* task_manager,
                    LibraryBackend* library,
                    int id,
+                   const QString& special_type,
                    QObject *parent)
   : QAbstractListModel(parent),
     is_loading_(false),
@@ -89,11 +99,15 @@ Playlist::Playlist(PlaylistBackend* backend,
     current_virtual_index_(-1),
     is_shuffled_(false),
     scrobble_point_(-1),
-    has_scrobbled_(false),
+    lastfm_status_(LastFM_New),
+    have_incremented_playcount_(false),
     playlist_sequence_(NULL),
     ignore_sorting_(false),
-    undo_stack_(new QUndoStack(this))
+    undo_stack_(new QUndoStack(this)),
+    special_type_(special_type)
 {
+  undo_stack_->setUndoLimit(kUndoStackSize);
+
   connect(this, SIGNAL(rowsInserted(const QModelIndex&, int, int)), SIGNAL(PlaylistChanged()));
   connect(this, SIGNAL(rowsRemoved(const QModelIndex&, int, int)), SIGNAL(PlaylistChanged()));
 
@@ -111,19 +125,7 @@ Playlist::Playlist(PlaylistBackend* backend,
 
   connect(queue_, SIGNAL(layoutChanged()), SLOT(QueueLayoutChanged()));
 
-  column_alignments_[Column_Length]
-      = column_alignments_[Column_Track]
-      = column_alignments_[Column_Disc]
-      = column_alignments_[Column_Year]
-      = column_alignments_[Column_BPM]
-      = column_alignments_[Column_Bitrate]
-      = column_alignments_[Column_Samplerate]
-      = column_alignments_[Column_Filesize]
-      = column_alignments_[Column_PlayCount]
-      = column_alignments_[Column_SkipCount]
-      = (Qt::AlignRight | Qt::AlignVCenter);
-
-  column_alignments_[Column_Score] = (Qt::AlignCenter);
+  column_alignments_ = PlaylistView::DefaultColumnAlignment();
 }
 
 Playlist::~Playlist() {
@@ -255,7 +257,7 @@ QVariant Playlist::data(const QModelIndex& index, int role) const {
         case Column_Disc:         return song.disc();
         case Column_Year:         return song.year();
         case Column_Genre:        return song.genre();
-        case Column_AlbumArtist:  return song.albumartist();
+        case Column_AlbumArtist:  return song.playlist_albumartist();
         case Column_Composer:     return song.composer();
 
         case Column_Rating:       return song.rating();
@@ -267,7 +269,7 @@ QVariant Playlist::data(const QModelIndex& index, int role) const {
         case Column_BPM:          return song.bpm();
         case Column_Bitrate:      return song.bitrate();
         case Column_Samplerate:   return song.samplerate();
-        case Column_Filename:     return song.filename();
+        case Column_Filename:     return song.url();
         case Column_BaseFilename: return song.basefilename();
         case Column_Filesize:     return song.filesize();
         case Column_Filetype:     return song.filetype();
@@ -370,14 +372,14 @@ bool Playlist::FilterContainsVirtualIndex(int i) const {
   return proxy_->filterAcceptsRow(virtual_items_[i], QModelIndex());
 }
 
-int Playlist::NextVirtualIndex(int i) const {
+int Playlist::NextVirtualIndex(int i, bool ignore_repeat_track) const {
   PlaylistSequence::RepeatMode repeat_mode = playlist_sequence_->repeat_mode();
   PlaylistSequence::ShuffleMode shuffle_mode = playlist_sequence_->shuffle_mode();
   bool album_only = repeat_mode == PlaylistSequence::Repeat_Album ||
-                    shuffle_mode == PlaylistSequence::Shuffle_Album;
+                    shuffle_mode == PlaylistSequence::Shuffle_InsideAlbum;
 
   // This one's easy - if we have to repeat the current track then just return i
-  if (repeat_mode == PlaylistSequence::Repeat_Track) {
+  if (repeat_mode == PlaylistSequence::Repeat_Track && !ignore_repeat_track) {
     if (!FilterContainsVirtualIndex(i))
       return virtual_items_.count(); // It's not in the filter any more
     return i;
@@ -414,7 +416,7 @@ int Playlist::PreviousVirtualIndex(int i) const {
   PlaylistSequence::RepeatMode repeat_mode = playlist_sequence_->repeat_mode();
   PlaylistSequence::ShuffleMode shuffle_mode = playlist_sequence_->shuffle_mode();
   bool album_only = repeat_mode == PlaylistSequence::Repeat_Album ||
-                    shuffle_mode == PlaylistSequence::Shuffle_Album;
+                    shuffle_mode == PlaylistSequence::Shuffle_InsideAlbum;
 
   // This one's easy - if we have to repeat the current track then just return i
   if (repeat_mode == PlaylistSequence::Repeat_Track) {
@@ -450,7 +452,7 @@ int Playlist::PreviousVirtualIndex(int i) const {
   return -1;
 }
 
-int Playlist::next_row() const {
+int Playlist::next_row(bool ignore_repeat_track) const {
   // Did we want to stop after this track?
   if (stop_after_.isValid() && current_row() == stop_after_.row())
     return -1;
@@ -460,7 +462,7 @@ int Playlist::next_row() const {
     return queue_->PeekNext();
   }
 
-  int next_virtual_index = NextVirtualIndex(current_virtual_index_);
+  int next_virtual_index = NextVirtualIndex(current_virtual_index_, ignore_repeat_track);
   if (next_virtual_index >= virtual_items_.count()) {
     // We've gone off the end of the playlist.
 
@@ -472,7 +474,7 @@ int Playlist::next_row() const {
         break;
 
       default:
-        next_virtual_index = NextVirtualIndex(-1);
+        next_virtual_index = NextVirtualIndex(-1, ignore_repeat_track);
         break;
     }
   }
@@ -540,9 +542,7 @@ void Playlist::set_current_row(int i) {
   }
 
   if (current_item_index_.isValid()) {
-    InformOfCurrentSongChange(current_item_index_,
-                              current_item_index_.sibling(current_item_index_.row(), ColumnCount-1),
-                              current_item_metadata());
+    InformOfCurrentSongChange();
   }
 
   // Update the virtual index
@@ -566,17 +566,13 @@ void Playlist::set_current_row(int i) {
     using smart_playlists::Generator;
 
     // Add more dynamic playlist items
-    const int count = current_item_index_.row() + Generator::kDynamicFuture - items_.count();
+    const int count = current_item_index_.row() + dynamic_playlist_->GetDynamicFuture() - items_.count();
     if (count > 0) {
-      GeneratorInserter* inserter = new GeneratorInserter(task_manager_, library_, this);
-      connect(inserter, SIGNAL(Error(QString)), SIGNAL(LoadTracksError(QString)));
-      connect(inserter, SIGNAL(PlayRequested(QModelIndex)), SIGNAL(PlayRequested(QModelIndex)));
-
-      inserter->Load(this, -1, false, false, dynamic_playlist_, count);
+      InsertDynamicItems(count);
     }
 
     // Remove the first item
-    if (current_item_index_.row() > Generator::kDynamicHistory) {
+    if (current_item_index_.row() > dynamic_playlist_->GetDynamicHistory()) {
       RemoveItemsWithoutUndo(0, 1);
     }
   }
@@ -584,6 +580,14 @@ void Playlist::set_current_row(int i) {
   UpdateScrobblePoint();
 }
 
+void Playlist::InsertDynamicItems(int count) {
+  GeneratorInserter* inserter = new GeneratorInserter(task_manager_, library_, this);
+  connect(inserter, SIGNAL(Error(QString)), SIGNAL(LoadTracksError(QString)));
+  connect(inserter, SIGNAL(PlayRequested(QModelIndex)), SIGNAL(PlayRequested(QModelIndex)));
+
+  inserter->Load(this, -1, false, false, dynamic_playlist_, count);
+}
+
 Qt::ItemFlags Playlist::flags(const QModelIndex &index) const {
   Qt::ItemFlags flags = Qt::ItemIsEnabled | Qt::ItemIsSelectable;
 
@@ -633,9 +637,12 @@ bool Playlist::dropMimeData(const QMimeData* data, Qt::DropAction action, int ro
       InsertSongItems<JamendoPlaylistItem>(song_data->songs, row, play_now, enqueue_now);
     else
       InsertSongItems<SongPlaylistItem>(song_data->songs, row, play_now, enqueue_now);
-  } else if (const RadioMimeData* radio_data = qobject_cast<const RadioMimeData*>(data)) {
-    // Dragged from the Radio pane
-    InsertRadioStations(radio_data->model, radio_data->indexes,
+  } else if (const InternetMimeData* internet_data = qobject_cast<const InternetMimeData*>(data)) {
+    // Dragged from the Internet pane
+    InsertInternetItems(internet_data->model, internet_data->indexes,
+                        row, play_now, enqueue_now);
+  } else if (const InternetSongMimeData* internet_song_data = qobject_cast<const InternetSongMimeData*>(data)) {
+    InsertInternetItems(internet_song_data->service, internet_song_data->songs,
                         row, play_now, enqueue_now);
   } else if (const GeneratorMimeData* generator_data = qobject_cast<const GeneratorMimeData*>(data)) {
     InsertSmartPlaylist(generator_data->generator_, row, play_now, enqueue_now);
@@ -650,7 +657,7 @@ bool Playlist::dropMimeData(const QMimeData* data, Qt::DropAction action, int ro
     Playlist* source_playlist = NULL;
 
     QDataStream stream(data->data(kRowsMimetype));
-    stream >> source_playlist;
+    stream.readRawData(reinterpret_cast<char*>(&source_playlist), sizeof(source_playlist));
     stream >> source_rows;
     qStableSort(source_rows); // Make sure we take them in order
 
@@ -662,7 +669,13 @@ bool Playlist::dropMimeData(const QMimeData* data, Qt::DropAction action, int ro
       PlaylistItemList items;
       foreach (int row, source_rows)
         items << source_playlist->item_at(row);
-      undo_stack_->push(new PlaylistUndoCommands::InsertItems(this, items, row));
+
+      if (items.count() > kUndoItemLimit) {
+        // Too big to keep in the undo stack
+        InsertItemsWithoutUndo(items, row, false);
+      } else {
+        undo_stack_->push(new PlaylistUndoCommands::InsertItems(this, items, row));
+      }
 
       // Remove the items from the source playlist if it was a move event
       if (action == Qt::MoveAction) {
@@ -672,6 +685,10 @@ bool Playlist::dropMimeData(const QMimeData* data, Qt::DropAction action, int ro
         }
       }
     }
+  } else if (data->hasFormat(kCddaMimeType)) {
+    SongLoaderInserter* inserter = new SongLoaderInserter(task_manager_, library_);
+    connect(inserter, SIGNAL(Error(QString)), SIGNAL(LoadTracksError(QString)));
+    inserter->LoadAudioCD(this, row, play_now, enqueue_now);
   } else if (data->hasUrls()) {
     // URL list dragged from the file list or some other app
     InsertUrls(data->urls(), row, play_now, enqueue_now);
@@ -760,12 +777,17 @@ void Playlist::MoveItemsWithoutUndo(int start, const QList<int>& dest_rows) {
   layoutAboutToBeChanged();
   PlaylistItemList moved_items;
 
-  if (start == -1)
+  if (start == -1) {
     start = items_.count() - dest_rows.count();
+  } else {
+    foreach (int dest_row, dest_rows) {
+      if (start >= dest_row)
+        start--;
+    }
+  }
 
-  // Take the items out of the list first, keeping track of whether the
-  // insertion point changes
-  for (int i=start ; i<start + dest_rows.count() ; ++i)
+  // Take the items out of the list first
+  for (int i = 0; i < dest_rows.count(); i++)
     moved_items << items_.takeAt(start);
 
   // Put the items back in
@@ -810,13 +832,19 @@ void Playlist::InsertItems(const PlaylistItemList& itemsIn, int pos, bool play_n
   SongList songs;
 
   foreach(PlaylistItemPtr item, items) {
-    songs << item.get()->Metadata();
+    songs << item->Metadata();
   }
 
-  QList<Song> vetoed;
+  const int song_count = songs.length();
+  QSet<Song> vetoed;
   foreach(SongInsertVetoListener* listener, veto_listeners_) {
     foreach(const Song& song, listener->AboutToInsertSongs(GetAllSongs(), songs)) {
-      vetoed.append(song);
+      // avoid veto-ing a song multiple times
+      vetoed.insert(song);
+    }
+    if (vetoed.count() == song_count) {
+      // all songs were vetoed and there's nothing more to do (there's no need for an undo step)
+      return;
     }
   }
 
@@ -824,10 +852,10 @@ void Playlist::InsertItems(const PlaylistItemList& itemsIn, int pos, bool play_n
     QMutableListIterator<PlaylistItemPtr> it(items);
     while (it.hasNext()) {
       PlaylistItemPtr item = it.next();
-      const Song& current = item.get()->Metadata();
+      const Song& current = item->Metadata();
 
       if(vetoed.contains(current)) {
-        vetoed.removeOne(current);
+        vetoed.remove(current);
         it.remove();
       }
     }
@@ -839,7 +867,13 @@ void Playlist::InsertItems(const PlaylistItemList& itemsIn, int pos, bool play_n
   }
 
   const int start = pos == -1 ? items_.count() : pos;
-  undo_stack_->push(new PlaylistUndoCommands::InsertItems(this, items, pos, enqueue));
+
+  if (items.count() > kUndoItemLimit) {
+    // Too big to keep in the undo stack
+    InsertItemsWithoutUndo(items, pos, enqueue);
+  } else {
+    undo_stack_->push(new PlaylistUndoCommands::InsertItems(this, items, pos, enqueue));
+  }
 
   if (play_now)
     emit PlayRequested(index(start, 0));
@@ -905,24 +939,22 @@ void Playlist::InsertSongsOrLibraryItems(const SongList& songs, int pos, bool pl
   InsertItems(items, pos, play_now, enqueue);
 }
 
-void Playlist::InsertRadioStations(const RadioModel* model,
+void Playlist::InsertInternetItems(const InternetModel* model,
                                    const QModelIndexList& items,
                                    int pos, bool play_now, bool enqueue) {
   PlaylistItemList playlist_items;
   QList<QUrl> song_urls;
 
   foreach (const QModelIndex& item, items) {
-    switch (item.data(RadioModel::Role_PlayBehaviour).toInt()) {
-    case RadioModel::PlayBehaviour_SingleItem:
-      playlist_items << shared_ptr<PlaylistItem>(new RadioPlaylistItem(
+    switch (item.data(InternetModel::Role_PlayBehaviour).toInt()) {
+    case InternetModel::PlayBehaviour_SingleItem:
+      playlist_items << shared_ptr<PlaylistItem>(new InternetPlaylistItem(
           model->ServiceForIndex(item),
-          item.data(RadioModel::Role_Url).toUrl(),
-          item.data(RadioModel::Role_Title).toString(),
-          item.data(RadioModel::Role_Artist).toString()));
+          item.data(InternetModel::Role_SongMetadata).value<Song>()));
       break;
 
-    case RadioModel::PlayBehaviour_UseSongLoader:
-      song_urls << item.data(RadioModel::Role_Url).toUrl();
+    case InternetModel::PlayBehaviour_UseSongLoader:
+      song_urls << item.data(InternetModel::Role_Url).toUrl();
       break;
     }
   }
@@ -935,6 +967,66 @@ void Playlist::InsertRadioStations(const RadioModel* model,
   InsertItems(playlist_items, pos, play_now, enqueue);
 }
 
+void Playlist::InsertInternetItems(InternetService* service,
+                                   const SongList& songs,
+                                   int pos, bool play_now, bool enqueue) {
+  PlaylistItemList playlist_items;
+  foreach (const Song& song, songs) {
+    playlist_items << shared_ptr<PlaylistItem>(new InternetPlaylistItem(service, song));
+  }
+
+  InsertItems(playlist_items, pos, play_now, enqueue);
+}
+
+void Playlist::UpdateItems(const SongList& songs) {
+  qLog(Debug) << "Updating playlist with new tracks' info";
+  // We first convert our songs list into a linked list (a 'real' list),
+  // because removals are faster with QLinkedList.
+  // Next, we walk through the list of playlist's items then the list of songs
+  // we want to update: if an item corresponds to the song (we rely on URL for
+  // this), we update the item with the new metadata, then we remove song from
+  // our list because we will not need to check it again.
+  // And we also update undo actions.
+  QLinkedList<Song> songs_list;
+  foreach (const Song& song, songs) songs_list.append(song);
+
+  for (int i=0; i<items_.size(); i++) {
+    // Update current items list
+    QMutableLinkedListIterator<Song> it(songs_list);
+    while (it.hasNext()) {
+      const Song& song = it.next();
+      PlaylistItemPtr& item = items_[i];
+      if (item->Metadata().url() == song.url() &&
+          (item->Metadata().filetype() == Song::Type_Unknown ||
+            // Stream may change and may need to be updated too
+            item->Metadata().filetype() == Song::Type_Stream)) {
+        PlaylistItemPtr new_item;
+        if (song.id() == -1) {
+          new_item = PlaylistItemPtr(new SongPlaylistItem(song));
+        } else {
+          new_item = PlaylistItemPtr(new LibraryPlaylistItem(song));
+          library_items_by_id_.insertMulti(song.id(), new_item);
+        }
+        items_[i] = new_item;
+        emit dataChanged(index(i, 0), index(i, ColumnCount-1));
+        // Also update undo actions
+        for (int i=0; i<undo_stack_->count(); i++) {
+          QUndoCommand *undo_action = const_cast<QUndoCommand*>(undo_stack_->command(i));
+          PlaylistUndoCommands::InsertItems *undo_action_insert =
+            dynamic_cast<PlaylistUndoCommands::InsertItems*>(undo_action);
+          if (undo_action_insert) {
+            bool found_and_updated = undo_action_insert->UpdateItem(new_item);
+            if (found_and_updated) break;
+          }
+        }
+        it.remove();
+        break;
+      }
+    }
+  }
+  Save();
+}
+
 QMimeData* Playlist::mimeData(const QModelIndexList& indexes) const {
   QMimeData* data = new QMimeData;
 
@@ -951,7 +1043,9 @@ QMimeData* Playlist::mimeData(const QModelIndexList& indexes) const {
   QBuffer buf;
   buf.open(QIODevice::WriteOnly);
   QDataStream stream(&buf);
-  stream << this;
+
+  const Playlist* self = this;
+  stream.writeRawData(reinterpret_cast<char*>(&self), sizeof(self));
   stream << rows;
   buf.close();
 
@@ -980,7 +1074,7 @@ bool Playlist::CompareItems(int column, Qt::SortOrder order,
     case Column_Disc:         cmp(disc);
     case Column_Year:         cmp(year);
     case Column_Genre:        strcmp(genre);
-    case Column_AlbumArtist:  strcmp(albumartist);
+    case Column_AlbumArtist:  strcmp(playlist_albumartist);
     case Column_Composer:     strcmp(composer);
 
     case Column_Rating:       cmp(rating);
@@ -992,7 +1086,7 @@ bool Playlist::CompareItems(int column, Qt::SortOrder order,
     case Column_BPM:          cmp(bpm);
     case Column_Bitrate:      cmp(bitrate);
     case Column_Samplerate:   cmp(samplerate);
-    case Column_Filename:     cmp(filename);
+    case Column_Filename:     cmp(url);
     case Column_BaseFilename: cmp(basefilename);
     case Column_Filesize:     cmp(filesize);
     case Column_Filetype:     cmp(filetype);
@@ -1043,6 +1137,20 @@ QString Playlist::column_name(Column column) {
   return "";
 }
 
+QString Playlist::abbreviated_column_name(Column column) {
+  const QString& column_name = Playlist::column_name(column);
+  switch (column) {
+    case Column_Disc:
+    case Column_PlayCount:
+    case Column_SkipCount:
+    case Column_Track:
+      return QString("%1#").arg(column_name[0]);
+    default: 
+      return column_name;
+  }
+  return "";
+}
+
 void Playlist::sort(int column, Qt::SortOrder order) {
   if (ignore_sorting_)
     return;
@@ -1137,7 +1245,7 @@ void Playlist::ItemsLoaded() {
   while (it.hasNext()) {
     PlaylistItemPtr item = it.next();
 
-    if (item->IsLocalLibraryItem() && item->Metadata().filename().isEmpty()) {
+    if (item->IsLocalLibraryItem() && item->Metadata().url().isEmpty()) {
       it.remove();
     }
   }
@@ -1162,9 +1270,9 @@ void Playlist::ItemsLoaded() {
       if (p.dynamic_backend == library_->songs_table())
         backend = library_;
       else if (p.dynamic_backend == MagnatuneService::kSongsTable)
-        backend = RadioModel::Service<MagnatuneService>()->library_backend();
+        backend = InternetModel::Service<MagnatuneService>()->library_backend();
       else if (p.dynamic_backend == JamendoService::kSongsTable)
-        backend = RadioModel::Service<JamendoService>()->library_backend();
+        backend = InternetModel::Service<JamendoService>()->library_backend();
 
       if (backend) {
         gen->set_library(backend);
@@ -1215,7 +1323,42 @@ bool Playlist::removeRows(int row, int count, const QModelIndex& parent) {
     return false;
   }
 
-  undo_stack_->push(new PlaylistUndoCommands::RemoveItems(this, row, count));
+  if (count > kUndoItemLimit) {
+    // Too big to keep in the undo stack
+    RemoveItemsWithoutUndo(row, count);
+  } else {
+    undo_stack_->push(new PlaylistUndoCommands::RemoveItems(this, row, count));
+  }
+
+  return true;
+}
+
+bool Playlist::removeRows(QList<int>& rows) {
+  if(rows.isEmpty()) {
+    return false;
+  }
+
+  // start from the end to be sure that indices won't 'move' during
+  // the removal process
+  qSort(rows.begin(), rows.end(), qGreater<int>());
+
+  QList<int> part;
+  while(!rows.isEmpty()) {
+    // we're splitting the input list into sequences of consecutive
+    // numbers
+    part.append(rows.takeFirst());
+    while(!rows.isEmpty() && rows.first() == part.last() - 1) {
+      part.append(rows.takeFirst());
+    }
+
+    // and now we're removing the current sequence
+    if(!removeRows(part.last(), part.size())) {
+      return false;
+    }
+
+    part.clear();
+  }
+
   return true;
 }
 
@@ -1276,6 +1419,7 @@ void Playlist::StopAfter(int row) {
 }
 
 void Playlist::SetStreamMetadata(const QUrl& url, const Song& song) {
+  qLog(Debug) << "Setting metadata for" << url << "to" << song.artist() << song.title();
   if (!current_item_)
     return;
 
@@ -1290,9 +1434,7 @@ void Playlist::SetStreamMetadata(const QUrl& url, const Song& song) {
   current_item_->SetTemporaryMetadata(song);
   UpdateScrobblePoint();
 
-  InformOfCurrentSongChange(index(current_item_index_.row(), 0),
-                            index(current_item_index_.row(), ColumnCount-1),
-                            song);
+  InformOfCurrentSongChange();
 }
 
 void Playlist::ClearStreamMetadata() {
@@ -1335,11 +1477,20 @@ void Playlist::UpdateScrobblePoint() {
                              240ll * kNsecPerSec);
   }
 
-  has_scrobbled_ = false;
+  set_lastfm_status(LastFM_New);
+  have_incremented_playcount_ = false;
 }
 
 void Playlist::Clear() {
-  undo_stack_->push(new PlaylistUndoCommands::RemoveItems(this, 0, items_.count()));
+  const int count = items_.count();
+
+  if (count > kUndoItemLimit) {
+    // Too big to keep in the undo stack
+    RemoveItemsWithoutUndo(0, count);
+  } else {
+    undo_stack_->push(new PlaylistUndoCommands::RemoveItems(this, 0, count));
+  }
+
   TurnOffDynamicPlaylist();
 
   Save();
@@ -1364,6 +1515,13 @@ void Playlist::RepopulateDynamicPlaylist() {
   InsertSmartPlaylist(dynamic_playlist_);
 }
 
+void Playlist::ExpandDynamicPlaylist() {
+  if (!dynamic_playlist_)
+    return;
+
+  InsertDynamicItems(5);
+}
+
 void Playlist::RemoveItemsNotInQueue() {
   if (queue_->is_empty()) {
     RemoveItemsWithoutUndo(0, items_.count());
@@ -1402,9 +1560,15 @@ void Playlist::ReloadItems(const QList<int>& rows) {
     PlaylistItemPtr item = item_at(row);
 
     item->Reload();
-    InformOfCurrentSongChange(index(row, 0), index(row, ColumnCount-1),
-                              item->Metadata());
+
+    if (row == current_row()) {
+      InformOfCurrentSongChange();
+    } else {
+      emit dataChanged(index(row, 0), index(row, ColumnCount-1));
+    }
   }
+
+  Save();
 }
 
 void Playlist::RateSong(const QModelIndex& index, double rating) {
@@ -1429,8 +1593,7 @@ void Playlist::RemoveSongInsertVetoListener(SongInsertVetoListener* listener) {
 }
 
 void Playlist::SongInsertVetoListenerDestroyed() {
-  // qobject_cast returns NULL here for Python SIP listeners.
-  veto_listeners_.removeAll(static_cast<SongInsertVetoListener*>(sender()));
+  veto_listeners_.removeAll(qobject_cast<SongInsertVetoListener*>(sender()));
 }
 
 void Playlist::Shuffle() {
@@ -1460,17 +1623,88 @@ void Playlist::Shuffle() {
   Save();
 }
 
+namespace {
+bool AlbumShuffleComparator(const QMap<QString, int>& album_key_positions,
+                            const QMap<int, QString>& album_keys,
+                            int left, int right) {
+  const int left_pos = album_key_positions[album_keys[left]];
+  const int right_pos = album_key_positions[album_keys[right]];
+
+  if (left_pos == right_pos)
+    return left < right;
+  return left_pos < right_pos;
+}
+}
+
 void Playlist::ReshuffleIndices() {
-  if (!is_shuffled_) {
+  if (!playlist_sequence_) {
+    return;
+  }
+
+  if (playlist_sequence_->shuffle_mode() == PlaylistSequence::Shuffle_Off) {
+    // No shuffling - sort the virtual item list normally.
     std::sort(virtual_items_.begin(), virtual_items_.end());
     if (current_row() != -1)
       current_virtual_index_ = virtual_items_.indexOf(current_row());
-  } else {
-    QList<int>::iterator begin = virtual_items_.begin();
-    if (current_virtual_index_ != -1)
-      std::advance(begin, current_virtual_index_ + 1);
+    return;
+  }
 
-    std::random_shuffle(begin, virtual_items_.end());
+  // If the user is already playing a song, advance the begin iterator to
+  // only shuffle items that haven't been played yet.
+  QList<int>::iterator begin = virtual_items_.begin();
+  QList<int>::iterator end = virtual_items_.end();
+  if (current_virtual_index_ != -1)
+    std::advance(begin, current_virtual_index_ + 1);
+
+  switch (playlist_sequence_->shuffle_mode()) {
+  case PlaylistSequence::Shuffle_Off:
+    // Handled above.
+    break;
+
+  case PlaylistSequence::Shuffle_All:
+  case PlaylistSequence::Shuffle_InsideAlbum:
+    std::random_shuffle(begin, end);
+    break;
+
+  case PlaylistSequence::Shuffle_Albums: {
+    QMap<int, QString> album_keys; // real index -> key
+    QSet<QString> album_key_set;   // unique keys
+
+    // Find all the unique albums in the playlist
+    for (QList<int>::iterator it = begin ; it != end ; ++it) {
+      const int index = *it;
+      const QString key = items_[index]->Metadata().AlbumKey();
+      album_keys[index] = key;
+      album_key_set << key;
+    }
+
+    // Shuffle them
+    QStringList shuffled_album_keys = album_key_set.toList();
+    std::random_shuffle(shuffled_album_keys.begin(),
+                        shuffled_album_keys.end());
+
+    // If the user is currently playing a song, force its album to be first.
+    if (current_virtual_index_ != -1) {
+      const QString key = items_[current_row()]->Metadata().AlbumKey();
+      const int pos = shuffled_album_keys.indexOf(key);
+      if (pos >= 1) {
+        std::swap(shuffled_album_keys[0], shuffled_album_keys[pos]);
+      }
+    }
+
+    // Create album key -> position mapping for fast lookup
+    QMap<QString, int> album_key_positions;
+    for (int i=0 ; i<shuffled_album_keys.count() ; ++i) {
+      album_key_positions[shuffled_album_keys[i]] = i;
+    }
+
+    // Sort the virtual items
+    std::stable_sort(begin, end,
+                     boost::bind(AlbumShuffleComparator, album_key_positions,
+                                 album_keys, _1, _2));
+
+    break;
+  }
   }
 }
 
@@ -1538,16 +1772,6 @@ void Playlist::QueueLayoutChanged() {
   }
 }
 
-QDataStream& operator <<(QDataStream& s, const Playlist* p) {
-  s.writeRawData(reinterpret_cast<char*>(&p), sizeof(p));
-  return s;
-}
-
-QDataStream& operator >>(QDataStream& s, Playlist*& p) {
-  s.readRawData(reinterpret_cast<char*>(&p), sizeof(p));
-  return s;
-}
-
 void Playlist::ItemChanged(PlaylistItemPtr item) {
   for (int row=0 ; row<items_.count() ; ++row) {
     if (items_[row] == item) {
@@ -1557,28 +1781,14 @@ void Playlist::ItemChanged(PlaylistItemPtr item) {
   }
 }
 
-void Playlist::set_column_alignments(const ColumnAlignmentMap& column_alignments) {
-  column_alignments_ = column_alignments;
-}
+void Playlist::InformOfCurrentSongChange() {
+  emit dataChanged(index(current_item_index_.row(), 0),
+                   index(current_item_index_.row(), ColumnCount-1));
 
-void Playlist::set_column_align_left(int column) {
-  column_alignments_[column] = (Qt::AlignLeft | Qt::AlignVCenter);
-}
-
-void Playlist::set_column_align_center(int column) {
-  column_alignments_[column] = Qt::AlignCenter;
-}
-
-void Playlist::set_column_align_right(int column) {
-  column_alignments_[column] = (Qt::AlignRight | Qt::AlignVCenter);
-}
-
-void Playlist::InformOfCurrentSongChange(const QModelIndex& top_left, const QModelIndex& bottom_right,
-                                         const Song& metadata) {
-  emit dataChanged(top_left, bottom_right);
   // if the song is invalid, we won't play it - there's no point in
   // informing anybody about the change
-  if(metadata.is_valid()) {
+  const Song metadata(current_item_metadata());
+  if (metadata.is_valid()) {
     emit CurrentSongChanged(metadata);
   }
 }
@@ -1591,7 +1801,7 @@ void Playlist::InvalidateDeletedSongs() {
     Song song = item->Metadata();
 
     if(!song.is_stream()) {
-      bool exists = QFile::exists(song.filename());
+      bool exists = QFile::exists(song.url().toLocalFile());
 
       if(!exists && !item->HasForegroundColor(kInvalidSongPriority)) {
         // gray out the song if it's not there
@@ -1607,6 +1817,21 @@ void Playlist::InvalidateDeletedSongs() {
   ReloadItems(invalidated_rows);
 }
 
+void Playlist::RemoveDeletedSongs() {
+  QList<int> rows_to_remove;
+
+  for (int row = 0; row < items_.count(); ++row) {
+    PlaylistItemPtr item = items_[row];
+    Song song = item->Metadata();
+
+    if(!song.is_stream() && !QFile::exists(song.url().toLocalFile())) {
+      rows_to_remove.append(row);
+    }
+  }
+
+  removeRows(rows_to_remove);
+}
+
 bool Playlist::ApplyValidityOnCurrentSong(const QUrl& url, bool valid) {
   PlaylistItemPtr current = current_item();
 
@@ -1615,8 +1840,9 @@ bool Playlist::ApplyValidityOnCurrentSong(const QUrl& url, bool valid) {
 
     // if validity has changed, reload the item
     if(!current_song.is_stream() &&
-        current_song.filename() == url.toLocalFile() &&
-        current_song.is_valid() != QFile::exists(current_song.filename())) {
+        !current_song.is_cdda() &&
+        current_song.url() == url &&
+        current_song.is_valid() != QFile::exists(current_song.url().toLocalFile())) {
       ReloadItems(QList<int>() << current_row());
     }
 
@@ -1630,3 +1856,7 @@ bool Playlist::ApplyValidityOnCurrentSong(const QUrl& url, bool valid) {
 
   return current;
 }
+
+void Playlist::SetColumnAlignment(const ColumnAlignmentMap& alignment) {
+  column_alignments_ = alignment;
+}
