@@ -38,6 +38,8 @@ using smart_playlists::GeneratorPtr;
 
 using boost::shared_ptr;
 
+const int PlaylistBackend::kSongTableJoins = 4;
+
 PlaylistBackend::PlaylistBackend(QObject* parent)
   : QObject(parent),
     library_(NULL)
@@ -55,7 +57,8 @@ PlaylistBackend::PlaylistList PlaylistBackend::GetAllPlaylists() {
   PlaylistList ret;
 
   QSqlQuery q("SELECT ROWID, name, last_played, dynamic_playlist_type,"
-              "       dynamic_playlist_data, dynamic_playlist_backend"
+              "       dynamic_playlist_data, dynamic_playlist_backend,"
+              "       special_type"
               " FROM playlists"
               " ORDER BY ui_order", db);
   q.exec();
@@ -70,6 +73,7 @@ PlaylistBackend::PlaylistList PlaylistBackend::GetAllPlaylists() {
     p.dynamic_type = q.value(3).toString();
     p.dynamic_data = q.value(4).toByteArray();
     p.dynamic_backend = q.value(5).toString();
+    p.special_type = q.value(6).toString();
     ret << p;
   }
 
@@ -81,7 +85,8 @@ PlaylistBackend::Playlist PlaylistBackend::GetPlaylist(int id) {
   QSqlDatabase db(db_->Connect());
 
   QSqlQuery q("SELECT ROWID, name, last_played, dynamic_playlist_type,"
-              "       dynamic_playlist_data, dynamic_playlist_backend"
+              "       dynamic_playlist_data, dynamic_playlist_backend,"
+              "       special_type"
               " FROM playlists"
               " WHERE ROWID=:id", db);
   q.bindValue(":id", id);
@@ -98,6 +103,7 @@ PlaylistBackend::Playlist PlaylistBackend::GetPlaylist(int id) {
   p.dynamic_type = q.value(3).toString();
   p.dynamic_data = q.value(4).toByteArray();
   p.dynamic_backend = q.value(5).toString();
+  p.special_type = q.value(6).toString();
 
   return p;
 }
@@ -109,8 +115,8 @@ QFuture<PlaylistItemPtr> PlaylistBackend::GetPlaylistItems(int playlist) {
   QString query = "SELECT songs.ROWID, " + Song::JoinSpec("songs") + ","
                   "       magnatune_songs.ROWID, " + Song::JoinSpec("magnatune_songs") + ","
                   "       jamendo_songs.ROWID, " + Song::JoinSpec("jamendo_songs") + ","
-                  "       p.type, p.url, p.title, p.artist, p.album, p.length,"
-                  "       p.radio_service, p.beginning, p.cue_path"
+                  "       p.ROWID, " + Song::JoinSpec("p") + ","
+                  "       p.type, p.radio_service"
                   " FROM playlist_items AS p"
                   " LEFT JOIN songs"
                   "    ON p.library_id = songs.ROWID"
@@ -140,7 +146,7 @@ QFuture<PlaylistItemPtr> PlaylistBackend::GetPlaylistItems(int playlist) {
 
 PlaylistItemPtr PlaylistBackend::NewSongFromQuery(const SqlRow& row, boost::shared_ptr<NewSongFromQueryState> state) {
   // The song tables get joined first, plus one each for the song ROWIDs
-  const int playlist_row = (Song::kColumns.count() + 1) * 3;
+  const int playlist_row = (Song::kColumns.count() + 1) * kSongTableJoins;
 
   PlaylistItemPtr item(PlaylistItem::NewFromType(row.value(playlist_row).toString()));
   if (item) {
@@ -190,7 +196,7 @@ PlaylistItemPtr PlaylistBackend::RestoreCueData(PlaylistItemPtr item, boost::sha
   }
 
   foreach(const Song& from_list, song_list) {
-    if(from_list.filename() == song.filename() &&
+    if(from_list.url() == song.url() &&
        from_list.beginning_nanosec() == song.beginning_nanosec()) {
       // we found a matching section; replace the input
       // item with a new one containing CUE metadata
@@ -219,9 +225,10 @@ void PlaylistBackend::SavePlaylist(int playlist, const PlaylistItemList& items,
 
   QSqlQuery clear("DELETE FROM playlist_items WHERE playlist = :playlist", db);
   QSqlQuery insert("INSERT INTO playlist_items"
-                   " (playlist, type, library_id, url, title, artist, album,"
-                   "  length, radio_service, beginning, cue_path)"
-                   " VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", db);
+                   " (playlist, type, library_id, radio_service, " +
+                      Song::kColumnSpec + ")"
+                   " VALUES (:playlist, :type, :library_id, :radio_service, " +
+                             Song::kBindSpec + ")", db);
   QSqlQuery update("UPDATE playlists SET "
                    "   last_played=:last_played,"
                    "   dynamic_playlist_type=:dynamic_type,"
@@ -239,7 +246,7 @@ void PlaylistBackend::SavePlaylist(int playlist, const PlaylistItemList& items,
 
   // Save the new ones
   foreach (PlaylistItemPtr item, items) {
-    insert.bindValue(0, playlist);
+    insert.bindValue(":playlist", playlist);
     item->BindToQuery(&insert);
 
     insert.exec();
@@ -265,12 +272,15 @@ void PlaylistBackend::SavePlaylist(int playlist, const PlaylistItemList& items,
   transaction.Commit();
 }
 
-int PlaylistBackend::CreatePlaylist(const QString &name) {
+int PlaylistBackend::CreatePlaylist(const QString &name,
+                                    const QString& special_type) {
   QMutexLocker l(db_->Mutex());
   QSqlDatabase db(db_->Connect());
 
-  QSqlQuery q("INSERT INTO playlists (name) VALUES (:name)", db);
+  QSqlQuery q("INSERT INTO playlists (name, special_type)"
+              " VALUES (:name, :special_type)", db);
   q.bindValue(":name", name);
+  q.bindValue(":special_type", special_type);
   q.exec();
   if (db_->CheckErrors(q))
     return -1;
