@@ -17,19 +17,22 @@
 
 #include "playlistcontainer.h"
 #include "playlistmanager.h"
+#include "specialplaylisttype.h"
 #include "ui_playlistcontainer.h"
+#include "core/logging.h"
 #include "playlistparsers/playlistparser.h"
 #include "ui/iconloader.h"
+#include "widgets/didyoumean.h"
 #include "widgets/maclineedit.h"
 
-#include <QUndoStack>
-#include <QInputDialog>
-#include <QSettings>
-#include <QTimeLine>
-#include <QSortFilterProxyModel>
-#include <QLabel>
 #include <QFileDialog>
+#include <QInputDialog>
+#include <QLabel>
 #include <QMessageBox>
+#include <QSettings>
+#include <QSortFilterProxyModel>
+#include <QTimeLine>
+#include <QUndoStack>
 
 const char* PlaylistContainer::kSettingsGroup = "Playlist";
 
@@ -39,14 +42,16 @@ PlaylistContainer::PlaylistContainer(QWidget *parent)
     manager_(NULL),
     undo_(NULL),
     redo_(NULL),
+    playlist_(NULL),
     starting_up_(true),
     tab_bar_visible_(false),
     tab_bar_animation_(new QTimeLine(500, this)),
-    no_matches_label_(new QLabel(this))
+    no_matches_label_(NULL),
+    did_you_mean_(NULL)
 {
   ui_->setupUi(this);
 
-  no_matches_label_->setText(tr("No matches found.  Clear the search box to show the whole playlist again."));
+  no_matches_label_ = new QLabel(ui_->playlist);
   no_matches_label_->setAlignment(Qt::AlignTop | Qt::AlignHCenter);
   no_matches_label_->setAttribute(Qt::WA_TransparentForMouseEvents);
   no_matches_label_->setWordWrap(true);
@@ -55,9 +60,9 @@ PlaylistContainer::PlaylistContainer(QWidget *parent)
 
   // Set the colour of the no matches label to the disabled text colour
   QPalette no_matches_palette = no_matches_label_->palette();
-  no_matches_palette.setColor(
-      QPalette::Normal, QPalette::WindowText,
-      no_matches_palette.color(QPalette::Disabled, QPalette::Text));
+  const QColor no_matches_color = no_matches_palette.color(QPalette::Disabled, QPalette::Text);
+  no_matches_palette.setColor(QPalette::Normal, QPalette::WindowText, no_matches_color);
+  no_matches_palette.setColor(QPalette::Inactive, QPalette::WindowText, no_matches_color);
   no_matches_label_->setPalette(no_matches_palette);
 
   // Make it bold
@@ -93,6 +98,9 @@ PlaylistContainer::PlaylistContainer(QWidget *parent)
   connect(ui_->playlist, SIGNAL(FocusOnFilterSignal(QKeyEvent*)), SLOT(FocusOnFilter(QKeyEvent*)));
   ui_->filter->installEventFilter(this);
 #endif
+
+  did_you_mean_ = new DidYouMean(filter_->widget(), this);
+  connect(did_you_mean_, SIGNAL(Accepted(QString)), SLOT(DidYouMeanAccepted(QString)));
 }
 
 PlaylistContainer::~PlaylistContainer() {
@@ -104,7 +112,8 @@ PlaylistView* PlaylistContainer::view() const {
 }
 
 void PlaylistContainer::SetActions(
-    QAction* new_playlist, QAction* save_playlist, QAction* load_playlist) {
+    QAction* new_playlist, QAction* save_playlist, QAction* load_playlist,
+    QAction* next_playlist, QAction* previous_playlist) {
   ui_->create_new->setDefaultAction(new_playlist);
   ui_->save->setDefaultAction(save_playlist);
   ui_->load->setDefaultAction(load_playlist);
@@ -114,6 +123,8 @@ void PlaylistContainer::SetActions(
   connect(new_playlist, SIGNAL(triggered()), SLOT(NewPlaylist()));
   connect(save_playlist, SIGNAL(triggered()), SLOT(SavePlaylist()));
   connect(load_playlist, SIGNAL(triggered()), SLOT(LoadPlaylist()));
+  connect(next_playlist, SIGNAL(triggered()), SLOT(GoToNextPlaylistTab()));
+  connect(previous_playlist, SIGNAL(triggered()), SLOT(GoToPreviousPlaylistTab()));
 }
 
 void PlaylistContainer::SetManager(PlaylistManager *manager) {
@@ -144,6 +155,24 @@ void PlaylistContainer::SetViewModel(Playlist* playlist) {
     disconnect(view()->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
                this, SLOT(SelectionChanged()));
   }
+  if (playlist_ && playlist_->proxy()) {
+    disconnect(playlist_->proxy(), SIGNAL(modelReset()),
+               this, SLOT(UpdateNoMatchesLabel()));
+    disconnect(playlist_->proxy(), SIGNAL(rowsInserted(QModelIndex,int,int)),
+               this, SLOT(UpdateNoMatchesLabel()));
+    disconnect(playlist_->proxy(), SIGNAL(rowsRemoved(QModelIndex,int,int)),
+               this, SLOT(UpdateNoMatchesLabel()));
+  }
+  if (playlist_) {
+    disconnect(playlist_, SIGNAL(modelReset()),
+               this, SLOT(UpdateNoMatchesLabel()));
+    disconnect(playlist_, SIGNAL(rowsInserted(QModelIndex,int,int)),
+               this, SLOT(UpdateNoMatchesLabel()));
+    disconnect(playlist_, SIGNAL(rowsRemoved(QModelIndex,int,int)),
+               this, SLOT(UpdateNoMatchesLabel()));
+  }
+
+  playlist_ = playlist;
 
   // Set the view
   playlist->IgnoreSorting(true);
@@ -160,6 +189,15 @@ void PlaylistContainer::SetViewModel(Playlist* playlist) {
   // Update filter
   filter_->set_text(playlist->proxy()->filterRegExp().pattern());
 
+  // Update the no matches label
+  connect(playlist_->proxy(), SIGNAL(modelReset()), SLOT(UpdateNoMatchesLabel()));
+  connect(playlist_->proxy(), SIGNAL(rowsInserted(QModelIndex,int,int)), SLOT(UpdateNoMatchesLabel()));
+  connect(playlist_->proxy(), SIGNAL(rowsRemoved(QModelIndex,int,int)), SLOT(UpdateNoMatchesLabel()));
+  connect(playlist_, SIGNAL(modelReset()), SLOT(UpdateNoMatchesLabel()));
+  connect(playlist_, SIGNAL(rowsInserted(QModelIndex,int,int)), SLOT(UpdateNoMatchesLabel()));
+  connect(playlist_, SIGNAL(rowsRemoved(QModelIndex,int,int)), SLOT(UpdateNoMatchesLabel()));
+  UpdateNoMatchesLabel();
+
   // Ensure that tab is current
   if (ui_->tab_bar->current_id() != manager_->current_id())
     ui_->tab_bar->set_current_id(manager_->current_id());
@@ -178,6 +216,12 @@ void PlaylistContainer::SetViewModel(Playlist* playlist) {
   ui_->redo->setDefaultAction(redo_);
 
   emit UndoRedoActionsChanged(undo_, redo_);
+
+  did_you_mean()->hide();
+
+  // Implement special playlist behaviour
+  const SpecialPlaylistType* type = manager_->GetPlaylistType(playlist->special_type());
+  filter_->set_hint(type->search_hint_text(playlist));
 }
 
 void PlaylistContainer::ActivePlaying() {
@@ -195,7 +239,12 @@ void PlaylistContainer::ActiveStopped() {
 void PlaylistContainer::UpdateActiveIcon(const QIcon& icon) {
   // Unset all existing icons
   for (int i=0 ; i<ui_->tab_bar->count() ; ++i) {
-    ui_->tab_bar->setTabIcon(i, QIcon());
+    // Get the default icon for this tab
+    const int id = ui_->tab_bar->tabData(i).toInt();
+    Playlist* playlist = manager_->playlist(id);
+    const SpecialPlaylistType* type = manager_->GetPlaylistType(playlist->special_type());
+
+    ui_->tab_bar->setTabIcon(i, type->icon(playlist));
   }
 
   // Set our icon
@@ -204,8 +253,12 @@ void PlaylistContainer::UpdateActiveIcon(const QIcon& icon) {
 }
 
 void PlaylistContainer::PlaylistAdded(int id, const QString &name) {
-  int index = ui_->tab_bar->count();
-  ui_->tab_bar->InsertTab(id, index, name);
+  Playlist* playlist = manager_->playlist(id);
+  const SpecialPlaylistType* type = manager_->GetPlaylistType(playlist->special_type());
+
+  const int index = ui_->tab_bar->count();
+  const QIcon icon = type->icon(playlist);
+  ui_->tab_bar->InsertTab(id, index, name, icon);
 
   // Are we startup up, should we select this tab?
   if (starting_up_ && settings_.value("current_playlist", 1).toInt() == id) {
@@ -241,7 +294,7 @@ void PlaylistContainer::PlaylistRenamed(int id, const QString &new_name) {
 }
 
 void PlaylistContainer::NewPlaylist() {
-  manager_->New(PromptForPlaylistName());
+  manager_->New(tr("Playlist"));
 }
 
 void PlaylistContainer::LoadPlaylist() {
@@ -295,6 +348,23 @@ void PlaylistContainer::SavePlaylist(int id = -1) {
   manager_->Save(id == -1 ? manager_->current_id() : id, filename);
 }
 
+void PlaylistContainer::GoToNextPlaylistTab() {
+  // Get the next tab' id
+  int id_next =
+    ui_->tab_bar->id_of((ui_->tab_bar->currentIndex()+1)%ui_->tab_bar->count());
+  // Switch to next tab
+  manager_->SetCurrentPlaylist(id_next);
+}
+
+void PlaylistContainer::GoToPreviousPlaylistTab() {
+  // Get the next tab' id
+  int id_previous = 
+    ui_->tab_bar->id_of((ui_->tab_bar->currentIndex()+ui_->tab_bar->count()-1)
+                          % ui_->tab_bar->count());
+  // Switch to next tab
+  manager_->SetCurrentPlaylist(id_previous);
+}
+
 void PlaylistContainer::Save() {
   if (starting_up_)
     return;
@@ -316,15 +386,43 @@ void PlaylistContainer::SetTabBarHeight(int height) {
 }
 
 void PlaylistContainer::UpdateFilter() {
-  manager_->current()->proxy()->setFilterFixedString(filter_->text());
-  ui_->playlist->JumpToCurrentlyPlayingTrack();
+  Playlist* playlist = manager_->current();
+  SpecialPlaylistType* type = manager_->GetPlaylistType(playlist->special_type());
 
-  bool no_matches = manager_->current()->proxy()->rowCount() == 0 &&
-                    manager_->current()->rowCount() > 0;
+  did_you_mean()->hide();
 
-  if (no_matches)
+  if (type->has_special_search_behaviour(playlist)) {
+    type->Search(filter_->text(), playlist);
+  } else {
+    manager_->current()->proxy()->setFilterFixedString(filter_->text());
+    ui_->playlist->JumpToCurrentlyPlayingTrack();
+  }
+
+  UpdateNoMatchesLabel();
+}
+
+void PlaylistContainer::UpdateNoMatchesLabel() {
+  Playlist* playlist = manager_->current();
+  SpecialPlaylistType* type = manager_->GetPlaylistType(playlist->special_type());
+  const QString empty_text = type->empty_playlist_text(playlist);
+
+  const bool has_rows = playlist->rowCount() != 0;
+  const bool has_results = playlist->proxy()->rowCount() != 0;
+
+  QString text;
+  if (!empty_text.isEmpty() && !has_results) {
+    text = empty_text;
+  } else if (has_rows && !has_results) {
+    text = tr("No matches found.  Clear the search box to show the whole playlist again.");
+  }
+
+  if (!text.isEmpty()) {
+    no_matches_label_->setText(text);
     RepositionNoMatchesLabel(true);
-  no_matches_label_->setVisible(no_matches);
+    no_matches_label_->show();
+  } else {
+    no_matches_label_->hide();
+  }
 }
 
 void PlaylistContainer::resizeEvent(QResizeEvent* e) {
@@ -343,7 +441,7 @@ void PlaylistContainer::RepositionNoMatchesLabel(bool force) {
 
   const int kBorder = 10;
 
-  QPoint pos = ui_->playlist->viewport()->mapTo(this, QPoint(kBorder, kBorder));
+  QPoint pos = ui_->playlist->viewport()->mapTo(ui_->playlist, QPoint(kBorder, kBorder));
   QSize size = ui_->playlist->viewport()->size();
   size.setWidth(size.width() - kBorder * 2);
   size.setHeight(size.height() - kBorder * 2);
@@ -381,8 +479,10 @@ bool PlaylistContainer::eventFilter(QObject *objectWatched, QEvent *event) {
   return QWidget::eventFilter(objectWatched, event);
 }
 
-QString PlaylistContainer::PromptForPlaylistName() {
-  return QInputDialog::getText(this, tr("New playlist"),
-                                       tr("Enter a name for the new playlist"),
-                                       QLineEdit::Normal, tr("Playlist"));
+void PlaylistContainer::DidYouMeanAccepted(const QString& text) {
+  filter_->set_text(text);
+
+  Playlist* playlist = manager_->current();
+  SpecialPlaylistType* type = manager_->GetPlaylistType(playlist->special_type());
+  type->DidYouMeanClicked(text, playlist);
 }
