@@ -19,14 +19,15 @@
 #define PLAYLISTITEM_H
 
 #include <QMap>
+#include <QMetaType>
 #include <QStandardItem>
 #include <QUrl>
 
 #include <boost/enable_shared_from_this.hpp>
-#include <boost/shared_ptr.hpp>
 
 #include "core/song.h"
 
+class QAction;
 class SqlRow;
 
 class PlaylistItem : public boost::enable_shared_from_this<PlaylistItem> {
@@ -41,59 +42,23 @@ class PlaylistItem : public boost::enable_shared_from_this<PlaylistItem> {
   enum Option {
     Default = 0x00,
 
-    // The URL returned by Url() isn't the actual URL of the music - the
-    // item needs to do something special before it can get an actual URL.
-    // Causes StartLoading() to get called when the user wants to play.
-    SpecialPlayBehaviour = 0x01,
-
-    // This item might be able to provide another track after one finishes, for
-    // example in a radio stream.  Causes LoadNext() to get called when the
-    // next URL is required.
-    ContainsMultipleTracks = 0x02,
-
     // Disables the "pause" action.
-    PauseDisabled = 0x04,
+    PauseDisabled = 0x01,
 
     // Enables the last.fm "ban" action.
-    LastFMControls = 0x08,
+    LastFMControls = 0x02,
+
+    // Disables the seek slider.
+    SeekDisabled = 0x04,
   };
   Q_DECLARE_FLAGS(Options, Option);
 
-  // Returned by StartLoading() and LoadNext(), indicates what the player
-  // should do when it wants to load a playlist item that is marked
-  // SpecialPlayBehaviour or ContainsMultipleTracks.
-  struct SpecialLoadResult {
-    enum Type {
-      // There wasn't a track available, and the player should move on to the
-      // next playlist item.
-      NoMoreTracks,
-
-      // There might be another track available, something will call the
-      // player's HandleSpecialLoad() slot later with the same original_url.
-      WillLoadAsynchronously,
-
-      // There was a track available.  Its url is in media_url.
-      TrackAvailable,
-    };
-
-    SpecialLoadResult(Type type = NoMoreTracks,
-                      const QUrl& original_url = QUrl(),
-                      const QUrl& media_url = QUrl());
-
-    Type type_;
-
-    // The url that the playlist items has in Url().
-    // Might be something unplayable like lastfm://...
-    QUrl original_url_;
-
-    // The actual url to something that gstreamer can play.
-    QUrl media_url_;
-  };
-
   virtual QString type() const { return type_; }
 
   virtual Options options() const { return Default; }
 
+  virtual QList<QAction*> actions() { return QList<QAction*>(); }
+
   virtual bool InitFromQuery(const SqlRow& query) = 0;
   void BindToQuery(QSqlQuery* query) const;
   virtual void Reload() {}
@@ -102,14 +67,6 @@ class PlaylistItem : public boost::enable_shared_from_this<PlaylistItem> {
   virtual Song Metadata() const = 0;
   virtual QUrl Url() const = 0;
 
-  // Called by the Player if SpecialPlayBehaviour is set - gives the playlist
-  // item a chance to do something clever to get a playable track.
-  virtual SpecialLoadResult StartLoading() { return SpecialLoadResult(); }
-
-  // Called by the player if ContainsMultipleTracks is set - gives the playlist
-  // item a chance to get another track to play.
-  virtual SpecialLoadResult LoadNext() { return SpecialLoadResult(); }
-
   void SetTemporaryMetadata(const Song& metadata);
   void ClearTemporaryMetadata();
   bool HasTemporaryMetadata() const { return temp_metadata_.is_valid(); }
@@ -131,25 +88,19 @@ class PlaylistItem : public boost::enable_shared_from_this<PlaylistItem> {
   // Convenience function to find out whether this item is from the local
   // library, as opposed to a device, a file on disk, or a stream.
   // Remember that even if this returns true, the library item might be
-  // invalid so you might want to check that it's id is not equal to -1
+  // invalid so you might want to check that its id is not equal to -1
   // before actually using it.
   virtual bool IsLocalLibraryItem() const { return false; }
 
  protected:
   enum DatabaseColumn {
     Column_LibraryId,
-    Column_Url,
-    Column_Title,
-    Column_Artist,
-    Column_Album,
-    Column_Length,
-    Column_RadioService,
-    Column_Beginning,
-    Column_CuePath,
+    Column_InternetService,
   };
 
   virtual QVariant DatabaseValue(DatabaseColumn) const {
     return QVariant(QVariant::String); }
+  virtual Song DatabaseSongMetadata() const { return Song(); }
 
   QString type_;
 
@@ -161,6 +112,8 @@ class PlaylistItem : public boost::enable_shared_from_this<PlaylistItem> {
 typedef boost::shared_ptr<PlaylistItem> PlaylistItemPtr;
 typedef QList<PlaylistItemPtr> PlaylistItemList;
 
-Q_DECLARE_OPERATORS_FOR_FLAGS(PlaylistItem::Options);
+Q_DECLARE_METATYPE(PlaylistItemPtr)
+Q_DECLARE_METATYPE(QList<PlaylistItemPtr>)
+Q_DECLARE_OPERATORS_FOR_FLAGS(PlaylistItem::Options)
 
 #endif // PLAYLISTITEM_H
