@@ -29,8 +29,10 @@
 class LibraryBackend;
 class Playlist;
 class PlaylistBackend;
+class PlaylistContainer;
 class PlaylistParser;
 class PlaylistSequence;
+class SpecialPlaylistType;
 class TaskManager;
 
 class QModelIndex;
@@ -54,21 +56,29 @@ public:
   virtual QList<Playlist*> GetAllPlaylists() const = 0;
   // Grays out and reloads all deleted songs in all playlists.
   virtual void InvalidateDeletedSongs() = 0;
+  // Removes all deleted songs from all playlists.
+  virtual void RemoveDeletedSongs() = 0;
 
-  virtual const QItemSelection& selection(int id) const = 0;
-  virtual const QItemSelection& current_selection() const = 0;
-  virtual const QItemSelection& active_selection() const = 0;
+  virtual QItemSelection selection(int id) const = 0;
+  virtual QItemSelection current_selection() const = 0;
+  virtual QItemSelection active_selection() const = 0;
 
-  virtual QString name(int index) const = 0;
+  virtual QString GetPlaylistName(int index) const = 0;
 
   virtual TaskManager* task_manager() const = 0;
   virtual LibraryBackend* library_backend() const = 0;
   virtual PlaylistBackend* playlist_backend() const = 0;
   virtual PlaylistSequence* sequence() const = 0;
   virtual PlaylistParser* parser() const = 0;
+  virtual PlaylistContainer* playlist_container() const = 0;
+
+  virtual void RegisterSpecialPlaylistType(SpecialPlaylistType* type) = 0;
+  virtual void UnregisterSpecialPlaylistType(SpecialPlaylistType* type) = 0;
+  virtual SpecialPlaylistType* GetPlaylistType(const QString& type) const = 0;
 
 public slots:
-  virtual void New(const QString& name, const SongList& songs = SongList()) = 0;
+  virtual void New(const QString& name, const SongList& songs = SongList(),
+                   const QString& special_type = QString()) = 0;
   virtual void Load(const QString& filename) = 0;
   virtual void Save(int id, const QString& filename) = 0;
   virtual void Rename(int id, const QString& new_name) = 0;
@@ -137,24 +147,36 @@ public:
   QList<Playlist*> GetAllPlaylists() const;
   // Grays out and reloads all deleted songs in all playlists.
   void InvalidateDeletedSongs();
+  // Removes all deleted songs from all playlists.
+  void RemoveDeletedSongs();
 
-  const QItemSelection& selection(int id) const;
-  const QItemSelection& current_selection() const { return selection(current_id()); }
-  const QItemSelection& active_selection() const { return selection(active_id()); }
+  // Returns a pretty automatic name for playlist created from the given list of
+  // songs.
+  static QString GetNameForNewPlaylist(const SongList& songs);
 
-  QString name(int index) const { return playlists_[index].name; }
+  QItemSelection selection(int id) const;
+  QItemSelection current_selection() const { return selection(current_id()); }
+  QItemSelection active_selection() const { return selection(active_id()); }
+
+  QString GetPlaylistName(int index) const { return playlists_[index].name; }
 
   void Init(LibraryBackend* library_backend, PlaylistBackend* playlist_backend,
-            PlaylistSequence* sequence);
+            PlaylistSequence* sequence, PlaylistContainer* playlist_container);
 
   TaskManager* task_manager() const { return task_manager_; }
   LibraryBackend* library_backend() const { return library_backend_; }
   PlaylistBackend* playlist_backend() const { return playlist_backend_; }
   PlaylistSequence* sequence() const { return sequence_; }
   PlaylistParser* parser() const { return parser_; }
+  PlaylistContainer* playlist_container() const { return playlist_container_; }
+
+  void RegisterSpecialPlaylistType(SpecialPlaylistType* type);
+  void UnregisterSpecialPlaylistType(SpecialPlaylistType* type);
+  SpecialPlaylistType* GetPlaylistType(const QString& type) const;
 
 public slots:
-  void New(const QString& name, const SongList& songs = SongList());
+  void New(const QString& name, const SongList& songs = SongList(),
+           const QString& special_type = QString());
   void Load(const QString& filename);
   void Save(int id, const QString& filename);
   void Rename(int id, const QString& new_name);
@@ -190,7 +212,7 @@ private slots:
   void LoadFinished(bool success);
 
 private:
-  Playlist* AddPlaylist(int id, const QString& name);
+  Playlist* AddPlaylist(int id, const QString& name, const QString& special_type);
 
 private:
   struct Data {
@@ -205,10 +227,14 @@ private:
   LibraryBackend* library_backend_;
   PlaylistSequence* sequence_;
   PlaylistParser* parser_;
+  PlaylistContainer* playlist_container_;
 
   // key = id
   QMap<int, Data> playlists_;
 
+  QMap<QString, SpecialPlaylistType*> special_playlist_types_;
+  SpecialPlaylistType* default_playlist_type_;
+
   int current_;
   int active_;
 };
