@@ -42,14 +42,21 @@ namespace PlaylistUndoCommands {
 
   class InsertItems : public Base {
    public:
-    InsertItems(Playlist* playlist, const PlaylistItemList& items, int pos);
+    InsertItems(Playlist* playlist, const PlaylistItemList& items, int pos,
+                bool enqueue = false);
 
     void undo();
     void redo();
+    // When load is async, items have already been pushed, so we need to update them.
+    // This function try to find the equivalent item, and replace it with the
+    // new (completely loaded) one.
+    // return true if the was found (and updated), false otherwise
+    bool UpdateItem(const PlaylistItemPtr& updated_item);
 
    private:
     PlaylistItemList items_;
     int pos_;
+    bool enqueue_;
   };
 
   class RemoveItems : public Base {
