@@ -16,11 +16,11 @@
 */
 
 #include "dynamicplaylistcontrols.h"
-#include "playlistview.h"
 #include "playlist.h"
-#include "playlistheader.h"
 #include "playlistdelegates.h"
-#include "playlist.h"
+#include "playlistheader.h"
+#include "playlistview.h"
+#include "core/logging.h"
 
 #include <QCleanlooksStyle>
 #include <QPainter>
@@ -50,6 +50,22 @@ PlaylistProxyStyle::PlaylistProxyStyle(QStyle* base)
 void PlaylistProxyStyle::drawControl(
     ControlElement element, const QStyleOption* option, QPainter* painter,
     const QWidget* widget) const {
+  if (element == CE_Header) {
+    const QStyleOptionHeader* header_option = qstyleoption_cast<const QStyleOptionHeader*>(option);
+    const QRect& rect = header_option->rect;
+    const QString& text = header_option->text;
+    const QFontMetrics& font_metrics = header_option->fontMetrics;
+    
+    // spaces added to make transition less abrupt
+    if (rect.width() < font_metrics.width(text + "  ")) {
+      const Playlist::Column column = static_cast<Playlist::Column>(header_option->section);
+      QStyleOptionHeader new_option(*header_option);
+      new_option.text = Playlist::abbreviated_column_name(column);
+      QProxyStyle::drawControl(element, &new_option, painter, widget);
+      return;
+    }
+  }
+
   if (element == CE_ItemViewItem)
     cleanlooks_->drawControl(element, option, painter, widget);
   else
@@ -69,8 +85,9 @@ PlaylistView::PlaylistView(QWidget *parent)
   : QTreeView(parent),
     style_(new PlaylistProxyStyle(style())),
     playlist_(NULL),
-    header_(new PlaylistHeader(Qt::Horizontal, this)),
+    header_(new PlaylistHeader(Qt::Horizontal, this, this)),
     setting_initial_header_layout_(false),
+    upgrading_from_qheaderview_(false),
     read_only_settings_(true),
     glow_enabled_(true),
     currently_glowing_(false),
@@ -94,12 +111,12 @@ PlaylistView::PlaylistView(QWidget *parent)
 
   connect(header_, SIGNAL(sectionResized(int,int,int)), SLOT(SaveGeometry()));
   connect(header_, SIGNAL(sectionMoved(int,int,int)), SLOT(SaveGeometry()));
+  connect(header_, SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)), SLOT(SaveGeometry()));
   connect(header_, SIGNAL(SectionVisibilityChanged(int,bool)), SLOT(SaveGeometry()));
   connect(header_, SIGNAL(sectionResized(int,int,int)), SLOT(InvalidateCachedCurrentPixmap()));
   connect(header_, SIGNAL(sectionMoved(int,int,int)), SLOT(InvalidateCachedCurrentPixmap()));
   connect(header_, SIGNAL(SectionVisibilityChanged(int,bool)), SLOT(InvalidateCachedCurrentPixmap()));
   connect(header_, SIGNAL(StretchEnabledChanged(bool)), SLOT(SaveSettings()));
-  connect(header_, SIGNAL(ColumnAlignmentChanged()), SLOT(SaveSettings()));
   connect(header_, SIGNAL(StretchEnabledChanged(bool)), SLOT(StretchChanged(bool)));
   connect(header_, SIGNAL(MouseEntered()), SLOT(RatingHoverOut()));
 
@@ -149,7 +166,7 @@ void PlaylistView::SetItemDelegates(LibraryBackend* backend) {
   setItemDelegateForColumn(Playlist::Column_LastPlayed, new LastPlayedItemDelegate(this));
 }
 
-void PlaylistView::SetPlaylist(Playlist *playlist) {
+void PlaylistView::SetPlaylist(Playlist* playlist) {
   if (playlist_) {
     disconnect(playlist_, SIGNAL(CurrentSongChanged(Song)),
                this, SLOT(MaybeAutoscroll()));
@@ -157,6 +174,8 @@ void PlaylistView::SetPlaylist(Playlist *playlist) {
                this, SLOT(DynamicModeChanged(bool)));
     disconnect(playlist_, SIGNAL(destroyed()), this, SLOT(PlaylistDestroyed()));
 
+    disconnect(dynamic_controls_, SIGNAL(Expand()),
+               playlist_, SLOT(ExpandDynamicPlaylist()));
     disconnect(dynamic_controls_, SIGNAL(Repopulate()),
                playlist_, SLOT(RepopulateDynamicPlaylist()));
     disconnect(dynamic_controls_, SIGNAL(TurnOff()),
@@ -175,6 +194,7 @@ void PlaylistView::SetPlaylist(Playlist *playlist) {
   connect(playlist_, SIGNAL(CurrentSongChanged(Song)), SLOT(MaybeAutoscroll()));
   connect(playlist_, SIGNAL(DynamicModeChanged(bool)), SLOT(DynamicModeChanged(bool)));
   connect(playlist_, SIGNAL(destroyed()), SLOT(PlaylistDestroyed()));
+  connect(dynamic_controls_, SIGNAL(Expand()), playlist_, SLOT(ExpandDynamicPlaylist()));
   connect(dynamic_controls_, SIGNAL(Repopulate()), playlist_, SLOT(RepopulateDynamicPlaylist()));
   connect(dynamic_controls_, SIGNAL(TurnOff()), playlist_, SLOT(TurnOffDynamicPlaylist()));
 }
@@ -199,27 +219,34 @@ void PlaylistView::LoadGeometry() {
   QSettings settings;
   settings.beginGroup(Playlist::kSettingsGroup);
 
-  if (!header_->restoreState(settings.value("state").toByteArray())) {
-    header_->HideSection(Playlist::Column_Disc);
-    header_->HideSection(Playlist::Column_Year);
-    header_->HideSection(Playlist::Column_Genre);
-    header_->HideSection(Playlist::Column_BPM);
-    header_->HideSection(Playlist::Column_Bitrate);
-    header_->HideSection(Playlist::Column_Samplerate);
-    header_->HideSection(Playlist::Column_Filename);
-    header_->HideSection(Playlist::Column_Filesize);
-    header_->HideSection(Playlist::Column_Filetype);
-    header_->HideSection(Playlist::Column_DateCreated);
-    header_->HideSection(Playlist::Column_DateModified);
-    header_->HideSection(Playlist::Column_AlbumArtist);
-    header_->HideSection(Playlist::Column_Composer);
-    header_->HideSection(Playlist::Column_Rating);
-    header_->HideSection(Playlist::Column_PlayCount);
-    header_->HideSection(Playlist::Column_SkipCount);
-    header_->HideSection(Playlist::Column_LastPlayed);
+  QByteArray state(settings.value("state").toByteArray());
+  if (!header_->RestoreState(state)) {
+    // Maybe we're upgrading from a version that persisted the state with
+    // QHeaderView.
+    if (!header_->restoreState(state)) {
+      header_->HideSection(Playlist::Column_Disc);
+      header_->HideSection(Playlist::Column_Year);
+      header_->HideSection(Playlist::Column_Genre);
+      header_->HideSection(Playlist::Column_BPM);
+      header_->HideSection(Playlist::Column_Bitrate);
+      header_->HideSection(Playlist::Column_Samplerate);
+      header_->HideSection(Playlist::Column_Filename);
+      header_->HideSection(Playlist::Column_Filesize);
+      header_->HideSection(Playlist::Column_Filetype);
+      header_->HideSection(Playlist::Column_DateCreated);
+      header_->HideSection(Playlist::Column_DateModified);
+      header_->HideSection(Playlist::Column_AlbumArtist);
+      header_->HideSection(Playlist::Column_Composer);
+      header_->HideSection(Playlist::Column_Rating);
+      header_->HideSection(Playlist::Column_PlayCount);
+      header_->HideSection(Playlist::Column_SkipCount);
+      header_->HideSection(Playlist::Column_LastPlayed);
 
-    header_->moveSection(header_->visualIndex(Playlist::Column_Track), 0);
-    setting_initial_header_layout_ = true;
+      header_->moveSection(header_->visualIndex(Playlist::Column_Track), 0);
+      setting_initial_header_layout_ = true;
+    } else {
+      upgrading_from_qheaderview_ = true;
+    }
   }
 
   // New columns that we add are visible by default if the user has upgraded
@@ -257,7 +284,7 @@ void PlaylistView::SaveGeometry() {
 
   QSettings settings;
   settings.beginGroup(Playlist::kSettingsGroup);
-  settings.setValue("state", header_->saveState());
+  settings.setValue("state", header_->SaveState());
   settings.setValue("state_version", kStateVersion);
 }
 
@@ -437,6 +464,11 @@ void PlaylistView::keyPressEvent(QKeyEvent* event) {
   } else if (event->matches(QKeySequence::Delete)) {
     RemoveSelected();
     event->accept();
+#ifdef Q_OS_DARWIN
+  } else if (event->key() == Qt::Key_Backspace) {
+    RemoveSelected();
+    event->accept();
+#endif
   } else if (event->key() == Qt::Key_Enter ||
              event->key() == Qt::Key_Return) {
     if (currentIndex().isValid())
@@ -470,22 +502,43 @@ void PlaylistView::contextMenuEvent(QContextMenuEvent* e) {
 }
 
 void PlaylistView::RemoveSelected() {
+  int rows_removed = 0;
   QItemSelection selection(selectionModel()->selection());
 
+  if (selection.isEmpty()) {
+    return;
+  }
+
   // Sort the selection so we remove the items at the *bottom* first, ensuring
   // we don't have to mess around with changing row numbers
   qSort(selection.begin(), selection.end(), CompareSelectionRanges);
 
+  // Store the last selected row, which is the first in the list
+  int last_row = selection.first().bottom();
+
   foreach (const QItemSelectionRange& range, selection) {
+    rows_removed += range.height();
     model()->removeRows(range.top(), range.height(), range.parent());
   }
 
-  // Select the new current item
-  if (currentIndex().isValid())
-    selectionModel()->select(
-        QItemSelection(currentIndex().sibling(currentIndex().row(), 0),
-                       currentIndex().sibling(currentIndex().row(), model()->columnCount()-1)),
-        QItemSelectionModel::Select);
+  int new_row = last_row-rows_removed+1;
+  // Index of the first column for the row to select
+  QModelIndex new_index = model()->index(new_row, 0);
+
+  // Select the new current item, we want always the item after the last selected
+  if (new_index.isValid()) {
+    // Update visual selection with the entire row
+    selectionModel()->select(QItemSelection(new_index, model()->index(new_row, model()->columnCount()-1)),
+                                            QItemSelectionModel::Select);
+    // Update keyboard selected row, if it's not the first row
+    if (new_row != 0)
+      keyPressEvent(new QKeyEvent(QEvent::KeyPress, Qt::Key_Down, Qt::NoModifier));
+  } else {
+    // We're removing the last item, select the new last row
+    selectionModel()->select(QItemSelection(model()->index(model()->rowCount()-1, 0),
+                             model()->index(model()->rowCount()-1, model()->columnCount()-1)),
+                             QItemSelectionModel::Select);
+  }
 }
 
 QList<int> PlaylistView::GetEditableColumns() {
@@ -819,21 +872,32 @@ void PlaylistView::ReloadSettings() {
   QSettings s;
   s.beginGroup(Playlist::kSettingsGroup);
   glow_enabled_ = s.value("glow_effect", true).toBool();
-  header_->SetStretchEnabled(s.value("stretch", true).toBool());
+  background_enabled_ = s.value("bg_enabled", true).toBool();
+
+  if (setting_initial_header_layout_ || upgrading_from_qheaderview_) {
+    header_->SetStretchEnabled(s.value("stretch", true).toBool());
+    upgrading_from_qheaderview_ = false;
+  }
 
   if (currently_glowing_ && glow_enabled_ && isVisible())
     StartGlowing();
   if (!glow_enabled_)
     StopGlowing();
 
+  setProperty("background_enabled", background_enabled_);
+  
   if (setting_initial_header_layout_) {
     header_->SetColumnWidth(Playlist::Column_Length, 0.06);
     header_->SetColumnWidth(Playlist::Column_Track, 0.05);
     setting_initial_header_layout_ = false;
   }
 
-  ColumnAlignmentMap column_alignments = s.value("column_alignments").value<ColumnAlignmentMap>();
-  if (!column_alignments.isEmpty()) playlist_->set_column_alignments(column_alignments);
+  column_alignment_ = s.value("column_alignments").value<ColumnAlignmentMap>();
+  if (column_alignment_.isEmpty()) {
+    column_alignment_ = DefaultColumnAlignment();
+  }
+
+  emit ColumnAlignmentChanged(column_alignment_);
 }
 
 void PlaylistView::SaveSettings() {
@@ -843,12 +907,13 @@ void PlaylistView::SaveSettings() {
   QSettings s;
   s.beginGroup(Playlist::kSettingsGroup);
   s.setValue("glow_effect", glow_enabled_);
-  s.setValue("stretch", header_->is_stretch_enabled());
-  s.setValue("column_alignments", QVariant::fromValue(playlist_->column_alignments()));
+  s.setValue("column_alignments", QVariant::fromValue(column_alignment_));
+  s.setValue("bg_enabled", background_enabled_);
 }
 
 void PlaylistView::StretchChanged(bool stretch) {
   setHorizontalScrollBarPolicy(stretch ? Qt::ScrollBarAlwaysOff : Qt::ScrollBarAsNeeded);
+  SaveGeometry();
 }
 
 void PlaylistView::DynamicModeChanged(bool dynamic) {
@@ -881,3 +946,46 @@ bool PlaylistView::eventFilter(QObject* object, QEvent* event) {
   }
   return QObject::eventFilter(object, event);
 }
+
+void PlaylistView::rowsInserted(const QModelIndex& parent, int start, int end) {
+  const bool at_end = end == model()->rowCount(parent) - 1;
+
+  QTreeView::rowsInserted(parent, start, end);
+
+  if (at_end) {
+    // If the rows were inserted at the end of the playlist then let's scroll
+    // the view so the user can see.
+    scrollTo(model()->index(start, 0, parent), QAbstractItemView::PositionAtTop);
+  }
+}
+
+ColumnAlignmentMap PlaylistView::DefaultColumnAlignment() {
+  ColumnAlignmentMap ret;
+
+  ret[Playlist::Column_Length] =
+  ret[Playlist::Column_Track] =
+  ret[Playlist::Column_Disc] =
+  ret[Playlist::Column_Year] =
+  ret[Playlist::Column_BPM] =
+  ret[Playlist::Column_Bitrate] =
+  ret[Playlist::Column_Samplerate] =
+  ret[Playlist::Column_Filesize] =
+  ret[Playlist::Column_PlayCount] =
+  ret[Playlist::Column_SkipCount] = (Qt::AlignRight | Qt::AlignVCenter);
+  ret[Playlist::Column_Score]     = (Qt::AlignCenter);
+
+  return ret;
+}
+
+void PlaylistView::SetColumnAlignment(int section, Qt::Alignment alignment) {
+  if (section < 0)
+    return;
+
+  column_alignment_[section] = alignment;
+  emit ColumnAlignmentChanged(column_alignment_);
+  SaveSettings();
+}
+
+Qt::Alignment PlaylistView::column_alignment(int section) const {
+  return column_alignment_.value(section, Qt::AlignLeft | Qt::AlignVCenter);
+}
