@@ -24,13 +24,15 @@
 
 #include "gtest/gtest_prod.h"
 
-#include "core/albumcoverloader.h"
 #include "core/backgroundthread.h"
 #include "core/song.h"
+#include "covers/albumcoverloader.h"
+#include "covers/coversearchstatistics.h"
 
 class AlbumCoverChoiceController;
 class AlbumCoverFetcher;
 class AlbumCoverSearcher;
+class CoverProviders;
 class LibraryBackend;
 class LineEditInterface;
 class SongMimeData;
@@ -44,7 +46,9 @@ class QProgressBar;
 class AlbumCoverManager : public QMainWindow {
   Q_OBJECT
  public:
-  AlbumCoverManager(LibraryBackend* backend, QWidget *parent = 0,
+  AlbumCoverManager(LibraryBackend* backend,
+                    CoverProviders* cover_providers,
+                    QWidget *parent = 0,
                     QNetworkAccessManager* network = 0);
   ~AlbumCoverManager();
 
@@ -56,6 +60,8 @@ class AlbumCoverManager : public QMainWindow {
   void Reset();
   void Init();
 
+  void ResetFetchCoversButton();
+
   SongList GetSongsInAlbum(const QModelIndex& index) const;
   SongList GetSongsInAlbums(const QModelIndexList& indexes) const;
   SongMimeData* GetMimeDataForAlbums(const QModelIndexList& indexes) const;
@@ -66,7 +72,6 @@ class AlbumCoverManager : public QMainWindow {
  protected:
   void showEvent(QShowEvent *);
   void closeEvent(QCloseEvent *);
-  bool event(QEvent *);
 
   // For the album view context menu events
   bool eventFilter(QObject *obj, QEvent *event);
@@ -77,7 +82,8 @@ class AlbumCoverManager : public QMainWindow {
   void CoverImageLoaded(quint64 id, const QImage& image);
   void UpdateFilter();
   void FetchAlbumCovers();
-  void AlbumCoverFetched(quint64 id, const QImage& image);
+  void AlbumCoverFetched(quint64 id, const QImage& image,
+                         const CoverSearchStatistics& statistics);
 
   // On the context menu
   void FetchSingleCover();
@@ -108,7 +114,7 @@ class AlbumCoverManager : public QMainWindow {
     Role_AlbumName,
     Role_PathAutomatic,
     Role_PathManual,
-    Role_FirstFilename,
+    Role_FirstUrl,
   };
 
   enum HideCovers {
@@ -137,10 +143,9 @@ class AlbumCoverManager : public QMainWindow {
   void SaveAndSetCover(QListWidgetItem* item, const QImage& image);
 
  private:
-  bool constructed_;
-
   Ui_CoverManager* ui_;
 
+  CoverProviders* cover_providers_;
   AlbumCoverChoiceController* album_cover_choice_controller_;
   LibraryBackend* backend_;
 
@@ -153,20 +158,20 @@ class AlbumCoverManager : public QMainWindow {
 
   AlbumCoverFetcher* cover_fetcher_;
   QMap<quint64, QListWidgetItem*> cover_fetching_tasks_;
+  CoverSearchStatistics fetch_statistics_;
 
   AlbumCoverSearcher* cover_searcher_;
 
   QIcon artist_icon_;
   QIcon all_artists_icon_;
   QIcon no_cover_icon_;
+  QImage no_cover_image_;
 
   QMenu* context_menu_;
   QList<QListWidgetItem*> context_menu_items_;
 
   QProgressBar* progress_bar_;
   int jobs_;
-  int got_covers_;
-  int missing_covers_;
 
   LineEditInterface* filter_;
 
