@@ -19,11 +19,12 @@
 #include "edittagdialog.h"
 #include "trackselectiondialog.h"
 #include "ui_edittagdialog.h"
-#include "core/albumcoverloader.h"
+#include "core/logging.h"
 #include "core/utilities.h"
+#include "covers/albumcoverloader.h"
+#include "covers/coverproviders.h"
 #include "library/library.h"
 #include "library/librarybackend.h"
-#include "musicbrainz/fingerprinter.h"
 #include "playlist/playlistdelegates.h"
 #include "ui/albumcoverchoicecontroller.h"
 #include "ui/coverfromurldialog.h"
@@ -41,10 +42,12 @@
 #include <QtDebug>
 
 const char* EditTagDialog::kHintText = QT_TR_NOOP("(different across multiple songs)");
+const char* EditTagDialog::kSettingsGroup = "EditTagDialog";
 
-EditTagDialog::EditTagDialog(QWidget* parent)
+EditTagDialog::EditTagDialog(CoverProviders* cover_providers, QWidget* parent)
   : QDialog(parent),
     ui_(new Ui_EditTagDialog),
+    cover_providers_(cover_providers),
     album_cover_choice_controller_(new AlbumCoverChoiceController(this)),
     backend_(NULL),
     loading_(false),
@@ -68,9 +71,11 @@ EditTagDialog::EditTagDialog(QWidget* parent)
           SLOT(FetchTagSongChosen(Song, Song)));
   connect(results_dialog_, SIGNAL(finished(int)), tag_fetcher_, SLOT(Cancel()));
 
+  album_cover_choice_controller_->SetCoverProviders(cover_providers);
+
   ui_->setupUi(this);
   ui_->splitter->setSizes(QList<int>() << 200 << width() - 200);
-  ui_->loading_container->hide();
+  ui_->loading_label->hide();
 
   // An editable field is one that has a label as a buddy.  The label is
   // important because it gets turned bold when the field is changed.
@@ -173,12 +178,12 @@ bool EditTagDialog::SetLoading(const QString& message) {
     return false;
   loading_ = loading;
 
-  ui_->loading_container->setVisible(loading);
   ui_->button_box->setEnabled(!loading);
   ui_->tab_widget->setEnabled(!loading);
   ui_->song_list->setEnabled(!loading);
   ui_->fetch_tag->setEnabled(!loading);
-  ui_->loading_label->setText(message);
+  ui_->loading_label->setVisible(loading);
+  ui_->loading_label->set_text(message);
   return true;
 }
 
@@ -189,7 +194,7 @@ QList<EditTagDialog::Data> EditTagDialog::LoadData(const SongList& songs) const
     if (song.IsEditable()) {
       // Try reloading the tags from file
       Song copy(song);
-      copy.InitFromFile(copy.filename(), copy.directory_id());
+      copy.InitFromFile(copy.url().toLocalFile(), copy.directory_id());
 
       if (copy.is_valid())
         ret << Data(copy);
@@ -242,8 +247,6 @@ void EditTagDialog::SetSongsFinished() {
   ui_->song_list->setVisible(multiple);
   previous_button_->setEnabled(multiple);
   next_button_->setEnabled(multiple);
-
-  ui_->tab_widget->setCurrentWidget(multiple ? ui_->tags_tab : ui_->summary_tab);
 }
 
 void EditTagDialog::SetTagCompleter(LibraryBackend* backend) {
@@ -268,7 +271,7 @@ QVariant EditTagDialog::Data::value(const Song& song, const QString& id) {
   if (id == "track")       return song.track();
   if (id == "disc")        return song.disc();
   if (id == "year")        return song.year();
-  qDebug() << "Unknown ID" << id;
+  qLog(Warning) << "Unknown ID" << id;
   return QVariant();
 }
 
@@ -408,11 +411,6 @@ void EditTagDialog::UpdateSummaryTab(const Song& song) {
 
   ui_->summary->setText(summary);
 
-#ifndef HAVE_LIBLASTFM
-  album_cover_choice_controller_->cover_from_file_action()->setEnabled(false);
-  album_cover_choice_controller_->search_for_cover_action()->setEnabled(false);
-#endif
-
   album_cover_choice_controller_->unset_cover_action()->setEnabled(art_is_set);
   album_cover_choice_controller_->show_cover_action()->setEnabled(art_is_set);
   ui_->summary_art_button->setEnabled(song.id() != -1);
@@ -427,7 +425,13 @@ void EditTagDialog::UpdateSummaryTab(const Song& song) {
         QLocale::system().dateTimeFormat(QLocale::LongFormat)));
   ui_->filesize->setText(Utilities::PrettySize(song.filesize()));
   ui_->filetype->setText(song.TextForFiletype());
-  ui_->filename->setText(QDir::toNativeSeparators(song.filename()));
+
+  if (song.url().scheme() == "file")
+    ui_->filename->setText(QDir::toNativeSeparators(song.url().toLocalFile()));
+  else
+    ui_->filename->setText(song.url().toString());
+
+  album_cover_choice_controller_->search_for_cover_action()->setEnabled(cover_providers_->HasAnyProviders());
 }
 
 void EditTagDialog::UpdateStatisticsTab(const Song& song) {
@@ -603,7 +607,7 @@ void EditTagDialog::SaveData(const QList<Data>& data) {
       continue;
 
     if (!ref.current_.Save()) {
-      emit Error(tr("An error occurred writing metadata to '%1'").arg(ref.current_.filename()));
+      emit Error(tr("An error occurred writing metadata to '%1'").arg(ref.current_.url().toLocalFile()));
     }
   }
 }
@@ -671,9 +675,23 @@ void EditTagDialog::showEvent(QShowEvent* e) {
   // Set the dialog's height to the smallest possible
   resize(width(), sizeHint().height());
 
+  // Restore the tab that was current last time.
+  QSettings s;
+  s.beginGroup(kSettingsGroup);
+  ui_->tab_widget->setCurrentIndex(s.value("current_tab").toInt());
+
   QDialog::showEvent(e);
 }
 
+void EditTagDialog::hideEvent(QHideEvent* e) {
+  // Save the current tab
+  QSettings s;
+  s.beginGroup(kSettingsGroup);
+  s.setValue("current_tab", ui_->tab_widget->currentIndex());
+
+  QDialog::hideEvent(e);
+}
+
 void EditTagDialog::SongRated(float rating) {
   const QModelIndexList sel = ui_->song_list->selectionModel()->selectedIndexes();
   if (sel.isEmpty())
@@ -709,11 +727,6 @@ void EditTagDialog::ResetPlayCounts() {
 }
 
 void EditTagDialog::FetchTag() {
-  if (!Fingerprinter::GstreamerHasOfa()) {
-    QMessageBox::warning(this, tr("Error"), tr("Your gstreamer installation is missing the 'ofa' plugin.  This is required for automatic tag fetching.  Try installing the 'gstreamer-plugins-bad' package."));
-    return;
-  }
-
   const QModelIndexList sel = ui_->song_list->selectionModel()->selectedIndexes();
 
   SongList songs;
@@ -737,12 +750,12 @@ void EditTagDialog::FetchTag() {
 }
 
 void EditTagDialog::FetchTagSongChosen(const Song& original_song, const Song& new_metadata) {
-  const QString filename = original_song.filename();
+  const QString filename = original_song.url().toLocalFile();
 
   // Find the song with this filename
   for (int i=0 ; i<data_.count() ; ++i) {
     Data* data = &data_[i];
-    if (data->original_.filename() != filename)
+    if (data->original_.url().toLocalFile() != filename)
       continue;
 
     // Is it currently being displayed in the UI?
