@@ -31,6 +31,7 @@
 
 class AlbumCoverChoiceController;
 class AlbumCoverLoader;
+class CoverProviders;
 class LibraryBackend;
 class Ui_EditTagDialog;
 
@@ -43,10 +44,11 @@ class EditTagDialog : public QDialog {
   Q_OBJECT
 
 public:
-  EditTagDialog(QWidget* parent = 0);
+  EditTagDialog(CoverProviders* cover_providers, QWidget* parent = 0);
   ~EditTagDialog();
 
   static const char* kHintText;
+  static const char* kSettingsGroup;
 
   void SetSongs(const SongList& songs, const PlaylistItemList& items = PlaylistItemList());
   void SetTagCompleter(LibraryBackend* backend);
@@ -61,6 +63,7 @@ signals:
 protected:
   bool eventFilter(QObject* o, QEvent* e);
   void showEvent(QShowEvent*);
+  void hideEvent(QHideEvent*);
 
 private slots:
   void SetSongsFinished();
@@ -134,6 +137,7 @@ private:
 private:
   Ui_EditTagDialog* ui_;
 
+  CoverProviders* cover_providers_;
   AlbumCoverChoiceController* album_cover_choice_controller_;
 
   LibraryBackend* backend_;
