@@ -16,7 +16,9 @@
 */
 
 #include "autoexpandingtreeview.h"
+#include "core/mimedata.h"
 
+#include <QMouseEvent>
 #include <QtDebug>
 
 const int AutoExpandingTreeView::kRowsToShow = 50;
@@ -86,4 +88,56 @@ void AutoExpandingTreeView::ItemClicked(const QModelIndex& index) {
 
 void AutoExpandingTreeView::ItemDoubleClicked(const QModelIndex& index) {
   ignore_next_click_ = true;
+
+  QMimeData* data = model()->mimeData(QModelIndexList() << index);
+  if (MimeData* mime_data = qobject_cast<MimeData*>(data)) {
+    mime_data->from_doubleclick_ = true;
+  }
+  emit AddToPlaylistSignal(data);
+}
+
+void AutoExpandingTreeView::mousePressEvent(QMouseEvent* event) {
+  if (event->modifiers() != Qt::NoModifier) {
+    ignore_next_click_ = true;
+  }
+
+  QTreeView::mousePressEvent(event);
+
+  //enqueue to playlist with middleClick
+  if (event->button() == Qt::MidButton) {
+    QMimeData* data = model()->mimeData(selectedIndexes());
+    if (MimeData* mime_data = qobject_cast<MimeData*>(data)) {
+      mime_data->enqueue_now_ = true;
+    }
+    emit AddToPlaylistSignal(data);
+  }
+}
+
+void AutoExpandingTreeView::keyPressEvent(QKeyEvent* e) {
+  switch (e->key()) {
+    case Qt::Key_Enter:
+    case Qt::Key_Return:
+      if (currentIndex().isValid())
+        emit doubleClicked(currentIndex());
+      e->accept();
+      break;
+
+    case Qt::Key_Backspace:
+    case Qt::Key_Escape:
+      emit FocusOnFilterSignal(e);
+      e->accept();
+      break;
+  }
+
+  QTreeView::keyPressEvent(e);
+}
+
+void AutoExpandingTreeView::UpAndFocus() {
+  setCurrentIndex(moveCursor(QAbstractItemView::MoveUp, Qt::NoModifier));
+  setFocus();
+}
+
+void AutoExpandingTreeView::DownAndFocus() {
+  setCurrentIndex(moveCursor(QAbstractItemView::MoveDown, Qt::NoModifier));
+  setFocus();
 }
