@@ -29,6 +29,7 @@
 
 #include "fancytabwidget.h"
 #include "stylehelper.h"
+#include "core/logging.h"
 
 #include <QDebug>
 
@@ -589,7 +590,7 @@ void FancyTabWidget::SetMode(Mode mode) {
   switch (mode) {
     case Mode_None:
     default:
-      qDebug() << "Unknown fancy tab mode" << mode;
+      qLog(Warning) << "Unknown fancy tab mode" << mode;
       // fallthrough
 
     case Mode_LargeSidebar: {
@@ -673,6 +674,7 @@ void FancyTabWidget::MakeTabBar(QTabBar::Shape shape, bool text, bool icons,
   bar->setShape(shape);
   bar->setDocumentMode(true);
   bar->setUsesScrollButtons(true);
+  bar->setElideMode(Qt::ElideRight);
 
   if (shape == QTabBar::RoundedWest) {
     bar->setIconSize(QSize(22, 22));
@@ -704,7 +706,10 @@ void FancyTabWidget::MakeTabBar(QTabBar::Shape shape, bool text, bool icons,
     else if (text)
       tab_id = bar->addTab(label);
 
-    bar->setTabToolTip(tab_id, item.tab_label_);
+    // Adds tooltips only in Tabs mode or IconOnlyTabs mode
+    // TODO in tab mode, show only if not elided, complicated since this doesn't inherit from QTabWidget
+    if (shape == QTabBar::RoundedNorth && ((!text && icons) || (text && !icons)))
+      bar->setTabToolTip(tab_id, item.tab_label_);
   }
 
   bar->setCurrentIndex(stack_->currentIndex());
