@@ -42,6 +42,8 @@ class FileView : public QWidget {
   FileView(QWidget* parent = 0);
   ~FileView();
 
+  static const char* kFileFilter;
+
   void SetPath(const QString& path);
   void SetTaskManager(TaskManager* task_manager);
 
@@ -102,6 +104,8 @@ class FileView : public QWidget {
   boost::shared_ptr<MusicStorage> storage_;
 
   QString lazy_set_path_;
+
+  QStringList filter_list_;
 };
 
 #endif // FILEVIEW_H
