@@ -17,8 +17,9 @@
 
 #include "fullscreenhypnotoad.h"
 #include "nowplayingwidget.h"
-#include "core/albumcoverloader.h"
-#include "core/kittenloader.h"
+#include "covers/albumcoverloader.h"
+#include "covers/coverproviders.h"
+#include "covers/kittenloader.h"
 #include "library/librarybackend.h"
 #include "ui/albumcoverchoicecontroller.h"
 #include "ui/iconloader.h"
@@ -54,8 +55,9 @@ const int NowPlayingWidget::kBottomOffset = 0;
 const int NowPlayingWidget::kTopBorder = 4;
 
 
-NowPlayingWidget::NowPlayingWidget(QWidget *parent)
+NowPlayingWidget::NowPlayingWidget(QWidget* parent)
   : QWidget(parent),
+    cover_providers_(NULL),
     album_cover_choice_controller_(new AlbumCoverChoiceController(this)),
     cover_loader_(new BackgroundThreadImplementation<AlbumCoverLoader, AlbumCoverLoader>(this)),
     kitten_loader_(NULL),
@@ -133,6 +135,11 @@ NowPlayingWidget::NowPlayingWidget(QWidget *parent)
 NowPlayingWidget::~NowPlayingWidget() {
 }
 
+void NowPlayingWidget::SetCoverProviders(CoverProviders* cover_providers) {
+  cover_providers_ = cover_providers;
+  album_cover_choice_controller_->SetCoverProviders(cover_providers_);
+}
+
 void NowPlayingWidget::CreateModeAction(Mode mode, const QString &text, QActionGroup *group, QSignalMapper* mapper) {
   QAction* action = new QAction(text, group);
   action->setCheckable(true);
@@ -164,6 +171,10 @@ void NowPlayingWidget::CoverLoaderInitialised() {
 }
 
 void NowPlayingWidget::UpdateHeight(AlbumCoverLoader* loader) {
+  if (!loader) {
+    return;
+  }
+
   switch (mode_) {
   case SmallSongDetails:
     cover_height_ = small_ideal_height_;
@@ -341,7 +352,7 @@ void NowPlayingWidget::DrawContents(QPainter *p) {
 
 void NowPlayingWidget::FadePreviousTrack(qreal value) {
   previous_track_opacity_ = value;
-  if (qFuzzyCompare(previous_track_opacity_, 0.0)) {
+  if (qFuzzyCompare(previous_track_opacity_, qreal(0.0))) {
     previous_track_ = QPixmap();
   }
 
@@ -373,19 +384,14 @@ void NowPlayingWidget::resizeEvent(QResizeEvent* e) {
 void NowPlayingWidget::contextMenuEvent(QContextMenuEvent* e) {
   // initial 'enabled' values depending on the kitty mode
   album_cover_choice_controller_->cover_from_file_action()->setEnabled(!aww_);
-  album_cover_choice_controller_->cover_to_file_action()->setEnabled(aww_);
   album_cover_choice_controller_->cover_from_url_action()->setEnabled(!aww_);
-  album_cover_choice_controller_->search_for_cover_action()->setEnabled(!aww_);
+  album_cover_choice_controller_->search_for_cover_action()->setEnabled(
+        !aww_ && cover_providers_->HasAnyProviders());
   album_cover_choice_controller_->unset_cover_action()->setEnabled(!aww_);
   album_cover_choice_controller_->show_cover_action()->setEnabled(!aww_);
 
   // some special cases
   if (!aww_) {
-  #ifndef HAVE_LIBLASTFM
-    album_cover_choice_controller_->cover_from_file_action()->setEnabled(false);
-    album_cover_choice_controller_->search_for_cover_action()->setEnabled(false);
-  #endif
-
     const bool art_is_not_set = metadata_.has_manually_unset_cover()
         || (metadata_.art_automatic().isEmpty() && metadata_.art_manual().isEmpty());
 
