@@ -16,6 +16,7 @@
 */
 
 #include "config.h"
+#include "core/logging.h"
 #include "osd.h"
 #include "osdpretty.h"
 #include "ui/systemtrayicon.h"
@@ -38,6 +39,10 @@ OSD::OSD(SystemTrayIcon* tray_icon, QObject* parent)
     show_on_volume_change_(false),
     show_art_(true),
     show_on_play_mode_change_(true),
+    use_custom_text_(false),
+    custom_text1_(QString()),
+    custom_text2_(QString()),
+    preview_mode_(false),
     force_show_next_(false),
     ignore_next_stopped_(false),
     pretty_popup_(new OSDPretty(OSDPretty::Mode_Popup)),
@@ -70,30 +75,67 @@ void OSD::ReloadSettings() {
   show_on_volume_change_ = s.value("ShowOnVolumeChange", false).toBool();
   show_art_ = s.value("ShowArt", true).toBool();
   show_on_play_mode_change_ = s.value("ShowOnPlayModeChange", true).toBool();
+  use_custom_text_ = s.value(("CustomTextEnabled"), false).toBool();
+  custom_text1_ = s.value("CustomText1").toString();
+  custom_text2_ = s.value("CustomText2").toString();
 
   if (!SupportsNativeNotifications() && behaviour_ == Native)
     behaviour_ = Pretty;
   if (!SupportsTrayPopups() && behaviour_ == TrayPopup)
     behaviour_ = Disabled;
 
+  ReloadPrettyOSDSettings();
+}
+
+// Reload just Pretty OSD settings, not everything
+void OSD::ReloadPrettyOSDSettings() {
   pretty_popup_->set_popup_duration(timeout_msec_);
   pretty_popup_->ReloadSettings();
 }
 
 void OSD::SongChanged(const Song &song) {
-  // no cover art yet
-  tray_icon_->SetNowPlaying(song, NULL);
-  QString summary(song.PrettyTitle());
-  if (!song.artist().isEmpty())
-    summary = QString("%1 - %2").arg(song.artist(), summary);
+  // Don't change tray icon details if it's a preview
+  if (!preview_mode_) {
+    // no cover art yet
+    tray_icon_->SetNowPlaying(song, NULL);
+  }
 
   QStringList message_parts;
-  if (!song.album().isEmpty())
-    message_parts << song.album();
-  if (song.disc() > 0)
-    message_parts << tr("disc %1").arg(song.disc());
-  if (song.track() > 0)
-    message_parts << tr("track %1").arg(song.track());
+  QString summary;
+  if (!use_custom_text_) {
+    summary = song.PrettyTitle();
+    if (!song.artist().isEmpty())
+      summary = QString("%1 - %2").arg(song.artist(), summary);
+    if (!song.album().isEmpty())
+      message_parts << song.album();
+    if (song.disc() > 0)
+      message_parts << tr("disc %1").arg(song.disc());
+    if (song.track() > 0)
+      message_parts << tr("track %1").arg(song.track());
+  } else {
+    QRegExp variable_replacer("[%][a-z]+[%]");
+    summary = custom_text1_;
+    QString message(custom_text2_);
+
+    // Replace the first line
+    int pos = 0;
+    variable_replacer.indexIn(custom_text1_);
+    while ((pos = variable_replacer.indexIn(custom_text1_, pos)) != -1) {
+      QStringList captured = variable_replacer.capturedTexts();
+      summary.replace(captured[0], ReplaceVariable(captured[0], song));
+      pos += variable_replacer.matchedLength();
+    }
+
+    // Replace the second line
+    pos = 0;
+    variable_replacer.indexIn(custom_text2_);
+    while ((pos = variable_replacer.indexIn(custom_text2_, pos)) != -1) {
+      QStringList captured = variable_replacer.capturedTexts();
+      message.replace(captured[0], ReplaceVariable(captured[0], song));
+      pos += variable_replacer.matchedLength();
+    }
+    message_parts << message;
+  }
 
   WaitingForAlbumArt waiting;
   waiting.icon = "notification-audio-play";
@@ -111,7 +153,9 @@ void OSD::SongChanged(const Song &song) {
 }
 
 void OSD::CoverArtPathReady(const Song& song, const QString& image_path) {
-  tray_icon_->SetNowPlaying(song, image_path);
+  // Don't change tray icon details if it's a preview
+  if (!preview_mode_)
+    tray_icon_->SetNowPlaying(song, image_path);
 }
 
 void OSD::AlbumArtLoaded(quint64 id, const QImage& image) {
@@ -121,6 +165,12 @@ void OSD::AlbumArtLoaded(quint64 id, const QImage& image) {
 
 void OSD::AlbumArtLoaded(const WaitingForAlbumArt info, const QImage& image) {
   ShowMessage(info.summary, info.message, info.icon, image);
+
+  // Reload the saved settings if they were changed for preview
+  if (preview_mode_) {
+    ReloadSettings();
+    preview_mode_ = false;
+  }
 }
 
 void OSD::Paused() {
@@ -166,7 +216,10 @@ void OSD::ShowMessage(const QString& summary,
                       const QString& message,
                       const QString& icon,
                       const QImage& image) {
-  switch (behaviour_) {
+  if (pretty_popup_->toggle_mode()) {
+    pretty_popup_->ShowMessage(summary, message, image);
+  } else {
+    switch (behaviour_) {
     case Native:
       if (image.isNull()) {
         ShowMessageNative(summary, message, icon, QImage());
@@ -187,12 +240,12 @@ void OSD::ShowMessage(const QString& summary,
       force_show_next_ = false;
       // fallthrough
     case Pretty:
-      pretty_popup_->SetMessage(summary, message, image);
-      pretty_popup_->show();
+      pretty_popup_->ShowMessage(summary, message, image);
       break;
 
     default:
       break;
+    }
   }
 }
 
@@ -238,9 +291,10 @@ void OSD::ShuffleModeChanged(PlaylistSequence::ShuffleMode mode) {
   if (show_on_play_mode_change_) {
     QString current_mode = QString();
     switch (mode) {
-      case PlaylistSequence::Shuffle_Off:   current_mode = tr("Don't shuffle");   break;
-      case PlaylistSequence::Shuffle_All:   current_mode = tr("Shuffle all");   break;
-      case PlaylistSequence::Shuffle_Album: current_mode = tr("Shuffle by album"); break;
+      case PlaylistSequence::Shuffle_Off:         current_mode = tr("Don't shuffle");   break;
+      case PlaylistSequence::Shuffle_All:         current_mode = tr("Shuffle all");     break;
+      case PlaylistSequence::Shuffle_InsideAlbum: current_mode = tr("Shuffle tracks in this album"); break;
+      case PlaylistSequence::Shuffle_Albums:      current_mode = tr("Shuffle albums");  break;
     }
     ShowMessage(QCoreApplication::applicationName(), current_mode);
   }
@@ -258,3 +312,74 @@ void OSD::RepeatModeChanged(PlaylistSequence::RepeatMode mode) {
     ShowMessage(QCoreApplication::applicationName(), current_mode);
   }
 }
+
+QString OSD::ReplaceVariable(const QString& variable, const Song& song) {
+  QString return_value;
+  if (variable == "%artist%") {
+    return song.artist();
+  } else if (variable == "%album%") {
+    return song.album();
+  } else if (variable == "%title%") {
+    return song.PrettyTitle();
+  } else if (variable == "%albumartist%") {
+    return song.effective_albumartist();
+  } else if (variable == "%year%") {
+    return song.PrettyYear();
+  } else if (variable == "%composer%") {
+    return song.composer();
+  } else if (variable == "%length%") {
+    return song.PrettyLength();
+  } else if (variable == "%disc%") {
+    return return_value.setNum(song.disc());
+  } else if (variable == "%track%") {
+    return return_value.setNum(song.track());
+  } else if (variable == "%genre%") {
+    return song.genre();
+  } else if (variable == "%playcount%") {
+    return return_value.setNum(song.playcount());
+  } else if (variable == "%skipcount%") {
+    return return_value.setNum(song.skipcount());
+  } else if (variable == "%newline%") {
+    // We need different strings depending on notification type
+    switch (behaviour_) {
+      case Native:
+#ifdef Q_OS_DARWIN
+        return "\n";
+#endif
+#ifdef Q_OS_LINUX
+        return "<br/>";
+#endif
+#ifdef Q_OS_WIN32
+        // Other OS don't support native notifications
+        qLog(Debug) << "New line not supported by this notification type under Windows";
+        return "";
+#endif
+      case TrayPopup:
+        qLog(Debug) << "New line not supported by this notification type";
+        return "";
+      case Pretty:
+      default:
+        // When notifications are disabled, we force the PrettyOSD
+        return "<br/>";
+    }
+  }
+
+  //if the variable is not recognized, just return it
+  return variable;
+}
+
+void OSD::ShowPreview(const Behaviour type, const QString& line1, const QString& line2, const Song& song) {
+  behaviour_ = type;
+  custom_text1_ = line1;
+  custom_text2_ = line2;
+  if (!use_custom_text_)
+    use_custom_text_ = true;
+
+  // We want to reload the settings, but we can't do this here because the cover art loading is asynch
+  preview_mode_ = true;
+  SongChanged(song);
+}
+
+void OSD::SetPrettyOSDToggleMode(bool toggle) {
+  pretty_popup_->set_toggle_mode(toggle);
+}
