@@ -20,14 +20,29 @@
 
 #include <QSlider>
 
-// It's the slider inside the TrackSliderSlider
+class TrackSliderPopup;
 
+// It's the slider inside the TrackSliderSlider
 class TrackSliderSlider : public QSlider {
+  Q_OBJECT
+
 public:
   TrackSliderSlider(QWidget* parent = 0);
 
- protected:
+protected:
   void mousePressEvent(QMouseEvent* e);
+  void mouseReleaseEvent(QMouseEvent* e);
+  void mouseMoveEvent(QMouseEvent* e);
+  void enterEvent(QEvent*);
+  void leaveEvent(QEvent*);
+
+private slots:
+  void UpdateDeltaTime();
+
+private:
+  TrackSliderPopup* popup_;
+
+  int mouse_hover_seconds_;
 };
 
 #endif // TRACKSLIDERSLIDER_H
