@@ -24,6 +24,8 @@
 
 class MockLibraryBackend : public LibraryBackendInterface {
  public:
+  MOCK_CONST_METHOD0(songs_table, QString());
+
   // Get a list of directories in the library.  Emits DirectoriesDiscovered.
   MOCK_METHOD0(LoadDirectoriesAsync, void());
 
@@ -33,7 +35,7 @@ class MockLibraryBackend : public LibraryBackendInterface {
   MOCK_METHOD1(FindSongsInDirectory, SongList(int));
   MOCK_METHOD1(SubdirsInDirectory, SubdirectoryList(int));
   MOCK_METHOD0(GetAllDirectories, DirectoryList());
-  MOCK_METHOD2(ChangeDirPath, void(int, const QString&));
+  MOCK_METHOD3(ChangeDirPath, void(int, const QString&, const QString&));
 
   MOCK_METHOD1(GetAllArtists, QStringList(const QueryOptions&));
   MOCK_METHOD1(GetAllArtistsWithAlbums, QStringList(const QueryOptions&));
@@ -51,6 +53,9 @@ class MockLibraryBackend : public LibraryBackendInterface {
 
   MOCK_METHOD1(GetSongById, Song(int));
 
+  MOCK_METHOD1(GetSongsByUrl, SongList(const QUrl&));
+  MOCK_METHOD2(GetSongByUrl, Song(const QUrl&, qint64));
+
   MOCK_METHOD1(AddDirectory, void(const QString&));
   MOCK_METHOD1(RemoveDirectory, void(const Directory&));
 
